package com.taobao.movie.android.app.common.widget

import android.content.Context
import android.support.v7.widget.DefaultItemAnimator
import android.support.v7.widget.LinearLayoutManager
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.taobao.listitem.recycle.CustomRecyclerAdapter
import com.taobao.movie.android.app.common.recycleritem.CommunityCommentMainTabItem
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.CommunityCommentMainTabModel
import com.taobao.movie.appinfo.util.ResHelper

class CommunityCommentView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : InterceptFrameLayout(context, attrs, defStyleAttr) {

    private val commentRecycleView: MaxHeightRecycleView
    private val maskView: View
    private val adapter by lazy {
        CustomRecyclerAdapter(context)
    }

    init {
        val view = LayoutInflater.from(context).inflate(R.layout.community_comment_view, this, true)
        commentRecycleView = view.findViewById(R.id.rv_comment)
        maskView = view.findViewById(R.id.view_comment_mask)
        commentRecycleView.layoutManager = LinearLayoutManager(context)
        commentRecycleView.adapter = adapter
        commentRecycleView.itemAnimator = DefaultItemAnimator()
        commentRecycleView.itemAnimator?.addDuration = 120
        commentRecycleView.itemAnimator?.changeDuration = 120
        commentRecycleView.itemAnimator?.moveDuration = 120
        commentRecycleView.itemAnimator?.removeDuration = 120
        maskView.setBackgroundResource(R.drawable.community_comment_mask_view)
    }

    fun initComment(commentList: List<CommunityCommentMainTabModel>?) {
        if (commentList.isNullOrEmpty()) {
            adapter.removeItem(CommunityCommentMainTabItem::class.java,true)
            return
        }
        adapter.clearItems()
        for (it in commentList) {
            adapter.addItem(CommunityCommentMainTabItem(it))
        }
        adapter.notifyItemRangeChanged(0,commentList.size)
        maskView.visibility = if (commentList.size >= 3) View.VISIBLE else View.GONE
    }

    fun addComment(communityCommentMainTabModel: CommunityCommentMainTabModel?) {
        if(communityCommentMainTabModel == null) {
            return
        }
        adapter.addItem(CommunityCommentMainTabItem(communityCommentMainTabModel),true)
        commentRecycleView.smoothScrollToPosition(adapter.itemCount)
    }
}