package com.taobao.movie.android.app.common.widget

import android.content.Context
import android.graphics.drawable.AnimationDrawable
import android.support.constraint.ConstraintLayout
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import com.facebook.shimmer.ShimmerFrameLayout
import com.taobao.movie.android.commonui.widget.SimpleDraweeView
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.orange.ConfigUtil
import com.taobao.movie.android.integration.orange.OrangeConstants
import com.taobao.movie.android.integration.oscar.model.UserVO
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.userlevel.UserLevelType

/**
 * 只有用户头像与右下角的等级标签
 */
class UserIconView : FrameLayout {
    var userIconView: SimpleDraweeView
    private var useShimmer = false
    var userLevelIcon: ImageView
    var diamondIcon:ImageView
    var shimmerFrameLayout: ShimmerFrameLayout
    var userIconContainer: ConstraintLayout
    //黑钻animation
    private var defaultAnimationDrawable: AnimationDrawable? = null

    constructor(context: Context) : this(context,null)

    constructor(context: Context,attributeSet: AttributeSet?) : this(context,attributeSet,0)

    constructor(context: Context,attributeSet: AttributeSet?,defStyleAttr: Int) : super(context,attributeSet,defStyleAttr) {
        val view = LayoutInflater.from(context).inflate(R.layout.user_icon_layout,this,true)
        userIconView = view.findViewById(R.id.user_icon) as SimpleDraweeView
        shimmerFrameLayout = view.findViewById(R.id.user_level_diamond_shimmer) as ShimmerFrameLayout
        diamondIcon = view.findViewById(R.id.user_level_diamond_icon) as ImageView
        userLevelIcon = view.findViewById(R.id.user_level) as ImageView
        userIconContainer = view.findViewById(R.id.user_icon_container)

        useShimmer = ConfigUtil.getConfigCenterString(OrangeConstants.CONFIG_KEY_COMMENT_USER_ANIMATE, "false") == "true"
        if (useShimmer) {
            diamondIcon.setImageResource(R.drawable.home_comment_user_diamonds_bg)
            shimmerFrameLayout.useDefaults()
            shimmerFrameLayout.duration = 1000
            shimmerFrameLayout.baseAlpha = 0f
            shimmerFrameLayout.dropoff = 0.3f
            shimmerFrameLayout.tilt = 45f
            shimmerFrameLayout.repeatDelay = 2000
            shimmerFrameLayout.maskShape = ShimmerFrameLayout.MaskShape.LINEAR
        } else {
            defaultAnimationDrawable = context.resources.getDrawable(R.drawable.heizuan_show_small_gif) as AnimationDrawable
            shimmerFrameLayout.visibility = View.GONE
            findViewById<View>(R.id.user_level_diamond_icon_left).visibility = View.GONE
        }
    }

    fun initIcon(userVO: UserVO?,showUserLevel : Boolean = true) {
        if(userVO == null) {
            userLevelIcon.visibility = View.INVISIBLE
            diamondIcon.visibility = View.GONE
            return
        }
        userIconView.url = userVO.avatar?:""
        userLevelIcon.visibility = View.VISIBLE
        diamondIcon.visibility = View.GONE

        if (UserLevelType.LEVEL_V1.levelV.equals(userVO.userLevel, ignoreCase = true)) {
            userLevelIcon.setImageResource(R.drawable.member_level_1_72)
        } else if (UserLevelType.LEVEL_V2.levelV.equals(userVO.userLevel, ignoreCase = true)) {
            userLevelIcon.setImageResource(R.drawable.member_level_2_72)
        } else if (UserLevelType.LEVEL_V3.levelV.equals(userVO.userLevel, ignoreCase = true)) {
            userLevelIcon.setImageResource(R.drawable.member_level_3_72)
        } else if (UserLevelType.LEVEL_V4.levelV.equals(userVO.userLevel, ignoreCase = true)) {
            userLevelIcon.visibility = View.INVISIBLE
            diamondIcon.visibility = View.VISIBLE
        } else {
            userLevelIcon.visibility = View.INVISIBLE
            diamondIcon.visibility = View.GONE
        }

        if (useShimmer) {
            findViewById<View>(R.id.user_level_diamond_icon_left).visibility = diamondIcon.visibility
            shimmerFrameLayout.visibility = diamondIcon.visibility
        } else {
            findViewById<View>(R.id.user_level_diamond_icon_left).visibility = View.GONE
            shimmerFrameLayout.visibility = View.GONE
        }

        if (!showUserLevel) {
            userLevelIcon.visibility = View.INVISIBLE
            diamondIcon.visibility = View.GONE
        }
    }

    fun setIconSize(width: Int) {
        if (userIconContainer != null) {
            val lp: ViewGroup.LayoutParams = userIconContainer.getLayoutParams()
            lp.width = DisplayUtil.dp2pxInt(width.toFloat())
            lp.height = DisplayUtil.dp2pxInt(width.toFloat())
            userIconContainer.setLayoutParams(lp)
        }
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        if (diamondIcon.visibility == View.VISIBLE) {
            if (useShimmer) {
                shimmerFrameLayout.startShimmerAnimation()
            } else if (defaultAnimationDrawable != null) {
                diamondIcon.setImageDrawable(defaultAnimationDrawable)
                defaultAnimationDrawable?.start()
            }
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        if (diamondIcon?.visibility == View.VISIBLE) {
            if (useShimmer) {
                shimmerFrameLayout.stopShimmerAnimation()
            } else if (defaultAnimationDrawable != null) {
                defaultAnimationDrawable?.stop()
                diamondIcon.setImageDrawable(null)
            }
        }
    }
}