package com.taobao.movie.android.app.community

import android.app.Activity
import android.os.Bundle
import android.support.v7.widget.RecyclerView.ItemDecoration
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.community.recycleitem.CommunityFilmCalendarItem
import com.taobao.movie.android.app.community.recycleitem.CommunityFilmCalendarTopTipsItem
import com.taobao.movie.android.app.community.recycleitem.CommunityStickItem
import com.taobao.movie.android.app.oscar.biz.util.OscarBizUtil
import com.taobao.movie.android.app.presenter.community.ArtShowCalendarVO
import com.taobao.movie.android.app.presenter.community.CommunityFilmCalendarPresenter
import com.taobao.movie.android.app.presenter.filmdetail.FilmDoFavorPresenter
import com.taobao.movie.android.app.ui.article.view.TimeItem
import com.taobao.movie.android.app.ui.common.WantedTipUtil
import com.taobao.movie.android.app.vinterface.community.ICommunityFilmCalendarView
import com.taobao.movie.android.app.vinterface.filmdetail.IFilmDoFavorView
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.item.article.*
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.common.login.LoginHelper
import com.taobao.movie.android.common.login.checkSession
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.common.scheme.NavigatorUtil
import com.taobao.movie.android.commonui.component.lcee.ILceeView
import com.taobao.movie.android.commonui.component.lcee.LceeListFragment
import com.taobao.movie.android.commonui.component.lcee.MultiPresenters
import com.taobao.movie.android.commonui.recyclerview.LoadingItem
import com.taobao.movie.android.commonui.utils.SqmKeeper
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.commonui.widget.DividerItemDecoration
import com.taobao.movie.android.commonui.widget.FavoriteManager
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.common.service.LoginExtService
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.model.ShowResultIndexMo
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTConstants
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.statemanager.state.CoreState
import com.taobao.movie.statemanager.state.EmptyState
import com.taobao.movie.statemanager.state.SimpleProperty
import java.util.*

/**
 *  Created by 天逐 on 2019-12-13.
 *  function:电影日历
 */
class CommunityFilmCalendarFragment : LceeListFragment<MultiPresenters<ILceeView>>(), ICommunityFilmCalendarView, View.OnClickListener, RecyclerExtDataItem.OnItemEventListener<Any>, FavoriteManager.notifyFavorite, IFilmDoFavorView {

    private lateinit var loadingItem: LoadingItem
    private lateinit var mFakeTopView: TextView
    private lateinit var mCommunityFilmCalendarPresenter: CommunityFilmCalendarPresenter
    private lateinit var mFarvorPresenter: FilmDoFavorPresenter<IFilmDoFavorView>
    private var pageFrom = PAGE_FROM_CALENDAR_TITLE

    companion object {
        const val Page_ArtMvCalendar = "Page_ArtMvCalendar"
        const val Page_ArtMvCalendarSPM = "13835015"
        const val KEY_PAGE_FROM = "key_page_from"
        const val PAGE_FROM_CALENDAR_TITLE = "1" //1=电影日历模块，2=查看更多
        const val PAGE_FROM_CALENDAR_MORE = "2" //1=电影日历模块，2=查看更多
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        UTConstants.SPM_Map[Page_ArtMvCalendar] = Page_ArtMvCalendarSPM
        super.onCreate(savedInstanceState)
        getIntentData()
        utPageName = Page_ArtMvCalendar
        setUTPageEnable(true)
        FavoriteManager.getInstance().registerDefault(this)

    }

    fun getIntentData() {
        arguments?.let {
            pageFrom = it?.getString(KEY_PAGE_FROM)
        }
    }

    override fun createPresenter(): MultiPresenters<ILceeView> {
        mCommunityFilmCalendarPresenter = CommunityFilmCalendarPresenter()
        mFarvorPresenter = FilmDoFavorPresenter<IFilmDoFavorView>()
        return MultiPresenters(mCommunityFilmCalendarPresenter, mFarvorPresenter)
    }

    override fun initViewContent(layoutView: View, savedInstanceState: Bundle?) {
        super.initViewContent(layoutView, savedInstanceState)
        loadingItem = LoadingItem(getString(R.string.exception_item), listener)
        mCommunityFilmCalendarPresenter.loadData(true)
        showLoadingView(false)
        mFakeTopView = layoutView.findViewById(R.id.tv_film_calendar_sticky_item_date)
        mFakeTopView.visibility = View.GONE
        recyclerView.addOnScrollListener(FilmCalendarStickyScrollListener(mFakeTopView, adapter))
        recyclerView.addItemDecoration(decoration)
    }


    override fun getDecoration(): ItemDecoration {
        val itemDecoration: DividerItemDecoration = object : DividerItemDecoration(context) {
            override fun needDraw(type: Int): Boolean {
                return type == adapter.getItemType(CommunityFilmCalendarItem::class.java)
            }
        }
        itemDecoration.setDrawOver(true)
        itemDecoration.linePaddingLeft = DisplayUtil.dp2pxInt(20f)
        itemDecoration.linePaddingRight = 0
        return itemDecoration
    }

    private val listener = View.OnClickListener {
        updateLoadingItem(false)
        mCommunityFilmCalendarPresenter.loadData(false)
    }

    private fun updateLoadingItem(isError: Boolean) {
        if (loadingItem == null || adapter == null) {
            return
        }
        val notifyIndex = adapter.indexOfItem(loadingItem)
        if (notifyIndex < 0) {
            adapter.addItem(loadingItem, true)
        }
        if (isError) {
            loadingItem.showErrorStatus()
        } else {
            loadingItem.showLoadingStatus()
        }
    }

    private fun removeLoadingItem() {
        if (loadingItem != null && adapter != null) {
            val notifyIndex = adapter.indexOfItem(loadingItem)
            if (notifyIndex >= 0) {
                adapter.removeItem(loadingItem)
                adapter.notifyItemRemoved(notifyIndex)
            }
        }
    }

    override fun showEmpty() {
        refreshFinished()
        var hasContent = adapter.getCount(CommunityFilmCalendarItem::class.java) > 0
        if (adapter != null && !hasContent) {
            stateHelper.showState(SimpleProperty(EmptyState.STATE).setHint("暂无内容")
                    .setButtonVisiable(true)
                    .setImgResId(R.drawable.common_empty_view)
                    .setTopHolderHeight(48))
        }
    }

    override fun showError(isToast: Boolean, resultCode: Int, returnCode: Int, returnMsg: String?) {
        var hasContent = false
        hasContent = adapter.getCount(CommunityFilmCalendarItem::class.java) > 0
        if (hasContent) {
            updateLoadingItem(true)
        }
        super.showError(hasContent, resultCode, returnCode, returnMsg)

    }

    private var mLastShowTime: String = ""

    override fun showFilmList(artShowCalendarVO: ArtShowCalendarVO?, isRefresh: Boolean) {
        stateHelper.showState(CoreState.STATE)
        removeLoadingItem()
        if (artShowCalendarVO?.artShows.isNullOrEmpty()) {
            return
        }
        if (isRefresh) {
            adapter.clearItems()
            mLastShowTime = ""
            adapter.addItem(CommunityFilmCalendarTopTipsItem(artShowCalendarVO, this))

        }
        var currentIndex = adapter.getCount(CommunityFilmCalendarItem::class.java)
        artShowCalendarVO?.artShows?.forEach {
            it.parse()
            val showTime = OscarBizUtil.getFilmCalendarDateTitle(it)

            if (!TextUtils.equals(showTime, mLastShowTime)) {
                adapter.addItem(CommunityStickItem(it, this), true)
            }
            mLastShowTime = showTime
            currentIndex++
            adapter.addItem(CommunityFilmCalendarItem(it, this, currentIndex), true)
        }
        if (mCommunityFilmCalendarPresenter.hasMore()) {
            updateLoadingItem(false)
        }
    }

    override fun onClick(v: View?) {
    }

    override fun onEvent(eventId: Int, d: Any?, extObject: Any?): Boolean {
        when (eventId) {
            ItemEvent.EVENT_ID_COMMUNITY_FILM_CALENDAR_ITEM_CLICK -> {
                (d as? ShowMo)?.let { onItemClick(it) }
            }
            ItemEvent.EVENT_ID_COMMUNITY_FILM_CALENDAR_BUY_BTN_CLICK -> {
                (d as? ShowMo)?.let {
                    when (d.soldType) {
                        ShowMo.SOLD_TYPE_PRE, ShowMo.SOLD_TYPE_NORMAL -> {
                            //去买票
                            UTFacade.buttonClickWithoutPage("BuyButtonClick", "showId", it?.id)
                            gotoBuyFilm(it)
                        }
                        ShowMo.SOLD_TYPE_VOD -> {
                            //去预告片页面
                            gotoVideoList(it)
                        }
                        else -> {
                            //点赞按钮的点击
                            UTFacade.buttonClickWithoutPage("WantToSeeClick", "showId", it?.id, "status", it.getUserShowStatus()?.toString())
                            doFavor(it)
                        }
                    }
                }

            }
        }
        return true
    }

    //点击影片
    private fun onItemClick(showMo: ShowMo?) {
        showMo?.let {
            MovieNavigator.gotoAction(this, IntentConstants.Action.ACTION_SHOWDETAIL, Bundle()
                    .apply {
                        putSerializable(IntentConstants.KEY_SHOW_MO, it)
                        putString(UTConstants.sqm, SqmKeeper.SQM_COMMUNITY)

                    })
        }
    }

    private fun gotoBuyFilm(showMo: ShowMo) {
        showMo?.let {
            MovieNavigator.gotoAction(this, IntentConstants.Action.ACTION_CINEMALIST, Bundle().apply {
                putString(IntentConstants.KEY_OSCAR_MOVIE_ID, it.id)
                putString(IntentConstants.KEY_OSCAR_MOVIE_NAME, showMo.showName)
                putString(UTConstants.sqm, SqmKeeper.SQM_COMMUNITY)

            })
        }
    }

    private fun doFavor(mo: ShowMo) {
        checkSession {
            mFarvorPresenter.changeFilmWantStatus(mo)
        }
    }

    private fun gotoVideoList(showMo: ShowMo) {
        NavigatorUtil.gotoVideoList(activity, showMo.id)
    }

    override fun onFavoriteChange(movieID: String?, favorite: Boolean, favorCount: Int?, userShowStatus: Int) {
        if (!UiUtils.isFragmentStateValid(this)) {
            return
        }
        adapter.mData?.filter { it is CommunityFilmCalendarItem && it.data?.id.equals(movieID) }
                ?.forEach {
                    /**
                     * 加这个条件是有原因的，现在app有多种更新想看状态和数量的方式
                     * 1.通过改变showMo中的状态和想看数更新。如果是这种情况下，会导致自身 userShowStatus == commonFilmInfoItem.getData().userShowStatus 而导致return
                     * 2.通过添加想看影评，在这种情况下是获取不到想看数的。只能靠下面的自己wantcount++了，userShowStatus == commonFilmInfoItem.getData()
                     * .userShowStatus就得return掉
                     */
                    if (it is CommunityFilmCalendarItem && it.data != null) {
                        val oldShowStatus: Int = it.data.getUserShowStatus()
                        if (userShowStatus == oldShowStatus && favorCount == null) { //状态没发生变化，数量不做加减法
                            return@forEach
                        }
                        it.getData().userShowStatus = userShowStatus
                        if (favorCount != null && favorCount >= 0) {
                            it.data.wantCount = favorCount
                        } else if (oldShowStatus != ShowMo.USER_SHOW_STATUS_WATCHED) { //从看过，到想看或者已想看，想看数不能变化
                            if (userShowStatus == ShowMo.USER_SHOW_STATUS_WANT) {
                                it.data.wantCount++
                            } else if (userShowStatus == ShowMo.USER_SHOW_STATUS_DEFAULT) {
                                it.data.wantCount--
                            }
                        }
                        if (it.data.scoreAndFavor != null && it.data.scoreAndFavor.favorCount != null) {
                            it.data.scoreAndFavor.favorCount = it.data.wantCount
                        }
                        it.refreshItem()
                    }
                }
    }

    override fun updateFavorStatue(id: String?, isFavor: Boolean, favorCount: Int?, userShowStatus: Int) {

    }

    override fun doFilmFavorStart() {
        if (UiUtils.isActivityStateValid(baseActivity)) {
            baseActivity.showProgressDialog("", true)
        }
    }

    override fun doFilmFavorFinish() {
        if (UiUtils.isActivityStateValid(baseActivity)) {
            baseActivity.dismissProgressDialog()
        }
    }

    override fun showWantedTip(isUpcoming: Boolean, showResultIndexMo: ShowResultIndexMo?, id: String?) {
        if (UiUtils.isActivityStateValid(baseActivity)) {
            WantedTipUtil(baseActivity).showWantedTip(true, showResultIndexMo, id)
        }
    }

    override fun onRefresh(pullToRefresh: Boolean): Boolean {
        if (pullToRefresh) {
            recyclerView.smoothScrollToPosition(0)
            mCommunityFilmCalendarPresenter.loadData(true)
        } else {
            showLoadingView(true)
            mCommunityFilmCalendarPresenter.loadData(false)
        }
        return false

    }

    override fun onRefreshClick() {
        onRefresh(true)
    }

    override fun onLoadMore(): Boolean {
        var status = mCommunityFilmCalendarPresenter.loadData(false)
        val notifyIndex = adapter.indexOfItem(loadingItem)
        if (notifyIndex >= 0) {
            loadingItem.showLoadingStatus()
        }
        return status
    }


    override fun onDestroy() {
        FavoriteManager.getInstance().unRegisterDefault(this)
        super.onDestroy()
    }

    override fun getLayoutId(): Int {
        return R.layout.fragment_community_film_calendar
    }

    override fun getProperties() = Properties().apply { put("source", pageFrom) }


}