package com.taobao.movie.android.app.community.recycleitem


import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.View
import android.widget.TextView
import com.taobao.movie.android.app.common.widget.CommunityCommentView
import com.taobao.movie.android.app.common.widget.CommunityInputView
import com.taobao.movie.android.app.common.widget.UserIconView
import com.taobao.movie.android.app.oscar.ui.community.view.SixPictureView
import com.taobao.movie.android.app.presenter.community.CommunityFeedCardModel
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.CommunityCommentMainTabModel
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DateUtil

/**
 * 聊电影
 */
class CommunityFeedDiscussionItem constructor(communityFeedCardModel: CommunityFeedCardModel,private val recycledViewPool : RecyclerView.RecycledViewPool,private val mListener: OnItemEventListener<Any>) : CommunityFeedBaseItem<CommunityFeedDiscussionItem.ViewHolder,CommunityFeedCardModel>(communityFeedCardModel),View.OnClickListener{

    override fun onBindViewHolder(viewHolder: ViewHolder?) {
        super.onBindViewHolder(viewHolder)
        viewHolder?.sixPictureView?.setData(recycledViewPool, data.discussionModel,mListener)
        viewHolder?.sixPictureView?.visibility = if (data.discussionModel.imageList.isNullOrEmpty()) View.GONE else View.VISIBLE
        when (data.discussionModel.imageList?.size) {
            0 -> viewHolder?.commentContent?.maxLines = 10
            1 -> viewHolder?.commentContent?.maxLines = 2
            2 -> viewHolder?.commentContent?.maxLines = 6
            3 -> viewHolder?.commentContent?.maxLines = 6
            4 -> viewHolder?.commentContent?.maxLines = 2
            5 -> viewHolder?.commentContent?.maxLines = 2
            6 -> viewHolder?.commentContent?.maxLines = 2
            else -> viewHolder?.commentContent?.maxLines = 10
        }
        viewHolder?.communityInputView?.setOnItemClickListener(mListener)
        viewHolder?.communityInputView?.initData(data)
        viewHolder?.itemView?.setOnClickListener(this)
        viewHolder?.communityCommentView?.setOnClickListener(this)
        viewHolder?.columnTxt?.text = data?.columnName
        viewHolder?.userNameTxt?.text = data?.user?.userNick
        viewHolder?.filmNametxt?.text = data?.showName
        viewHolder?.commentContent?.text = data?.content
        viewHolder?.communityCommentView?.initComment(data?.commentItemList)
        viewHolder?.communityCommentView?.visibility = if (data?.commentItemList.isNullOrEmpty()) View.GONE else View.VISIBLE
        viewHolder?.commentContent?.visibility = if (data?.content.isNullOrEmpty()) View.GONE else View.VISIBLE
        viewHolder?.userIconView?.initIcon(data?.user)
        viewHolder?.userIconView?.setIconSize(40)
        viewHolder?.commentTimeTxt?.visibility = if (data?.publishTime == null) View.GONE else View.VISIBLE
        data?.publishTime?.let {
            viewHolder?.commentTimeTxt?.text = DateUtil.getRecommentDate(it/1000)
        }
        viewHolder?.columnTxt?.visibility = if (data?.columnName.isNullOrEmpty()) View.GONE else View.VISIBLE
        viewHolder?.filmNametxt?.visibility = if (data?.showName.isNullOrEmpty()) View.GONE else View.VISIBLE
        viewHolder?.columnLineView?.visibility = if(data?.columnName.isNullOrEmpty() || data?.showName.isNullOrEmpty()) View.GONE else View.VISIBLE
    }

    override fun addComment(communityCommentMainTabModel: CommunityCommentMainTabModel) {
        if(viewHolder == null) {
            return
        }
        if (data?.commentItemList == null) {
            data?.commentItemList = ArrayList()
        }
        data?.commentItemList?.add(communityCommentMainTabModel)
        viewHolder.communityCommentView.visibility = View.VISIBLE
        viewHolder.communityCommentView.addComment(communityCommentMainTabModel)
    }

    override fun onClick(v: View) {
        UTFacade.buttonClickWithoutPage("DiscussCardClick", "type", data.columnType.toString() + "", "discuss_id", data.discussionId, "show_id", data.referShowId)
        MovieNavigator.handleUrl(v.context,data.jumpUrl)
    }

    override fun getLayoutId(): Int {
        return R.layout.community_feed_discussion_item
    }

    class ViewHolder(itemView: View) : CommunityFeedBaseItem.ViewHolder(itemView) {
        var sixPictureView: SixPictureView
        var columnTxt: TextView
        var filmNametxt: TextView
        var userIconView: UserIconView
        var userNameTxt: TextView
        var communityCommentView : CommunityCommentView
        var communityInputView : CommunityInputView
        var commentContent : TextView
        var commentTimeTxt: TextView
        var columnLineView: View

        init {
            sixPictureView = itemView.findViewById(R.id.picture_view)
            columnTxt = itemView.findViewById(R.id.tv_column_name)
            filmNametxt = itemView.findViewById(R.id.film_name_txt)
            userIconView = itemView.findViewById(R.id.user_icon_view)
            userNameTxt = itemView.findViewById(R.id.tv_user_name)
            communityCommentView = itemView.findViewById(R.id.community_comment_view)
            communityInputView = itemView.findViewById(R.id.community_input_view)
            commentContent = itemView.findViewById(R.id.tv_discuss_content)
            commentTimeTxt = itemView.findViewById(R.id.tv_time)
            columnLineView = itemView.findViewById(R.id.text_line)
        }
    }
}