package com.taobao.movie.android.app.community.recycleitem

import android.app.Activity
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import com.taobao.movie.android.app.common.widget.CommunityCommentView
import com.taobao.movie.android.app.common.widget.CommunityInputView
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.MuteYoukuViewController
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.layer.MuteMobileNetResolutionLayer
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.layer.MuteMobileNetResolutionLayer.OnMobileResolutionListener
import com.taobao.movie.android.app.presenter.community.CommunityFeedCardModel
import com.taobao.movie.android.app.video.videoplaymanager.VideoNewCommunityManager
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.CommunityCommentMainTabModel
import com.taobao.movie.android.integration.oscar.model.SmartVideoMo
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.android.video.report.ReportVideoUtils
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.ResHelper

class CommunityFeedVideoItem(communityFeedCardModel: CommunityFeedCardModel?, private val videoCommunityManager: VideoNewCommunityManager, private val mListener: OnItemEventListener<Any>) : CommunityFeedBaseItem<CommunityFeedVideoItem.ViewHolder,CommunityFeedCardModel>(communityFeedCardModel),View.OnClickListener{

    override fun onBindViewHolder(viewHolder: ViewHolder?) {
        super.onBindViewHolder(viewHolder)
        if (viewHolder?.mMuteYoukuViewController == null && viewHolder?.itemView?.context is Activity) {
            viewHolder.mMuteYoukuViewController = MuteYoukuViewController(viewHolder.itemView.context as Activity,
                    ReportVideoUtils.VideoPage.PLAYER_PAGE_FROM_COMMUNITY, videoCommunityManager)
            viewHolder.mMuteYoukuViewController?.onMobileResolutionListener = mOnMobileResolutionListener
            viewHolder.mMuteYoukuViewController?.setTotalMarginLeftAndRight(DisplayUtil.dp2px(30f).toInt())
            val layoutParams = FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
            viewHolder.videoLayout.addView(viewHolder.mMuteYoukuViewController?.videoView, layoutParams)
            val videoContainerLayoutParam: ViewGroup.LayoutParams? = viewHolder.videoLayout.layoutParams
            if (videoContainerLayoutParam != null) {
                val width = (DisplayUtil.getDisplayWidthInPx() - DisplayUtil.dp2px(30f)).toInt()
                val height = width * 9 / 16
                videoContainerLayoutParam.width = width
                videoContainerLayoutParam.height = height
            }
        }

        val smartVideoMo: SmartVideoMo? = data.smartVideoModel
        if (smartVideoMo == null) {
            viewHolder?.mMuteYoukuViewController?.videoView?.visibility = View.GONE
        } else {
            val layoutParams: ViewGroup.LayoutParams? = viewHolder?.videoLayout?.layoutParams
            val width = (DisplayUtil.getDisplayWidthInPx() - DisplayUtil.dp2px(30f)).toInt()
            val height = width * 9 / 16
            if (layoutParams == null) {
                return
            }
            if (smartVideoMo.verticalVideo) {
                layoutParams.width = width
                layoutParams.height = width
            } else {
                layoutParams.width = width
                layoutParams.height = height
            }
            viewHolder?.mMuteYoukuViewController?.videoView?.visibility = View.VISIBLE
            viewHolder?.mMuteYoukuViewController?.bindData(smartVideoMo)
        }

        viewHolder?.communityCommentView?.initComment(data.commentItemList)
        viewHolder?.communityCommentView?.setOnClickListener(this)
        viewHolder?.communityInputView?.setOnItemClickListener(mListener)
        viewHolder?.communityInputView?.initData(data)
        viewHolder?.itemView?.setOnClickListener(this)
        viewHolder?.columnTxt?.text = data.columnName
        viewHolder?.filmNameTxt?.text = data.showName
        viewHolder?.communityCommentView?.visibility = if(data.commentItemList.isNullOrEmpty()) View.GONE else View.VISIBLE
        ShapeBuilder.create().gradient(ResHelper.getColor(R.color.transparent_black_96),
                ResHelper.getColor(R.color.transparent_black_060),
                ResHelper.getColor(R.color.transparent_black_00)).build(viewHolder?.topMask)
    }

    var mOnMobileResolutionListener = object : OnMobileResolutionListener {
        override fun onChooseMobileNetPlayClick() {
            viewHolder?.itemView?.context?.let {
                UTFacade.buttonClickWithoutPage("DiscussCardClick", "type", data.columnType.toString() + "", "discuss_id", data.discussionId, "show_id", data.referShowId)
                val bundle = Bundle()
                //对应4G下点击流量框，到下个页面自动播放的场景
                bundle.putString(MuteMobileNetResolutionLayer.KEY_CLICK_RESOLUTION_AUTO_PLAY_4G_FIRST,
                        MuteMobileNetResolutionLayer.VALUE_RESOLUTION_AUTO_PLAY_4G_FIRST)
                bundle.putSerializable(IntentConstants.KEY_VIDEO_MODEL, data.smartVideoModel)
                MovieNavigator.gotoAction(viewHolder?.itemView?.context, IntentConstants.Action.ACTION_IMMERSE_VIDEO, bundle)
            }
        }
    }


    override fun onClick(v: View) {
        UTFacade.buttonClickWithoutPage("DiscussCardClick", "type", data.columnType.toString() + "", "discuss_id", data.discussionId, "show_id", data.referShowId)
        val bundle = Bundle()
            //对应4G下点击流量框，到下个页面自动播放的场景
        bundle.putString(MuteMobileNetResolutionLayer.KEY_CLICK_RESOLUTION_AUTO_PLAY_4G_FIRST,
                    MuteMobileNetResolutionLayer.VALUE_RESOLUTION_AUTO_PLAY_4G_FIRST)
        bundle.putSerializable(IntentConstants.KEY_VIDEO_MODEL, data.smartVideoModel)
        MovieNavigator.gotoAction(v.context, IntentConstants.Action.ACTION_IMMERSE_VIDEO, bundle)
    }

    override fun getLayoutId(): Int {
        return R.layout.community_feed_video_item
    }

    override fun addComment(communityCommentMainTabModel: CommunityCommentMainTabModel?) {
        if(viewHolder == null) {
            return
        }
        viewHolder.communityCommentView.visibility = View.VISIBLE
        viewHolder.communityCommentView.addComment(communityCommentMainTabModel)
    }

    class ViewHolder(itemView: View) : CommunityFeedBaseItem.ViewHolder(itemView){

        var communityCommentView : CommunityCommentView
        var communityInputView : CommunityInputView
        var videoLayout: FrameLayout
        var mMuteYoukuViewController: MuteYoukuViewController? = null
        var columnTxt: TextView
        var filmNameTxt: TextView
        var topMask: View
        var topBlackMask: View

        init {
            communityCommentView = itemView.findViewById(R.id.community_comment_view)
            communityInputView = itemView.findViewById(R.id.community_input_view)
            videoLayout = itemView.findViewById(R.id.fl_video)
            columnTxt = itemView.findViewById(R.id.colum_name_txt)
            filmNameTxt = itemView.findViewById(R.id.film_name_txt)
            topMask = itemView.findViewById(R.id.top_mask)
            topBlackMask = itemView.findViewById(R.id.top_black_mask)
        }
    }
}