package com.taobao.movie.android.app.community.recycleitem

import android.graphics.Color
import android.graphics.Typeface
import android.support.v4.content.res.ResourcesCompat
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerViewHolder
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.oscar.biz.util.OscarBizUtil
import com.taobao.movie.android.app.oscar.ui.cinema.widget.CinemaFeatureLayout
import com.taobao.movie.android.app.oscar.ui.film.widget.DerivationContainer
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.common.scheme.NavigatorUtil
import com.taobao.movie.android.commonui.utils.ButtonStyleHelper
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.commonui.widget.FilmImagePlayNew
import com.taobao.movie.android.commonui.widget.IconFontTextView
import com.taobao.movie.android.commonui.widget.RoundedTextView
import com.taobao.movie.android.commonui.widget.TitleMarkView
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.RecommentTagVO
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.model.ShowMo.SOLD_TYPE_VOD
import com.taobao.movie.android.sdk.infrastructure.usertrack.ViewOnClickListener
import com.taobao.movie.appinfo.util.DateUtil
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.LogUtil
import com.taobao.movie.appinfo.util.ResHelper
import com.taobao.movie.shawshank.time.ServerTime
import java.text.DecimalFormat

/**
 * Created by 天逐 on 2019-12-16.
 * function:艺术电影日历
 */
class CommunityFilmCalendarItem(showMo: ShowMo?, l: RecyclerExtDataItem.OnItemEventListener<*>?, index: Int) : RecyclerExtDataItem<CommunityFilmCalendarItem.FilmCalendarViewHolder, ShowMo>(showMo, l, index), View.OnClickListener {

    var typeface: Typeface? = null


    //预告片播放点击
    var preViewListener: View.OnClickListener = object : ViewOnClickListener() {
        public override fun onClicked(v: View) {
            if (getData() == null || viewHolder == null) {
                return
            }
            NavigatorUtil.gotoVideoList(v.context, getData().id)
        }
    }


    override fun onClick(v: View?) {
        if (getData() == null || viewHolder == null) {
            return
        }
        when (v) {
            viewHolder.itemView -> {
                onEvent(ItemEvent.EVENT_ID_COMMUNITY_FILM_CALENDAR_ITEM_CLICK)

            }
            viewHolder.buyBtn -> {
                onEvent(ItemEvent.EVENT_ID_COMMUNITY_FILM_CALENDAR_BUY_BTN_CLICK)
            }

        }

    }

    override fun onBindViewHolder(viewholder: FilmCalendarViewHolder) {
        if (getData() == null) {
            return
        }
        var showMo = getData()
        setTypeface(viewholder.wantcount)
        setTypeface(viewholder.remark)
        if (showMo.poster.isNullOrEmpty()) {
            viewholder.poster.filmImage.setImageURI("")
        } else {
            viewholder.poster.filmImage.url = showMo.poster
        }
        //---------------------------海报相关---------------------------//
        if (!showMo.preview.isNullOrEmpty()) {
            viewholder.poster.setIconVisible(true)
            viewholder.poster.setOnClickListener(preViewListener)
        } else {
            viewholder.poster.setIconVisible(false)
            viewholder.poster.setOnClickListener(null)
            viewholder.poster.isClickable = false
        }
        //------------------------影片名和制式、以及点映标-----------------//
        viewholder.name.setTitleAndMark(showMo.showName, showMo.uiMarks)
        if (!showMo.preScheduleDates.isNullOrEmpty()) {
            viewholder.name.setPreScheduleTag("点映")
        } else {
            viewholder.name.setPreScheduleTag("")
        }
        //--------------------导演和主演---------------------------------//
        if (showMo.uiDirectorStr.isNullOrEmpty()) {
            viewholder.director.visibility = View.GONE
        } else {
            viewholder.director.visibility = View.VISIBLE
            viewholder.director.text = showMo.uiDirectorStr
        }

        if (showMo.uiLeadingRoleStr.isNullOrEmpty()) {
            viewholder.leadingRole.visibility = View.GONE
        } else {
            viewholder.leadingRole.visibility = View.VISIBLE
            viewholder.leadingRole.text = showMo.uiLeadingRoleStr
        }

        viewholder.promotionActivity.visibility = View.GONE

        //----------------------推荐理由----------------------------//
        addFilmFestivalViews(viewholder.recommentTagContainer, showMo.showTags)

        //------------------咨询链接-------------------------------------//
        viewholder.container.setFriendAbout(null, 0, null)
        viewholder.container.setDerivationInfo(null, showMo.starMeeting, showMo.uiDerivationMos)
        viewholder.container.setPromotionClickListener { }
        if (viewholder.container.visibility != View.VISIBLE) {
            viewholder.movieContainer.setPadding(viewholder.movieContainer.paddingLeft, viewholder.movieContainer.paddingTop, viewholder.movieContainer.paddingRight,
                    viewholder.movieContainer.resources.getDimensionPixelSize(R.dimen.margin_16))
        } else {
            viewholder.movieContainer.setPadding(viewholder.movieContainer.paddingLeft, viewholder.movieContainer.paddingTop, viewholder.movieContainer.paddingRight,
                    viewholder.movieContainer.resources.getDimensionPixelSize(R.dimen.margin_8))
        }

        //-------------------------购票或者预售按钮---------------------------------------------//
        UiUtils.ensureVisbility(viewholder.nowplayingBtnContainer, View.VISIBLE)
        UiUtils.ensureVisbility(viewholder.upcomingBtnZone, View.GONE)
        val buyBtnParams = viewholder.buyBtn.layoutParams
        if (ShowMo.SOLD_TYPE_PRE == showMo.soldType) {
            if ("预售" != viewholder.buyBtn.text) {
                viewholder.buyBtn.text = "预售"
            }
            viewholder.buyBtn.visibility = View.VISIBLE
            viewholder.mTvHasWatched.visibility = View.GONE
            ButtonStyleHelper.setButtonStyle(viewholder.buyBtn, ButtonStyleHelper.ButtonStyleType.TYPE_PRE_SALE)
            viewholder.buyBtn.setCompoundDrawablesWithIntrinsicBounds(null, null, null, null)
            buyBtnParams.width = DisplayUtil.dp2pxInt(50f)

        } else if (SOLD_TYPE_VOD == showMo.soldType) {

            viewholder.buyBtn.visibility = View.VISIBLE
            viewholder.mTvHasWatched.visibility = View.GONE
            val content = ResHelper.getString(R.string.homepage_watch_film)
            viewholder.buyBtn.text = content
            val drawableLeft = ResHelper.getDrawable(R.drawable.play_white_icon)
            drawableLeft.setBounds(0, 0, DisplayUtil.dp2pxInt(9f), DisplayUtil.dp2pxInt(9f))
            ButtonStyleHelper.setButtonStyle(viewholder.buyBtn, ButtonStyleHelper.ButtonStyleType.TYPE_BUY, false)
            viewholder.buyBtn.setCompoundDrawablesWithIntrinsicBounds(drawableLeft, null, null, null)
            viewholder.buyBtn.compoundDrawablePadding = DisplayUtil.dp2pxInt(3f)
            buyBtnParams.width = DisplayUtil.dp2pxInt(70f)
            viewholder.buyBtn.setPadding(DisplayUtil.dp2pxInt(8f), 0, DisplayUtil.dp2pxInt(8f), 0)
            viewholder.buyBtn.gravity = Gravity.CENTER
            viewholder.buyBtn.layoutParams = buyBtnParams
            viewholder.buyBtn.includeFontPadding = false

        } else if (ShowMo.SOLD_TYPE_NORMAL == showMo.soldType) {
            viewholder.buyBtn.visibility = View.VISIBLE
            viewholder.mTvHasWatched.visibility = View.GONE
            viewholder.buyBtn.text = "购票"
            buyBtnParams.width = DisplayUtil.dp2pxInt(50f)
            viewholder.buyBtn.setCompoundDrawablesWithIntrinsicBounds(null, null, null, null)
            ButtonStyleHelper.setButtonStyle(viewholder.buyBtn, ButtonStyleHelper.ButtonStyleType.TYPE_BUY)
        } else {
            //想看看过已想看
            //想看和看过逻辑
            if (showMo.getUserShowStatus() == ShowMo.USER_SHOW_STATUS_DEFAULT) {
                viewholder.buyBtn.visibility = View.VISIBLE
                viewholder.mTvHasWatched.visibility = View.GONE
                viewholder.buyBtn.text = "想看"
                viewholder.buyBtn.visibility = View.VISIBLE
                ButtonStyleHelper.setButtonStyle(viewholder.buyBtn, ButtonStyleHelper.ButtonStyleType.TYPE_WANT, true)
            } else if (showMo.getUserShowStatus() == ShowMo.USER_SHOW_STATUS_WANT) {
                viewholder.buyBtn.visibility = View.VISIBLE
                viewholder.mTvHasWatched.visibility = View.GONE
                viewholder.buyBtn.text = "已想看"
                viewholder.buyBtn.visibility = View.VISIBLE
                ButtonStyleHelper.setButtonStyle(viewholder.buyBtn, ButtonStyleHelper.ButtonStyleType.TYPE_WANT_DONE, true)
            } else if (showMo.getUserShowStatus() == ShowMo.USER_SHOW_STATUS_WATCHED) { //看过
                viewholder.buyBtn.visibility = View.GONE
                viewholder.mTvHasWatched.visibility = View.VISIBLE
            }
        }
        viewholder.buyBtn.layoutParams = buyBtnParams
        viewholder.buyBtn.setOnClickListener(this)
        viewholder.itemView.setOnClickListener(this)

        if (DateUtil.compareDate(showMo.getOpenDay(), ServerTime.getCurrentServerTime())) {//上映前
            UiUtils.ensureVisbility(viewholder.wantContainer, View.VISIBLE)
            if (showMo.scoreAndFavor != null && showMo.scoreAndFavor.score != null && showMo.scoreAndFavor.score.score != null
                    && !TextUtils.isEmpty(showMo.scoreAndFavor.score.scoreName)) {
                UiUtils.ensureVisbility(viewholder.remarkContainer, View.VISIBLE)
                UiUtils.ensureVisbility(viewholder.divider1, View.GONE)
                UiUtils.ensureVisbility(viewholder.remarkPeople, View.GONE)
                viewholder.remarkTitle.text = showMo.scoreAndFavor.score.scoreName
                if (showMo.scoreAndFavor.score.score != null && showMo.scoreAndFavor.score.score.toInt() != 0) {
                    viewholder.remark.text = DecimalFormat("0.0").format(OscarBizUtil.remarkRule5_1(showMo.scoreAndFavor.score.score))
                } else {
                    viewholder.remark.text = ""
                }
            } else {
                UiUtils.ensureVisbility(viewholder.remarkContainer, View.GONE)
            }
            if (showMo.scoreAndFavor != null && showMo.scoreAndFavor.favorCount != null && showMo.scoreAndFavor.favorCount > 0) {
                UiUtils.ensureVisbility(View.VISIBLE, viewholder.wantCountTitle, viewholder.wantcount, viewholder.divider2)
                viewholder.wantcount.text = UiUtils.formatTosepara(showMo.scoreAndFavor.favorCount)
            } else {
                UiUtils.ensureVisbility(View.GONE, viewholder.wantCountTitle, viewholder.wantcount, viewholder.divider2)
            }

            if (!TextUtils.isEmpty(showMo.uiDateStr)) {
                UiUtils.ensureVisbility(viewholder.date, View.VISIBLE)
                viewholder.date.text = showMo.uiDateStr
            } else {
                UiUtils.ensureVisbility(viewholder.divider2, View.GONE)
                UiUtils.ensureVisbility(viewholder.date, View.GONE)
            }

        } else {//上映后
            UiUtils.ensureVisbility(viewholder.wantContainer, View.GONE)
            UiUtils.ensureVisbility(viewholder.remarkContainer, View.VISIBLE)
            if (showMo.scoreAndFavor != null) {
                //显示 正式评分 ，点映评分，或者暂无评分
                if (showMo.scoreAndFavor.score != null && showMo.scoreAndFavor.score.scoreName != null) {

                    viewholder.remarkTitle.text = showMo.scoreAndFavor.score.scoreName
                    if (showMo.scoreAndFavor.score.score != null && showMo.scoreAndFavor.score.score.toInt() != 0) {
                        viewholder.remark.text = DecimalFormat("0.0").format(OscarBizUtil.remarkRule5_1(showMo.scoreAndFavor.score.score))
                    } else {
                        viewholder.remark.text = ""
                    }
                    if (showMo.scoreAndFavor.score.score != null && !TextUtils.isEmpty(showMo.scoreAndFavor.score.scoreCountDesc)) {
                        viewholder.remarkPeople.text = showMo.scoreAndFavor.score.scoreCountDesc.replace(" ".toRegex(), "")
                        UiUtils.ensureVisbility(viewholder.remarkPeople, View.VISIBLE)
                        UiUtils.ensureVisbility(viewholder.divider1, View.VISIBLE)
                    } else {
                        UiUtils.ensureVisbility(viewholder.remarkPeople, View.GONE)
                        UiUtils.ensureVisbility(viewholder.divider1, View.GONE)
                    }
                } else {
                    viewholder.remarkContainer.visibility = View.INVISIBLE
                }
            } else {
                viewholder.remarkContainer.visibility = View.INVISIBLE
            }
        }
    }

    private fun setTypeface(filmType: TextView) {
        try {
            if (typeface == null) {
                typeface = ResourcesCompat.getFont(filmType.context, com.taobao.movie.android.component.R.font.rubik_medium)
            }
            if (typeface != null) {
                filmType.typeface = typeface
            }
        } catch (e: Exception) {
            LogUtil.printStackTraceAndMore(e)
        }

    }

    private fun addFilmFestivalViews(recommentTagContainer: CinemaFeatureLayout?, recommentTagList: List<RecommentTagVO>?) {
        if (recommentTagContainer != null && !recommentTagList.isNullOrEmpty()) {
            UiUtils.ensureVisbility(recommentTagContainer, View.VISIBLE)
            try {
                recommentTagContainer.removeAllViews()
                val charWidth = DisplayUtil.dp2px(9f).toInt()
                val margin = DisplayUtil.dp2px(4f).toInt()
                for (recommentTagVO in recommentTagList) {
                    if (TextUtils.isEmpty(recommentTagVO.color) || TextUtils.isEmpty(recommentTagVO.title)) {
                        continue
                    }
                    if (recommentTagVO.type == RecommentTagVO.TYPE_TEMPERARY) {
                        val roundedTextView = RoundedTextView(recommentTagContainer.context, 6, ResHelper.getColor(R.color.color_red_with_alpha),
                                3, 3, RoundedTextView.ROUND_TYPE.FILL.ordinal)
                        roundedTextView.setTextColor(ResHelper.getColor(R.color.common_color_1045))
                        roundedTextView.text = recommentTagVO.title
                        roundedTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, charWidth.toFloat())
                        roundedTextView.measure(0, 0)
                        val lp = ViewGroup.MarginLayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
                        lp.rightMargin = margin
                        recommentTagContainer.addView(roundedTextView, lp)
                    } else {
                        val colorString = recommentTagVO.color
                        val bgColor = Color.parseColor("#66$colorString")
                        val textColor = Color.parseColor("#$colorString")
                        val roundedTextView = RoundedTextView(recommentTagContainer.context, 6, bgColor, 3, 3,
                                RoundedTextView.ROUND_TYPE.STROKE.ordinal)
                        roundedTextView.setTextColor(textColor)
                        roundedTextView.text = recommentTagVO.title
                        roundedTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, charWidth.toFloat())
                        roundedTextView.measure(0, 0)
                        val lp = ViewGroup.MarginLayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
                        lp.rightMargin = margin
                        recommentTagContainer.addView(roundedTextView, lp)
                    }
                }
            } catch (e: Exception) {
                LogUtil.e("recommentTagContainer", e)
            }

        } else {
            UiUtils.ensureVisbility(recommentTagContainer, View.GONE)
        }
    }


    class FilmCalendarViewHolder(v: View) : CustomRecyclerViewHolder<CommunityFilmCalendarItem>(v) {

        var poster: FilmImagePlayNew
        var name: TitleMarkView
        var leadingRole: TextView
        var director: TextView
        var container: DerivationContainer
        var buyBtn: Button
        var promotionActivity: TextView
        var movieContainer: View
        var remark: TextView
        var remarkTitle: TextView
        var remarkPeople: TextView
        var remarkContainer: View
        //即将上映
        var date: TextView
        var wantcount: TextView
        var wantCountTitle: TextView
        var wantContainer: View
        var divider1: View
        var divider2: View
        //推荐理由
        var recommentTagContainer: CinemaFeatureLayout
        //按钮区域
        var nowplayingBtnContainer: View
        var upcomingBtnZone: View
        val mTvHasWatched: IconFontTextView


        init {
            poster = v.findViewById(R.id.pic_poster)
            poster.filmImage.setLoadImageSize(R.style.FilmImageSize)
            name = v.findViewById(R.id.name)
            name.setType(TitleMarkView.TYPE_FILM)
            leadingRole = v.findViewById(R.id.leading_role)
            director = v.findViewById(R.id.director)
            container = v.findViewById(R.id.derivation)
            nowplayingBtnContainer = v.findViewById(R.id.nowplaying_btn_zone)
            upcomingBtnZone = v.findViewById(R.id.upcoming_btn_zone)
            buyBtn = v.findViewById(R.id.btn_buy)
            mTvHasWatched = v.findViewById(R.id.tv_has_watched)
            promotionActivity = v.findViewById(R.id.promotion_activity)
            movieContainer = v.findViewById(R.id.movie_container)
            remark = v.findViewById(R.id.tv_remark)
            remarkTitle = v.findViewById(R.id.tv_remark_title)
            remarkPeople = v.findViewById(R.id.tv_remark_people)
            remarkContainer = v.findViewById(R.id.remark_container)
            divider1 = v.findViewById(R.id.divider1)
            divider2 = v.findViewById(R.id.divider2)
            //即将上映
            date = v.findViewById(R.id.date)
            wantcount = v.findViewById(R.id.wantcount)
            wantCountTitle = v.findViewById(R.id.tv_want_count_title)
            wantContainer = v.findViewById(R.id.date_count_container)
            recommentTagContainer = v.findViewById(R.id.recomment_tag_container)
        }
    }


    override fun getLayoutId(): Int {
        return R.layout.oscar_film_frag_list_nowplaying_or_upcoming_item
    }


}
