package com.taobao.movie.android.app.order.ui.widget

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.commonui.widget.IconFontTextView
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.commonutil.kotlin.isTrue
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.product.model.VisitorCardVO
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.ResHelper
import kotlinx.android.synthetic.main.electronic_item_layout.view.*

/**
 *  Created by 天逐 on 2020-01-19.
 *  function:电子访客证UI
 */
class ElectronicVisitorView : LinearLayout, View.OnClickListener {
    companion object {
        const val TYPE_FROM_TICKET_REMIND = "type_from_ticket_remind"
        const val TYPE_FROM_BUY_TICKET_SUCCESS = "type_from_buy_ticket_success"
        const val TYPE_FROM_TICKET_DETAIL = "type_from_ticket_detail"

    }

    private lateinit var mRightTv: TextView
    private lateinit var mRightArrow: IconFontTextView
    private var mData: VisitorCardVO? = null
    private var mFrom = TYPE_FROM_TICKET_REMIND

    constructor(context: Context?) : super(context) {
        init()

    }

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {
        init()

    }

    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init()

    }

    private fun init() {
        LayoutInflater.from(context).inflate(R.layout.electronic_item_layout, this, true)
        mRightTv = findViewById(R.id.tv_electronic_right)
        mRightArrow = findViewById(R.id.electronic_right_arrow)
        setOnClickListener(this)
    }

    fun setData(data: VisitorCardVO?, from: String) {
        if (data == null) {
            return
        }
        this.mData = data
        this.mFrom = from
        if (mData?.hasCard.isTrue()) {
            mRightTv.text = "查看"
            mRightTv.setTextColor(ResHelper.getColor(R.color.common_text_color37))
            mRightArrow.setTextColor(ResHelper.getColor(R.color.common_text_color37))
        } else {
            mRightTv.text = "去设置"
            mRightTv.setTextColor(ResHelper.getColor(R.color.common_color_1045))
            mRightArrow.setTextColor(ResHelper.getColor(R.color.common_color_1045))
        }
        when (from) {
            TYPE_FROM_TICKET_REMIND -> {
                electronic_line.visibility = View.GONE
                tv_electronic_des.visibility = View.GONE
                ShapeBuilder.create().solid(ResHelper.getColor(R.color.color_tpp_primary_page_bg_gray)).radius(DisplayUtil.dp2px(21f)).build(this)
            }
            TYPE_FROM_BUY_TICKET_SUCCESS -> {
                electronic_line.visibility = View.VISIBLE
                tv_electronic_des.visibility = View.VISIBLE
                tv_electronic_des.text = mData?.message
                ShapeBuilder.create().solid(ResHelper.getColor(R.color.common_text_color46)).radius(DisplayUtil.dp2px(9f)).build(this)
            }
            TYPE_FROM_TICKET_DETAIL -> {
                electronic_line.visibility = View.GONE
                tv_electronic_des.visibility = View.GONE
                ShapeBuilder.create().solid(ResHelper.getColor(R.color.common_text_color15)).radius(DisplayUtil.dp2px(21f)).build(this)
            }
        }

    }

    override fun onClick(v: View?) {
        mData?.jumpUrl?.let {
            when (mFrom) {
                TYPE_FROM_TICKET_REMIND -> {
                UTFacade.buttonClickWithoutPage("PopupETicketBtnClick", "status", if (mData?.hasCard.isTrue()) "2" else "1")
            }
                else -> {
                    UTFacade.buttonClickWithoutPage("ETicketBtnClick", "status", if (mData?.hasCard.isTrue()) "2" else "1")
                }
            }

            MovieNavigator.handleUrl(context, mData?.jumpUrl)
        }

    }

}