package com.taobao.movie.android.app.oscar.ui.community.fragment

import android.os.Bundle
import android.view.View
import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalDataModel
import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalItemModel
import com.taobao.movie.android.app.oscar.ui.community.presenter.FilmFestivalIndexPresenter
import com.taobao.movie.android.app.oscar.ui.community.presenter.IFilmFestivalIndex
import com.taobao.movie.android.app.oscar.ui.smartvideo.item.FilmFestivalIndexItem
import com.taobao.movie.android.commonui.component.lcee.LceeLoadingListFragment
import com.taobao.movie.android.commonui.recyclerview.LoadingItem
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTConstants
import com.taobao.movie.appinfo.util.DisplayUtil
import java.util.*

class FilmFestivalIndexFragment : LceeLoadingListFragment<FilmFestivalIndexPresenter>(), IFilmFestivalIndex {


    companion object {
        const val Page_MovieFestivalIndex = "Page_MovieFestivalIndex"
        const val Page_MovieFestivalIndexSPM = "13826012"
        const val KEY_PAGE_FROM = "key_page_from"
        const val PAGE_FROM_FESTIVAL_TITLE = "1" //1=电影日历模块，2=查看更多
        const val PAGE_FROM_FESTIVAL_MORE = "2" //1=电影日历模块，2=查看更多
    }

    private var pageFrom = PAGE_FROM_FESTIVAL_TITLE


    override fun onCreate(savedInstanceState: Bundle?) {
        UTConstants.SPM_Map[Page_MovieFestivalIndex] = Page_MovieFestivalIndexSPM
        super.onCreate(savedInstanceState)
        utPageName = Page_MovieFestivalIndex
        setUTPageEnable(true)
        getIntentData()
    }

    fun getIntentData() {
        arguments?.let {
            pageFrom = it?.getString(KEY_PAGE_FROM)
        }
    }

    private var isRefresh = false

    override fun initViewContent(layoutView: View?, savedInstanceState: Bundle?) {
        super.initViewContent(layoutView, savedInstanceState)
        layoutView?.setPadding(0, DisplayUtil.dp2pxInt(15f), DisplayUtil.dp2pxInt(6f), 0)
        presenter.refresh()
    }

    override fun createPresenter(): FilmFestivalIndexPresenter = FilmFestivalIndexPresenter()

    override fun setDataErrorView(isToast: Boolean, resultCode: Int, returnCode: Int, returnMsg: String?): Boolean = false

    override fun setDataContentView(isCache: Boolean, data: Any?) {
        if (isRefresh) {
            adapter.clearItems()
            isRefresh = false
        }
        if (data is FilmFestivalDataModel && data.filmFestivalItemModelList.isNotEmpty()) {
            if (adapter.getCount(FilmFestivalIndexItem::class.java) <= 0) {
                addFilmFestival(data.filmFestivalItemModelList)
            } else {
                var lastData = adapter.getItem(adapter.itemCount - 1).data as FilmFestivalItemModel
                if (lastData.groupName == data.filmFestivalItemModelList[0].groupName) {
                    (adapter.getItem(adapter.itemCount - 1) as FilmFestivalIndexItem).refreshFilmFestival(data.filmFestivalItemModelList[0].filmFestivalList)
                    if (data.filmFestivalItemModelList.size > 1) {
                        addFilmFestival(data.filmFestivalItemModelList.subList(1, data.filmFestivalItemModelList.size))
                    }
                } else {
                    addFilmFestival(data.filmFestivalItemModelList)
                }
            }
        }

    }

    private fun addFilmFestival(filmFestivalItemModelList: List<FilmFestivalItemModel>) {
        for (it in filmFestivalItemModelList) {
            adapter.addItem(FilmFestivalIndexItem(it))
        }
    }

    override fun onLoadMore(): Boolean {
        isRefresh = false
        presenter.loadMore()
        return super.onLoadMore()
    }


    override fun showLoadingView(hasData: Boolean) {
        super.showLoadingView(adapter.getCount(FilmFestivalIndexItem::class.java) > 0)
    }

    override fun showError(isToast: Boolean, resultCode: Int, returnCode: Int, returnMsg: String?) {
        super.showError(adapter.getCount(FilmFestivalIndexItem::class.java) > 0, resultCode, returnCode, returnMsg)
    }

    override fun removeLoadingItem() {
        if (adapter.indexOfItem(LoadingItem::class.java) >= 0) {
            adapter.removeItem(LoadingItem::class.java, true)
        }
    }

    override fun onRefreshClick() {
        presenter.refresh()
    }

    override fun onRefresh(pullToRefresh: Boolean): Boolean {
        isRefresh = true
        presenter.refresh()
        setCanLoadMore(true)
        return true
    }

    override fun getProperties() = Properties().apply { put("source", pageFrom) }


}