package com.taobao.movie.android.app.oscar.ui.community.presenter

import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalDataModel
import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalItemModel
import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalResponse
import com.taobao.movie.android.app.oscar.ui.community.request.FilmFestivalRequest
import com.taobao.movie.android.app.presenter.common.LceeDefaultPresenter
import com.taobao.movie.android.common.Region.RegionExtServiceImpl
import com.taobao.movie.android.net.mtop.rx.ApiException
import com.taobao.movie.android.net.mtop.rx.ShawShankApiObserver

class FilmFestivalIndexPresenter : LceeDefaultPresenter<IFilmFestivalIndex>(){

    var lastId : String? = null
    var isLoading = false
    var hasMore = true

    val filmFestivalRequest by lazy{
        FilmFestivalRequest()
    }

    val mCityCode by lazy{
        RegionExtServiceImpl().userRegion.cityCode
    }

    private fun requestFilmFestival(isLoadMore : Boolean) {
        isLoading = true
        filmFestivalRequest.cityCode = mCityCode
        if (isLoadMore) {
            filmFestivalRequest.lastId = lastId
        } else {
            filmFestivalRequest.lastId = null
        }
        filmFestivalRequest.subscribe(getAutomaticResource(),object:ShawShankApiObserver.ApiConsumer<FilmFestivalResponse>(){
            override fun onBefore() {
                if (isAttached) {
                    view.showLoadingView(false)
                }
            }

            override fun onSuccess(resp: FilmFestivalResponse?) {
                isLoading = false
                if(!isAttached) {
                    return
                }

                if (resp == null || resp.showFestivals.isEmpty()) {
                    hasMore = false
                    view.removeLoadingItem()
                    view.showEmpty()
                } else {
                    hasMore = resp.showFestivals.size >=10
                    lastId = resp.showFestivals.last().id
                    var filmFestivalDataModel = FilmFestivalDataModel()
                    var filmFestivalItemModelList = ArrayList<FilmFestivalItemModel>()
                    var filmFestivalItemModel = FilmFestivalItemModel()
                    var groupName = ""
                    for(showFestival in resp.showFestivals) {
                        showFestival.groupName?.let {
                            if (it != groupName) {
                                groupName = it
                                filmFestivalItemModel = FilmFestivalItemModel()
                                filmFestivalItemModel.groupName = groupName
                                filmFestivalItemModel.filmFestivalList.add(showFestival)
                                filmFestivalItemModelList.add(filmFestivalItemModel)
                            } else {
                                filmFestivalItemModel.filmFestivalList.add(showFestival)
                            }
                        }
                    }
                    filmFestivalDataModel.filmFestivalItemModelList = filmFestivalItemModelList
                    view.showContentView(false,filmFestivalDataModel)
                    view.setCanLoadMore(hasMore)
                }
            }

            override fun onFail(e: ApiException) {
                isLoading = false
                super.onFail(e)
                if(isAttached) {
                    view.showError(true,e.resultCode,e.returnCode,e.message)
                }
            }
        })
    }

    override fun hasMore() = hasMore

    fun loadMore() {
        if(isLoading || !hasMore) {
            return
        }
        requestFilmFestival(true)
    }

    fun refresh() {
        if (isLoading) {
            return
        }
        requestFilmFestival(false)
    }



}