package com.taobao.movie.android.app.oscar.ui.homepage.adapter

import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v4.app.FragmentStatePagerAdapter
import android.support.v4.view.PagerAdapter
import android.util.SparseArray
import android.view.ViewGroup
import com.alipay.mobile.h5container.api.H5Param
import com.taobao.movie.android.app.abtest.ABTestConfigProvider
import com.taobao.movie.android.app.oscar.ui.homepage.EVENT_REFRESH_FINISH
import com.taobao.movie.android.app.oscar.ui.homepage.fragment.FeedTypeDoubleRowRecommandFragment
import com.taobao.movie.android.app.oscar.ui.homepage.fragment.FeedTypeRecommendFragment
import com.taobao.movie.android.common.Region.RegionBizServiceImpl
import com.taobao.movie.android.common.h5nebula.fragment.NebulaH5Fragment
import com.taobao.movie.android.commonui.component.lcee.RefreshView
import com.taobao.movie.android.commonutil.LocalEvent
import com.taobao.movie.android.commonutil.LocalEventBus
import com.taobao.movie.android.integration.oscar.model.ABTestAllExperimentModel
import com.taobao.movie.android.integration.oscar.uiInfo.HomepageVO
import com.taobao.movie.android.integration.oscar.uiInfo.PositionTab
import com.taobao.movie.damai.DamaiTabFragment

class HomePagerAdapter(val fm: FragmentManager?, var homepageVO: HomepageVO) : FragmentStatePagerAdapter(fm), NebulaH5Fragment.OnPageLoadListener, DamaiTabFragment.OnPageLoadListener {

    var tempTabs: List<PositionTab>? = null
    var currentFragment: Fragment? = null
    private val fragments = SparseArray<Fragment>()
    private var abTestModel: ABTestAllExperimentModel? = null

    init {
        abTestModel = ABTestConfigProvider.getProvider().allExperimentConfigs
    }

    override fun getItem(p0: Int): Fragment {
        val positionTab = homepageVO.tabs?.get(p0)

        when (positionTab?.type) {
            PositionTab.TYPE_H5 -> {
                //h5
                if (fragments.get(p0) == null) {
                    val fragment = NebulaH5Fragment()
                            .apply {
                                arguments = Bundle().apply {
                                    putString(H5Param.URL, positionTab.url)
                                    putBoolean(NebulaH5Fragment.SHOW_NATIVE_LOADING, false)
                                    putBoolean(NebulaH5Fragment.IS_IN_PAGER, true)
                                    putBoolean(NebulaH5Fragment.LAZY_LOAD, true)
                                    putBoolean(NebulaH5Fragment.NEED_PROGRESS, false)
                                }

                                fragments.put(p0, this)
                            }
                    fragment.addOnPageLoadListener(this)
                    return fragment
                }
                return fragments[p0]
            }
            PositionTab.TYPE_DAMAI -> {
                //大麦
                if (fragments.get(p0) == null) {
                    return DamaiTabFragment().getInstance(positionTab.targetParam,
                            RegionBizServiceImpl.getUserRegion().cityCode,
                            positionTab.name).apply {
                        fragments.put(p0, this)
                        setOnPageLoadListener(this@HomePagerAdapter)
                    }
                }
                return fragments[p0]
            }
            else -> {
                //feed
                if (fragments.get(p0) == null) {
//                    return if (abTestModel?.findFeedScenary("index_feed_entrance", "double") != null) {
                        FeedTypeDoubleRowRecommandFragment().setHomepageVO(homepageVO).apply {
                            fragments.put(p0, this)
                        }
//                    } else {
//                        FeedTypeRecommendFragment().setHomepageVO(homepageVO).setNeedRequestGetAB(abTestModel?.findFeedScenary("index_feed_entrance", "single") != null).apply {
//                            fragments.put(p0, this)
//                        }
//                    }
                }
                return fragments[p0]
            }
        }
    }

    override fun getCount(): Int {
        return homepageVO.tabs?.size ?: 0
    }

    override fun setPrimaryItem(container: ViewGroup, position: Int, obj: Any) {
        super.setPrimaryItem(container, position, obj)
        if (obj is Fragment) {
            currentFragment = obj
        }
    }

    override fun getItemPosition(obj: Any): Int {
        if (obj is Fragment) {

            if (tempTabs?.size == homepageVO.tabs?.size) {
                val index = fragments.indexOfValue(obj)
                if (index >= 0) {
                    if (tempTabs?.get(index) != homepageVO.tabs?.get(index) && index < fragments.size()) {
                        fragments.removeAt(index)
                        return PagerAdapter.POSITION_NONE
                    } else if (tempTabs?.get(index) == homepageVO.tabs?.get(index) && obj is RefreshView && obj == currentFragment) {
                        obj.setRefreshData(homepageVO)
                        obj.onViewRefresh(true)
                    }
                } else {
                    return PagerAdapter.POSITION_NONE
                }
            } else {
                fragments.clear()
                return PagerAdapter.POSITION_NONE
            }
        }
        return super.getItemPosition(obj)
    }

    override fun onPageEvent(progress: Int) {
        if (progress >= 100) {
            LocalEventBus.getDefault().post(LocalEvent(EVENT_REFRESH_FINISH))
        }
    }

    fun notifyDataSetChanged(homepageVO: HomepageVO) {
        tempTabs = this.homepageVO.tabs
        this.homepageVO = homepageVO
        notifyDataSetChanged()
        tempTabs = null
    }

    override fun onPageProgress(progress: Int) {
        if (progress >= 100) {
            LocalEventBus.getDefault().post(LocalEvent(EVENT_REFRESH_FINISH))
        }
    }
}