package com.taobao.movie.android.app.oscar.ui.homepage.item.preview

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.graphics.Rect
import android.graphics.drawable.GradientDrawable
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.LinearSnapHelper
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.StaggeredGridLayoutManager
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerViewHolder
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.oscar.ui.homepage.item.HomeFeedRecommendDividerItem
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.IMuteYouKuController
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.IYoukuViewController.IPlayReportListener
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.MuteYoukuViewController
import com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.layer.MuteMobileNetResolutionLayer
import com.taobao.movie.android.app.video.videoplaymanager.VideoFeedPreviewManager
import com.taobao.movie.android.arch.items.CustomListAdapter
import com.taobao.movie.android.arch.items.CustomViewHolder
import com.taobao.movie.android.arch.items.OnItemClickListener
import com.taobao.movie.android.arch.items.RecyclerItem
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.commonui.utils.RippleUtils
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.model.SmartVideoMo
import com.taobao.movie.android.integration.oscar.uiInfo.PreviewsPlayModuleVO
import com.taobao.movie.android.ut.DogCat
import com.taobao.movie.android.video.report.ReportVideoUtils
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.ResHelper

class MovieTrailerItem(val vo: PreviewsPlayModuleVO , val videoFeedPreviewManager: VideoFeedPreviewManager?, val mListener : RecyclerExtDataItem.OnItemEventListener<Any>, val mPlayReportListener: IPlayReportListener)
    : RecyclerExtDataItem<MovieTrailerItem.MovieTrailerViewHolder, PreviewsPlayModuleVO>(vo, mListener), OnItemClickListener<RecyclerItem<ShowMo>> {

    private var listAdapter: MovieTrailerListAdapter? = null
    private var currentSelectItem = 0
    private var snapHelper = LinearSnapHelper()
    private var scrolled = false
    private val scrollListener = object: RecyclerView.OnScrollListener(){
        override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
            if (newState == RecyclerView.SCROLL_STATE_IDLE && scrolled) {
                scrolled = false
                DogCat.getInstance().click().ctrlName("TrailerModuleMovieSlide").commit()
            }
        }

        override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
            scrolled = dx != 0
        }
    }

    override fun onBindViewHolder(t: MovieTrailerViewHolder?) {
        bindTitle(t)
        bindLink(t)
        bindVideo(t)
        bindPreviewList(t)
    }

    private fun bindTitle(t: MovieTrailerViewHolder?) {
        t?.title?.apply {
            DogCat.getInstance().exposure(this).block("TrailerModuleExpose").viewId(t.adapterPosition.toString()).commit()
            text = data.title
        }
    }

    private fun bindLink(t: MovieTrailerViewHolder?) {
        if (data.rankType?.rankName.isNullOrEmpty()) {
            t?.linkLayout?.visibility = View.GONE
        } else {
            t?.linkLayout?.apply {
                DogCat.getInstance().click(this).ctrlName("TrailerModuleHotlistClick").commit()
                visibility = View.VISIBLE
                setOnClickListener {
                    data.rankType?.rankListH5Url?.apply {
                        MovieNavigator.handleUrl(t.itemView.context, this)
                    }
                }
            }
            t?.linkTxt?.text = data.rankType?.rankName
        }
    }


    var  mOnMuteVideoPlayListener = IMuteYouKuController.OnMuteVideoPlayListener {
        if(hasNextVideoMo()){
            changeCurrentIndex(++currentSelectItem)
            bindVideo(viewHolder)
            videoFeedPreviewManager?.onStopScroll()
        }

    }


    private fun bindVideo(t: MovieTrailerViewHolder?) {
        t?.let {
            if (t.mMuteYoukuViewController == null) {
                t.mMuteYoukuViewController = MuteYoukuViewController(t.itemView.context as Activity?, ReportVideoUtils.VideoPage.PLAYER_PAGE_FROM_FEED_PREVIEW, videoFeedPreviewManager)
                val width = (DisplayUtil.getDisplayWidthInPx())
                val height = width * 9 / 16
                val layoutParams = FrameLayout.LayoutParams(width, height)
                t.mMuteYoukuViewController?.videoView?.layoutParams = layoutParams
                t.mVideoContainer?.addView(t?.mMuteYoukuViewController?.videoView, 0)
            }
//            t.mMuteYoukuViewController?.bindData(getData().shows?.get(0)?.videoList?.get(0))
            t.mMuteYoukuViewController?.bindData(getCurrentVideoMo())
            t.mMuteYoukuViewController?.eventListener = listener
            t.mMuteYoukuViewController?.setOnMuteVideoPlayListener(mOnMuteVideoPlayListener)
            t.mMuteYoukuViewController?.setOnMobileResolutionListener {
                listener?.onEvent(ItemEvent.EVENT_ID_COMMUNITY_BUSINESS_PREVIEW_VIDEO_CLICK, getCurrentVideoMo(), MuteMobileNetResolutionLayer.VALUE_RESOLUTION_AUTO_PLAY_4G_FIRST)
            }
            t.mVideoContainer?.setOnClickListener {
                listener?.onEvent(ItemEvent.EVENT_ID_COMMUNITY_BUSINESS_PREVIEW_VIDEO_CLICK, getCurrentVideoMo(), MuteMobileNetResolutionLayer.VALUE_RESOLUTION_AUTO_PLAY_4G_FIRST)
            }
            t.mMuteYoukuViewController?.playReportListener = mPlayReportListener
        }

    }

    private fun bindPreviewList(t: MovieTrailerViewHolder?) {
        t?.apply {
            if (listAdapter == null) {
                listAdapter = MovieTrailerListAdapter().apply {
                    movieList?.let {
                        attach(it)
                        it.layoutManager = LinearLayoutManager(it.context).apply {
                            orientation = LinearLayoutManager.HORIZONTAL
                        }
                        it.addItemDecoration(object: RecyclerView.ItemDecoration() {
                            override fun getItemOffsets(outRect: Rect, view: View, parent: RecyclerView, state: RecyclerView.State) {
                                super.getItemOffsets(outRect, view, parent, state)
                                outRect.left = DisplayUtil.dp2pxInt(3f)
                                outRect.right = DisplayUtil.dp2pxInt(3f)
                                if (parent.getChildLayoutPosition(view) == 0) {
                                    outRect.left = DisplayUtil.dp2px(15f).toInt()
                                } else if (parent.getChildLayoutPosition(view) == parent.adapter!!.itemCount - 1) {
                                    outRect.right = DisplayUtil.dp2pxInt(15f)
                                }
                            }
                        })
                        it.removeOnScrollListener(scrollListener)
                        it.addOnScrollListener(scrollListener)

                        if (it.layoutParams is ViewGroup.MarginLayoutParams) {
                            (it.layoutParams as ViewGroup.MarginLayoutParams).topMargin = t.mMuteYoukuViewController?.videoView?.layoutParams?.height
                                    ?: DisplayUtil.getDisplayWidthInPx() * 9 / 16
                        } else {
                            it.layoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.MATCH_PARENT, ViewGroup.MarginLayoutParams.WRAP_CONTENT).apply {
                                topMargin = t.mMuteYoukuViewController?.videoView?.layoutParams?.height ?: DisplayUtil.getDisplayWidthInPx() * 9 / 16
                            }
                        }
                    }
                    onItemClickListener = this@MovieTrailerItem
                }
            }
            val list = mutableListOf<RecyclerItem<ShowMo>>()
            data.shows?.forEach { list.add(listAdapter?.convert(it) as RecyclerItem<ShowMo>) }
            listAdapter?.submitList(list)
            changeCurrentIndex(0)
        }
    }

    override fun onItemClick(itemView: View?, position: Int, adapter: CustomListAdapter<RecyclerItem<ShowMo>, CustomViewHolder<RecyclerItem<ShowMo>>>?) {
        itemView?.context?.let { clickItem(it, position, adapter) }
    }

    override fun onItemChildClick(itemChild: View?, position: Int, adapter: CustomListAdapter<RecyclerItem<ShowMo>, CustomViewHolder<RecyclerItem<ShowMo>>>?) {
        if (itemChild?.id == R.id.card_movie_trailer) {
            clickItem(itemChild.context, position, adapter)
        }
    }

    private fun clickItem(context: Context, position: Int, adapter: CustomListAdapter<RecyclerItem<ShowMo>, CustomViewHolder<RecyclerItem<ShowMo>>>?) {
        adapter?.let {
            val itemData = it.getItem(position)
            if (itemData.tag is Boolean && itemData.tag == true) {
                //reselect
                val bundle = Bundle()
                bundle.putSerializable(IntentConstants.KEY_SHOW_MO, itemData.listData)
                bundle.putBoolean(IntentConstants.KEY_SHOWING, true)
                bundle.putString(IntentConstants.KEY_SHOWID, itemData.listData.id)
                MovieNavigator.gotoAction(context, IntentConstants.Action.ACTION_SHOWDETAIL, bundle)
                DogCat.getInstance().click()
                        .ctrlName("TrailerModuleMovieClick")
                        .param("show_id", itemData.listData.id)
                        .param("is_switch", "0")
                        .commit()
            } else {
                //select
                viewHolder.mMuteYoukuViewController?.doStop(true)
                changeCurrentIndex(position)
                bindVideo(viewHolder)
                videoFeedPreviewManager?.onStopScroll()
                DogCat.getInstance().click()
                        .ctrlName("TrailerModuleMovieClick")
                        .param("show_id", itemData.listData.id)
                        .param("is_switch", "1")
                        .commit()
            }
        }
    }

    private fun changeCurrentIndex(index: Int) {
        listAdapter?.let {
            val size = it.itemCount
            if (index >= size) {
                return
            }
            for (recyclerItem in it.currentList) {
                recyclerItem.tag = false
            }
            it.getItem(index).tag = true

            currentSelectItem = index
            it.notifyDataSetChanged()
            //it.recyclerView?.smoothScrollToPosition(currentSelectItem)
            it.recyclerView?.findViewHolderForLayoutPosition(index)?.let { holder ->
                holder.itemView.apply {
                    it.recyclerView?.layoutManager?.let { layoutManager ->
                        snapHelper.calculateDistanceToFinalSnap(layoutManager, this)
                    }?.let { disArr ->
                        it.recyclerView?.smoothScrollBy(disArr[0], disArr[1])
                    }
                }
            }
        }
    }

    private fun getCurrentVideoMo(): SmartVideoMo? {
        var smartVideoMo: SmartVideoMo? = null
        listAdapter?.let {
            val size = it.itemCount
            if (currentSelectItem >= size) {
                return@let
            }
            smartVideoMo = it.getItem(currentSelectItem).listData.firstVideoMo
        }
        if (listAdapter == null && smartVideoMo == null) {
            data.shows?.let {
                smartVideoMo = it[0].firstVideoMo
            }
        }
        return smartVideoMo
    }

    private fun hasNextVideoMo(): Boolean {
        listAdapter?.let {
            val size = it.itemCount
            if (currentSelectItem < size - 1) {
                return true
            }
        }
        return false
    }

    fun canPlayCurrentVideo(): Boolean {
//        val linearLayout: LinearLayoutManager = viewHolder.movieList?.layoutManager as LinearLayoutManager
//        val firstVisibleItemPosition = linearLayout.findFirstCompletelyVisibleItemPosition()
//        var lastVisibleItemPosition = linearLayout.findLastCompletelyVisibleItemPosition()
//        if (currentSelectItem >= firstVisibleItemPosition && currentSelectItem <= lastVisibleItemPosition) {
//            return true
//        }
        return true
    }

    override fun getLayoutId(): Int {
        return R.layout.item_home_movie_trailer
    }

    class MovieTrailerViewHolder(itemView: View) : CustomRecyclerViewHolder<HomeFeedRecommendDividerItem>(itemView) {

        var title: TextView? = findViewById(R.id.tv_home_movie_trailer_title)
        var linkTxt: TextView? = findViewById(R.id.tv_home_movie_trailer_link)
        var linkLayout: LinearLayout? = findViewById(R.id.layout_home_movie_trailer_right)
        var movieList: RecyclerView? = findViewById(R.id.list_home_movie_trailer)
        var mVideoContainer: FrameLayout? = findViewById(R.id.fl_video_container)
        var mMuteYoukuViewController : MuteYoukuViewController? = null

        init {

            RippleUtils.setupRipple(linkLayout, ShapeBuilder.create()
                    .radius(DisplayUtil.dp2px(18f))
                    .solid(ResHelper.getColor(R.color.white))
                    .stroke(DisplayUtil.dp2pxInt(0.5f), ResHelper.getColor(R.color.common_image_bg_color))
                    .build())

            //ShapeBuilder.create().gradient(GradientDrawable.Orientation.BOTTOM_TOP, Color.BLACK, ResHelper.getColor(R.color.transparent_black_07)).build(overlay)

            val layoutParams = itemView.layoutParams
            if (layoutParams is StaggeredGridLayoutManager.LayoutParams) {
                layoutParams.isFullSpan = true
                itemView.layoutParams = layoutParams
            }
        }
    }

}