package com.taobao.movie.android.app.oscar.ui.homepage.item.preview

import android.graphics.Color
import android.graphics.Typeface
import android.graphics.drawable.GradientDrawable
import android.os.Bundle
import android.support.v4.content.res.ResourcesCompat
import android.text.TextUtils
import android.view.View
import android.view.ViewGroup
import com.taobao.movie.android.app.oscar.biz.util.OscarBizUtil
import com.taobao.movie.android.app.ui.common.WantedTipUtil
import com.taobao.movie.android.arch.items.*
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.Region.RegionBizServiceImpl
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.common.scheme.NavigatorUtil
import com.taobao.movie.android.commonui.component.BaseActivity
import com.taobao.movie.android.commonui.utils.ButtonStyleHelper
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.model.ShowResultIndexMo
import com.taobao.movie.android.integration.oscar.service.OscarExtService
import com.taobao.movie.android.net.listener.MtopResultListener
import com.taobao.movie.android.sdk.infrastructure.utils.ContextUtil
import com.taobao.movie.android.ut.DogCat
import com.taobao.movie.appinfo.util.*
import com.taobao.movie.shawshank.service.ShawshankServiceManager
import com.taobao.movie.shawshank.time.ServerTime
import kotlinx.android.synthetic.main.item_home_movie_trailer_list.view.*
import java.text.DecimalFormat
import java.util.*

/**
 * @see {HomepageSoonShowItem}
 */
class MovieTrailerListAdapter : CustomListAdapter<RecyclerItem<ShowMo>, MovieTrailerListAdapter.MovieTrailerListViewHolder>(RecyclerItem.DiffCallback()), ItemDataConverter<ShowMo> {

    override fun convert(data: ShowMo): ItemData<ShowMo> {
        return RecyclerItem(data, data.id.toInt(), R.layout.item_home_movie_trailer_list)
    }

    override fun createCustomViewHolder(view: View?, parent: ViewGroup, viewType: Int): MovieTrailerListViewHolder? {
        return if (view != null) MovieTrailerListViewHolder(view, this) else null
    }

    class MovieTrailerListViewHolder(itemView: View, adapter: MovieTrailerListAdapter) : CustomViewHolder<RecyclerItem<ShowMo>>(itemView) {

        private val oscarExtService: OscarExtService? = ShawshankServiceManager.getShawshankService(OscarExtService::class.java.name)

        private val wantedTipUtil = WantedTipUtil(ContextUtil.getActivity(itemView))

        private val wantListener = object : MtopResultListener<ShowResultIndexMo> {
            override fun onPreExecute() {
                val activity: BaseActivity? = ContextUtil.getActivity(itemView)
                activity?.showProgressDialog("", true)
            }

            override fun hitCache(hit: Boolean, model: ShowResultIndexMo?) {
            }

            override fun onSuccess(model: ShowResultIndexMo?) {
                val activity: BaseActivity? = ContextUtil.getActivity(itemView)
                activity?.apply {
                    if (UiUtils.isActivityStateValid(this)) {
                        dismissProgressDialog()
                        val itemData = adapter.getItem(layoutPosition)
                        itemData.listData.userShowStatus = model?.status
                        if (model?.status == 0) {
                            itemData.listData.wantCount -= 1
                            itemData.listData.scoreAndFavor?.apply {
                                if (favorCount != null) {
                                    favorCount -= 1
                                }
                            }

                            adapter.notifyItemChanged(adapterPosition)
                        } else if (model?.status == 1) {
                            itemData.listData.wantCount += 1
                            itemData.listData.scoreAndFavor?.apply {
                                if (favorCount != null) {
                                    favorCount += 1
                                }
                            }

                            adapter.notifyItemChanged(adapterPosition)
                            wantedTipUtil.showWantedTip(true, model, itemData.listData.id)
                        }
                    }
                }

            }

            override fun onFail(resultCode: Int, returnCode: Int, returnMessage: String?) {
                val activity: BaseActivity? = ContextUtil.getActivity(itemView)
                activity?.apply {
                    dismissProgressDialog()
                    ToastUtil.showToast(getString(R.string.error_system_failure))
                }
            }

        }

        override fun onCreate() {
            itemView.overlay_home_movie_trailer_item.background = GradientDrawable().apply {
                colors = intArrayOf(Color.BLACK, Color.TRANSPARENT)
                orientation = GradientDrawable.Orientation.BOTTOM_TOP
            }
            itemView.overlay_home_movie_trailer_border.background = GradientDrawable().apply {
                cornerRadius = DisplayUtil.dp2px(6f)
                setStroke(DisplayUtil.dp2pxInt(2f), Color.WHITE)
            }
            postClick(itemView.card_movie_trailer)
        }

        override fun onBindItem(position: Int, itemData: RecyclerItem<ShowMo>) {
            if (itemData.tag is Boolean && itemData.tag == true) {
                itemView.overlay_home_movie_trailer_border.visibility = View.VISIBLE
            } else {
                itemView.overlay_home_movie_trailer_border.visibility = View.GONE
            }
            itemView.img_movie_trailer_show_image.url = itemData.listData.poster
            itemView.tv_movie_trailer_show_name.text = itemData.listData.showName
            itemView.tv_movie_trailer_show_open_time.text = getUpcomingShowDate(itemData.listData)
            itemView.layout_movie_trailer_remark_or_want.visibility = View.GONE
            DogCat.getInstance().exposure(itemView.img_movie_trailer_show_image)
                    .block("TrailerModuleMovieExpose")
                    .viewId(itemData.listData.id.toString())
                    .param("show_id", itemData.listData.id)
                    .commit()
            itemView.btn_movie_trailer_action.apply {
                when (itemData.listData.soldType) {
                    ShowMo.SOLD_TYPE_PRE -> {
                        text = "预售"
                        ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_PRE_SALE)
                        setOnClickListener {
                            Bundle().apply {
                                putString(IntentConstants.KEY_OSCAR_MOVIE_ID, itemData.listData.id)
                                putString(IntentConstants.KEY_OSCAR_MOVIE_NAME, itemData.listData.showName)
                                MovieNavigator.gotoAction(context, IntentConstants.Action.ACTION_CINEMALIST, this)
                            }
                        }
                    }
                    ShowMo.SOLD_TYPE_NORMAL -> {
                        text = "购票"
                        ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_BUY)
                        setOnClickListener {
                            Bundle().apply {
                                putString(IntentConstants.KEY_OSCAR_MOVIE_ID, itemData.listData.id)
                                putString(IntentConstants.KEY_OSCAR_MOVIE_NAME, itemData.listData.showName)
                                MovieNavigator.gotoAction(context, IntentConstants.Action.ACTION_CINEMALIST, this)
                            }
                            DogCat.getInstance().click()
                                    .ctrlName("TrailerModuleBuyButtonClick")
                                    .param("show_id", itemData.listData.id)
                                    .commit()
                        }
                    }
                    ShowMo.SOLD_TYPE_VOD -> {
                        text = "看正片"
                        ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_WATCH_FILM)
                        setOnClickListener {
                            NavigatorUtil.gotoVideoList(context, itemData.listData.id)
                        }
                    }
                    else -> {
                        setOnClickListener {
                            oscarExtService?.changeShowWantStatus(hashCode(), itemData.listData.id, itemData.listData.getUserShowStatus(), RegionBizServiceImpl.getUserRegion().cityCode, wantListener)
                            DogCat.getInstance().click().ctrlName("WantToSeeClick")
                                    .param("position", "trailermodule")
                                    .param("show_id", itemData.listData.id)
                                    .param("status", itemData.listData.userShowStatus.toString())
                                    .commit()
                        }
                        when (itemData.listData.userShowStatus) {
                            0 -> {
                                text = "想看"
                                ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_WANT)
                            }
                            2 -> {
                                text = "已看过"
                                ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_HAS_WATCH)
                                setOnClickListener(null)// 已看过不可点
                            }
                            else -> {
                                text = "已想看"
                                ButtonStyleHelper.setButtonStyle(this, ButtonStyleHelper.ButtonStyleType.TYPE_WANT_DONE)
                            }
                        }
                    }
                }
            }

            if (DateUtil.compareDate(itemData.listData.getOpenDay(), ServerTime.getCurrentServerTime())) { //未上映
                itemData.listData.scoreAndFavor?.score?.score?.apply {
                    if (this > 0) {
                        itemView.layout_movie_trailer_remark_or_want.visibility = View.VISIBLE
                        itemView.tv_movie_trailer_remark_or_want_left.text = "${itemData.listData.scoreAndFavor?.score?.scoreName ?: "淘票票评分"}"
                        itemView.tv_movie_trailer_remark_or_want_right.text = "${DecimalFormat("0.0").format(OscarBizUtil.remarkRule5_1(this))}"
                        itemView.tv_movie_trailer_remark_or_want_left.typeface = Typeface.DEFAULT
                        itemView.tv_movie_trailer_remark_or_want_right.typeface = ResourcesCompat.getFont(itemView.context, R.font.rubik_medium)
                    }
                }
                itemData.listData.scoreAndFavor?.favorCount?.apply {
                    if (this > 0) {
                        itemView.layout_movie_trailer_remark_or_want.visibility = View.VISIBLE
                        itemView.tv_movie_trailer_remark_or_want_left.text = "${DataUtil.getCount4S5RFormatV9(this).get(0)}${DataUtil.getCount4S5RFormatV9(this).get(1)}"
                        itemView.tv_movie_trailer_remark_or_want_right.text = "${ResHelper.getString(com.taobao.movie.android.business.R.string.want_title)}"
                        itemView.tv_movie_trailer_remark_or_want_left.typeface = ResourcesCompat.getFont(itemView.context, R.font.rubik_medium)
                        itemView.tv_movie_trailer_remark_or_want_right.typeface = Typeface.DEFAULT
                    }
                }
            } else {
                itemData.listData.scoreAndFavor?.score?.score?.apply {
                    if (this > 0) {
                        itemView.layout_movie_trailer_remark_or_want.visibility = View.VISIBLE
                        itemView.tv_movie_trailer_remark_or_want_left.text = "${itemData.listData.scoreAndFavor?.score?.scoreName ?: "淘票票评分"}"
                        itemView.tv_movie_trailer_remark_or_want_right.text = "${DecimalFormat("0.0").format(OscarBizUtil.remarkRule5_1(this))}"
                        itemView.tv_movie_trailer_remark_or_want_left.typeface = Typeface.DEFAULT
                        itemView.tv_movie_trailer_remark_or_want_right.typeface = ResourcesCompat.getFont(itemView.context, R.font.rubik_medium)
                    }
                } ?: itemData.listData.scoreAndFavor?.score?.scoreName?.let {
                    itemView.layout_movie_trailer_remark_or_want.visibility = View.VISIBLE
                    itemView.tv_movie_trailer_remark_or_want_left.text = it
                    itemView.tv_movie_trailer_remark_or_want_left.typeface = Typeface.DEFAULT
                    itemView.tv_movie_trailer_remark_or_want_right.text = ""
                }
            }
        }

        private fun getUpcomingShowDate(showMo: ShowMo): String? {
            val dateString = if (!TextUtils.isEmpty(showMo.openTime)) showMo.openTime else showMo.openDay
            val sb = StringBuilder()
            if (DateUtil.isValidDateYMD(dateString) && dateString.length >= 10) {
                val showTimes = dateString.split("-").toTypedArray()
                if (showTimes.size < 3) {
                    return null
                }
                val c = Calendar.getInstance()
                c.time = Date(ServerTime.getCurrentServerTime())
                val year = c[Calendar.YEAR]
                if (!TextUtils.equals(showTimes[0], "" + year)) {
                    sb.append(showTimes[0].substring(2, 4)).append("年")
                }
                sb.append(showMo.getNoZeroPrefix(showTimes[1])).append("月")
                if (showTimes[2].length > 2) {
                    sb.append(showMo.getNoZeroPrefix(showTimes[2].split(" ").toTypedArray()[0]))
                } else {
                    sb.append(showMo.getNoZeroPrefix(showTimes[2]))
                }
                sb.append("日")
            } else if (DateUtil.isValidDateYM(dateString) && dateString.length >= 7) {
                val showTimes = dateString.split("-").toTypedArray()
                if (showTimes.size < 2) {
                    return null
                }
                val c = Calendar.getInstance()
                c.time = Date(ServerTime.getCurrentServerTime())
                val year = c[Calendar.YEAR]
                if (!TextUtils.equals(showTimes[0], "" + year)) {
                    sb.append(showTimes[0].substring(2, 4)).append("年")
                }
                sb.append(showMo.getNoZeroPrefix(showTimes[1])).append("月").append("待定")
            } else {
                return ""
            }
            return sb.toString() + "上映"
        }
    }

}