package com.taobao.movie.android.app.oscar.ui.homepage.tab

import android.os.Handler
import android.os.Looper
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.StaggeredGridLayoutManager
import com.taobao.listitem.recycle.CustomRecyclerAdapter
import com.taobao.movie.android.app.oscar.ui.homepage.fragment.FeedListBaseFragment
import com.taobao.movie.android.app.oscar.ui.homepage.item.HomeFeedRecommendDividerItem
import com.taobao.movie.android.arch.FragmentModule
import com.taobao.movie.android.commonui.component.lcee.ScrollableView
import com.taobao.movie.android.ut.DogCat

/**
 * 双击底部 tab 的滑动 Module
 *
 * @author yidong.yyd
 * @since 2019-12-23
 */
class FeedTypeScrollModuleImpl(fragment: FeedListBaseFragment, private val recyclerView: RecyclerView)
    : FragmentModule<FeedListBaseFragment>(fragment), ScrollableView {

    companion object {
        /**
         * Returns the adapter position of the first visible view.
         * This position does not include adapter changes that were dispatched after the last layout pass.
         */
        fun findFirstVisibleItemPosition(recyclerView: RecyclerView): Int {
            return when (recyclerView.layoutManager) {
                is StaggeredGridLayoutManager -> findMin((recyclerView.layoutManager as StaggeredGridLayoutManager).findFirstVisibleItemPositions(null))
                is LinearLayoutManager -> (recyclerView.layoutManager as LinearLayoutManager).findFirstVisibleItemPosition()
                else -> -1
            }
        }

        /**
         * Returns the adapter position of the first fully visible view.
         * This position does not include adapter changes that were dispatched after the last layout pass.
         */
        fun findFirstCompletelyVisibleItemPosition(recyclerView: RecyclerView): Int {
            return when (recyclerView.layoutManager) {
                is StaggeredGridLayoutManager -> findMin((recyclerView.layoutManager as StaggeredGridLayoutManager).findFirstCompletelyVisibleItemPositions(null))
                is LinearLayoutManager -> (recyclerView.layoutManager as LinearLayoutManager).findFirstCompletelyVisibleItemPosition()
                else -> -1
            }
        }


        /**
         * Returns the adapter position of the last visible view.
         * This position does not include adapter changes that were dispatched after the last layout pass.
         */
        fun findLastVisibleItemPosition(recyclerView: RecyclerView): Int {
            return when (recyclerView.layoutManager) {
                is StaggeredGridLayoutManager -> findMin((recyclerView.layoutManager as StaggeredGridLayoutManager).findLastVisibleItemPositions(null))
                is LinearLayoutManager -> (recyclerView.layoutManager as LinearLayoutManager).findLastVisibleItemPosition()
                else -> -1
            }
        }

        /**
         * 找到最小的
         */
        private fun findMin(array: IntArray): Int = if (array.isEmpty()) {
            -1
        } else {
            array.sort()
            array[0]
        }
    }

    /* handler 从 0 滑到 dividerIndex 的时候滑不到固定位置，所以再在 post 一下 */
    val handler = Handler(Looper.getMainLooper())

    override fun scrollToTop() {
        val localAdapter = if (recyclerView.adapter is CustomRecyclerAdapter) (recyclerView.adapter as CustomRecyclerAdapter) else return
        var dividerIndex = localAdapter.indexOfItem(HomeFeedRecommendDividerItem::class.java)

        val first = findFirstVisibleItemPosition(recyclerView)
        val firstComplete = findFirstCompletelyVisibleItemPosition(recyclerView)
        val last = findLastVisibleItemPosition(recyclerView)

        handler.removeCallbacksAndMessages(null)
        if (dividerIndex == -1 || first == -1 || last == -1) {
            recyclerView.smoothScrollToPosition(0)
        } else if (first >= dividerIndex) {
            recyclerView.smoothScrollToPosition(0)
        } else if (dividerIndex in first until last) {
            localAdapter.getItem(dividerIndex)?.apply {
                recyclerView.smoothScrollBy(0, recycleViewHolder.itemView.top)
            }
        } else if (first == firstComplete && first == 0) {
            if (last >= dividerIndex) {
                localAdapter.getItem(dividerIndex)?.apply {
                    recyclerView.smoothScrollBy(0, recycleViewHolder.itemView.top)
                }
            } else {
                recyclerView.scrollToPosition(dividerIndex)
                // 从 0 滑到 dividerIndex 的时候滑不到固定位置，所以再在 post 一下
                localAdapter.getItem(dividerIndex)?.apply {
                    handler.post { recyclerView.smoothScrollBy(0, recycleViewHolder?.itemView?.top ?: 0) }
                }
            }
        } else {
            recyclerView.smoothScrollToPosition(0)
        }
        DogCat.getInstance().click().ctrlName("HomepageFeedShortcutClick").commit()
    }
}