package com.taobao.movie.android.app.oscar.ui.homepage.tab

import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.support.v4.content.res.ResourcesCompat
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.View
import android.view.ViewGroup
import android.view.ViewTreeObserver
import android.widget.ImageView
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerAdapter
import com.taobao.movie.android.app.oscar.ui.homepage.fragment.HomePageListFragment
import com.taobao.movie.android.app.oscar.ui.homepage.util.HomeLottieSwitchHelper
import com.taobao.movie.android.app.oscar.ui.widget.SafeLottieAnimationView
import com.taobao.movie.android.arch.FragmentModule
import com.taobao.movie.android.commonui.utils.ImmersionStatusBar
import com.taobao.movie.android.commonui.widget.Appbar
import com.taobao.movie.android.commonui.widget.MaterialTabLayout
import com.taobao.movie.android.commonutil.kotlin.isTrue
import com.taobao.movie.android.commonutil.kotlin.parseRGB
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.orange.ConfigUtil
import com.taobao.movie.android.integration.orange.OrangeConstants
import com.taobao.movie.android.integration.oscar.uiInfo.PositionTab
import com.taobao.movie.android.integration.skin.model.HomeThemeMo
import com.taobao.movie.android.sdk.infrastructure.glide.GlideApp
import com.taobao.movie.android.sdk.infrastructure.monitor.business.MonitorPointConstant
import com.taobao.movie.appinfo.util.CDNHelper
import com.taobao.movie.appinfo.util.DateUtil
import com.taobao.movie.appinfo.util.LogUtil

class HomeTopThemeModuleImpl(fragment: HomePageListFragment) : FragmentModule<HomePageListFragment>(fragment), HomeTopThemeModule {

    private var topThemeLayout: ViewGroup? = null
    private var topThemeImage: ImageView? = null
    private var topThemeAnimImage: SafeLottieAnimationView? = null
    private var topThemeNewcomerImage: ImageView? = null

    private var appbar: Appbar? = null
    private var tabLayout: MaterialTabLayout? = null
    /* 是否使用 lottie */
    private var useLottie = true

    override fun onViewCreated() {
        topThemeLayout = findViewById(R.id.layoutTopTheme)
        topThemeImage = findViewById(R.id.imgTheme)
        topThemeAnimImage = findViewById(R.id.imgAnim)
        topThemeAnimImage?.setLottieStatisticScene(MonitorPointConstant.SCENE_HOME_TOP)

        appbar = findViewById(R.id.title_bar)
        tabLayout = findViewById(R.id.tabLayout)
    }

    override fun setHasTabs(hasTabs: Boolean) {
        // 放这里是因为确保数据刷新的时候能够更新上 useLottie 这个值，放 setTheme 的话 onTabSelected 中会被调用，可能导致没有下拉刷新的情况下两次点击tab呈现效果不一样
        useLottie = HomeLottieSwitchHelper.instance.isHomeThemeBannerLottie()
        topThemeLayout = findViewById(R.id.layoutTopTheme)
        topThemeImage = findViewById(R.id.imgTheme)
        topThemeAnimImage = findViewById(R.id.imgAnim)
        topThemeAnimImage?.setLottieStatisticScene(MonitorPointConstant.SCENE_HOME_TOP)

        topThemeNewcomerImage = findViewById(R.id.iv_theme_bar_newcomer)

        appbar = findViewById(R.id.title_bar)
        tabLayout = findViewById(R.id.tabLayout)

        topThemeLayout?.viewTreeObserver?.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                var h = 0
                val layoutParam = topThemeLayout?.layoutParams
                if (hasTabs && tabLayout?.visibility == View.VISIBLE) {
                    if (tabLayout?.measuredHeight!! == 0) {
                        return
                    }
                    h += tabLayout?.measuredHeight!!
                }
                if (appbar?.visibility == View.VISIBLE && appbar?.measuredHeight!! > 0) {
                    h += appbar?.measuredHeight!!
                    layoutParam?.height = h
                    topThemeLayout?.layoutParams = layoutParam
                    topThemeLayout?.viewTreeObserver?.removeOnGlobalLayoutListener(this)
                }
            }
        })
    }

    override fun setAlpha(alpha: Float) {
        topThemeLayout?.alpha = alpha
    }

    override fun setTheme(@Appbar.Theme theme: Int, tab: PositionTab?, isNewcomer: Boolean) {
        LogUtil.w("HomeTopTheme", "theme ${theme} tab=${tab}")
        appbar?.setTheme(theme)
        tabLayout?.apply {
            for (i in 0 until tabCount) {
                getTabAt(i)?.customView?.findViewById<TextView>(R.id.txt)?.setTextColor(ResourcesCompat.getColorStateList(resources,
                        if (theme == Appbar.THEME_DARK) R.color.color_tab_home_dark_theme else R.color.color_tab_home, null))
            }
        }

        topThemeNewcomerImage?.visibility = View.GONE
        if (handleNewcomer(theme != Appbar.THEME_DARK, isNewcomer)) {
            return
        }

        if (theme == Appbar.THEME_DARK) {
            topThemeLayout?.setBackgroundColor(Color.TRANSPARENT)
            tabLayout?.setSelectedTabIndicatorColor(Color.WHITE)
            topThemeNewcomerImage?.visibility = View.GONE
            val url = tab?.backgroundLottieUrl
            if (!TextUtils.isEmpty(url) && useLottie) {
                if (topThemeAnimImage?.tag !is String || !TextUtils.equals(url, topThemeAnimImage?.tag as String)) {
                    topThemeAnimImage?.setAnimationFromUrl(url)
                    topThemeAnimImage?.tag = url
                    topThemeAnimImage?.setOnLottieFailListener { topThemeAnimImage?.visibility = View.GONE }
                }
                topThemeAnimImage?.playAnimation()
                topThemeAnimImage?.visibility = View.VISIBLE
            } else {
                if (topThemeAnimImage?.isAnimating.isTrue()) {
                    topThemeAnimImage?.cancelAnimation()
                }
                topThemeAnimImage?.visibility = View.GONE
            }

            if (!TextUtils.isEmpty(tab?.backgroundImageUrl)) {
                topThemeImage?.let {
                    it.visibility = View.VISIBLE
                    GlideApp.with(fragment).load(context?.let { it1 -> CDNHelper.getInstance().autoFix(it1, tab?.backgroundImageUrl) }).into(it)
                }
            } else {
                topThemeImage?.visibility = View.GONE
            }

            if (!TextUtils.isEmpty(tab?.backgroundGradualColor)) {
                tab?.backgroundGradualColor?.split("-")?.apply {
                    if (size > 1) {
                        try {
                            val colorStart = this[0].parseRGB()
                            val colorEnd = this[1].parseRGB()
                            topThemeLayout?.background = GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, intArrayOf(colorStart, colorEnd))
                        } catch (e: Exception) {
                            e.printStackTrace()
                            topThemeLayout?.setBackgroundColor(if (TextUtils.isEmpty(tab?.backgroundImageUrl)) Color.TRANSPARENT else Color.WHITE)
                        }
                    }
                }
            } else if (!TextUtils.isEmpty(tab?.backgroundColor)) {
                tab?.backgroundColor?.parseRGB()?.let { topThemeLayout?.setBackgroundColor(it) }
            } else {
                topThemeLayout?.setBackgroundColor(if (TextUtils.isEmpty(tab?.backgroundImageUrl)) Color.TRANSPARENT else Color.WHITE)
            }

            ImmersionStatusBar.statusBarDark(fragment, false)
        } else {
            topThemeLayout?.setBackgroundColor(Color.WHITE)
            tabLayout?.setSelectedTabIndicatorColor(0xFFFF2E62.toInt())
            topThemeImage?.visibility = View.GONE
            topThemeAnimImage?.visibility = View.GONE
            ImmersionStatusBar.statusBarDark(fragment, true)
        }
    }

    /**
     * 处理新人模式
     */
    private fun handleNewcomer(isDarkFont: Boolean, isNewcomer: Boolean): Boolean {
        // 新人，直接干掉其他的
        if (isNewcomer) {
            val homeThemeMo = ConfigUtil.getConfigCenterObj(HomeThemeMo::class.java,
                    OrangeConstants.CONFIG_KEY_NEW_COMER_NEW_YEAR_SKIN_PERIOD)
            if (DateUtil.isTimeValid(homeThemeMo?.startDate ?: "", homeThemeMo?.endDate ?: "")) {
                topThemeNewcomerImage?.setImageResource(R.drawable.bg_newcomer_bar_flower)
                topThemeNewcomerImage?.visibility = View.VISIBLE
                topThemeLayout?.background = GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, intArrayOf(-0x1fbec9, -0x1fbec9))
            } else {
                topThemeLayout?.background = GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, intArrayOf(-0x8b6d, -0x8b6d))
            }
            topThemeImage?.visibility = View.GONE
            topThemeAnimImage?.visibility = View.GONE
            ImmersionStatusBar.statusBarDark(fragment, isDarkFont)
            return true
        }
        return false
    }

    override fun getOnScrollListener(): RecyclerView.OnScrollListener = scrollListener

    /**
     * RecyclerView 滑动状态监听
     */
    private val scrollListener: RecyclerView.OnScrollListener = object : RecyclerView.OnScrollListener() {
        override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
            if (recyclerView.adapter is CustomRecyclerAdapter && topThemeAnimImage != null && topThemeAnimImage?.visibility == View.VISIBLE) {
                if (newState != RecyclerView.SCROLL_STATE_IDLE && topThemeAnimImage?.isAnimating.isTrue()) {
                    topThemeAnimImage?.pauseAnimation()
                }
                if (newState == RecyclerView.SCROLL_STATE_IDLE && !topThemeAnimImage?.isAnimating.isTrue()) {
                    topThemeAnimImage?.playAnimation()
                }
            }
        }

        override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {}
    }
}