package com.taobao.movie.android.app.oscar.ui.homepage.tab

import android.arch.lifecycle.Lifecycle
import android.graphics.Color
import android.graphics.drawable.Drawable
import android.support.v4.app.Fragment
import android.support.v4.content.res.ResourcesCompat
import android.support.v4.view.ViewCompat
import android.support.v4.view.ViewPager
import android.support.v4.view.ViewPager.OnPageChangeListener
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import com.taobao.movie.android.app.oscar.ui.homepage.adapter.HomePagerAdapter
import com.taobao.movie.android.app.oscar.ui.homepage.fragment.HomePageListFragment
import com.taobao.movie.android.arch.FragmentModule
import com.taobao.movie.android.commonui.component.BaseFragment
import com.taobao.movie.android.commonui.component.lcee.RefreshView
import com.taobao.movie.android.commonui.widget.Appbar
import com.taobao.movie.android.commonui.widget.MaterialTabLayout
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.commonutil.kotlin.isTrue
import com.taobao.movie.android.commonutil.kotlin.toNumber
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.CommonConstants
import com.taobao.movie.android.integration.oscar.uiInfo.HomepageVO
import com.taobao.movie.android.integration.oscar.uiInfo.PositionTab
import com.taobao.movie.android.sdk.infrastructure.glide.GlideApp
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTConstants
import com.taobao.movie.android.ut.DogCat
import com.taobao.movie.appinfo.util.*
import kotlinx.android.synthetic.main.home_page_fragment.*

class PagerModuleImpl(fragment: HomePageListFragment) : FragmentModule<HomePageListFragment>(fragment), PagerModule {

    private var pagerAdapter: HomePagerAdapter? = null
    private val marginTop: Int = DisplayUtil.dp2pxInt(44f)
    private var onTabSelectedListener: MaterialTabLayout.OnTabSelectedListener? = null
    private var lastIndex = -1

    companion object {
        const val TAB_COUNT = 4
    }

    override fun onReceiveData(data: Any) {
        if (data is HomepageVO) {
            setupPager(data)
        }
    }

    private fun setupPager(homepageVO: HomepageVO) {

        fragment.homePager?.setScrollable(true)

        // 如果是新人，直接把其他的干掉只保留精选
        val isNewcomer = isNewcomer(homepageVO)
        val needShowNewcomer = needShowNewcomer(homepageVO)
        if (isNewcomer) {
            homepageVO.tabs?.firstOrNull { it.type == PositionTab.TYPE_RECOMMEND }
                    ?.let { homepageVO.tabs = listOf(it) }
                    ?: run { homepageVO.tabs = listOf(PositionTab().apply { type = 3 }) }
        }

        if (DataUtil.isNullOrEmpty(homepageVO.tabs)) {
            homepageVO.tabs = listOf(PositionTab().apply { type = 3 })
        }

        if (homepageVO.tabs?.find { it.type == PositionTab.TYPE_RECOMMEND } == null){
            homepageVO.tabs?.add(0, PositionTab().apply {
                type = 3
                name = "精选"
            })
        }


        if (pagerAdapter == null) {
            pagerAdapter = HomePagerAdapter(fragment.childFragmentManager, homepageVO)
            fragment.homePager?.adapter = pagerAdapter
        } else {
            pagerAdapter!!.notifyDataSetChanged(homepageVO)
        }

        val tabCounts = homepageVO.tabs?.size ?: 0
        val hasMultiTabs = tabCounts > 1 && !isNewcomer
        setPagerTopMargin(hasMultiTabs)

        val homeTopThemeModule = getModule(HomeTopThemeModule::class.java)
        homeTopThemeModule?.setHasTabs(hasMultiTabs)

        fragment.tabLayout.removeAllBadges()

        if (hasMultiTabs) {
            fragment.tabLayout.visibility = View.VISIBLE
            fragment.tabLayout.setupWithViewPager(fragment.homePager)
            fragment.homePager.offscreenPageLimit = if (tabCounts > TAB_COUNT) TAB_COUNT else tabCounts
            for (i in 0 until fragment.tabLayout.tabCount) {
                fragment.tabLayout.getTabAt(i)?.apply {
                    setCustomView(R.layout.tab_home_top_scroll)
                }
                val positionTab = homepageVO.tabs?.get(i)
                if (positionTab?.activeImage != null && positionTab.image != null) {
                    fragment.tabLayout.getTabAt(i)?.customView?.findViewById<ImageView>(R.id.img)?.apply {
                        visibility = View.VISIBLE
                        val count = fragment.tabLayout.tabCount
                        val imageView = this
                        GlideApp.with(this).load(fragment.context?.let { CDNHelper.getInstance().autoFix(it, if (i == fragment.homePager.currentItem) positionTab.activeImage else positionTab.image) })
                                .listener(object : RequestListener<Drawable> {
                                    override fun onLoadFailed(e: GlideException?, model: Any?, target: Target<Drawable>?, isFirstResource: Boolean): Boolean {
                                        return false
                                    }

                                    override fun onResourceReady(resource: Drawable?, model: Any?, target: Target<Drawable>?, dataSource: DataSource?, isFirstResource: Boolean): Boolean {
                                        post {
                                            val tagView = fragment.tabLayout.getTabAt(i)?.customView?.findViewById<TextView>(R.id.tvSubscript)?.tag
                                            if (tagView is TextView) {
                                                val viewWidth = imageView.width
                                                val itemWidth = DisplayUtil.getDisplayWidthInPx() / count
                                                val marginLeft = itemWidth * (i) + (itemWidth / 2 + viewWidth / 2) - DisplayUtil.dp2pxInt(8F)
                                                tagView.x = marginLeft.toFloat()
                                            }
                                        }
                                        return false
                                    }
                                })
                                .into(this)
                    }
                    fragment.tabLayout.getTabAt(i)?.customView?.findViewById<TextView>(R.id.txt)?.apply {
                        visibility = View.GONE
                    }
                } else if (!TextUtils.isEmpty(positionTab?.name)) {
                    fragment.tabLayout.getTabAt(i)?.customView?.findViewById<ImageView>(R.id.img)?.apply {
                        visibility = View.GONE
                    }
                    fragment.tabLayout.getTabAt(i)?.customView?.findViewById<TextView>(R.id.txt)?.apply {
                        visibility = View.VISIBLE
                        text = positionTab?.name
                        textSize = if (i == fragment.homePager.currentItem) 18f else 16f
                        paint.isFakeBoldText = i == fragment.homePager.currentItem
                        setTextColor(ResourcesCompat.getColorStateList(resources, if (getTheme(positionTab) == Appbar.THEME_DARK) R.color.color_tab_home_dark_theme else R.color.color_tab_home, null))
                    }
                }
                if (i != 0 && !TextUtils.isEmpty(positionTab?.superscript)) {
                    MovieCacheSet.getInstance().getString(CommonConstants.BADGE_TAB_HOME_SHOW + "_" + positionTab?.id).let {
                        if (!TextUtils.equals(it, positionTab?.superscript)) {
                            fragment.tabLayout.getTabAt(i)?.customBadgeView = TextView(fragment.context).apply {
                                gravity = Gravity.CENTER
                                maxLines = 1
                                setTextColor(Color.WHITE)
                                textSize = 8f
                                text = positionTab?.superscript
                                layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
                                ShapeBuilder.create()
                                        .roundRadius(DisplayUtil.dp2px(4f), DisplayUtil.dp2px(7f), 0f, DisplayUtil.dp2px(7f))
                                        .solid(0xffff2e62.toInt())
                                        .stroke(DisplayUtil.dp2pxInt(1F), Color.WHITE)
                                        .build(this)
                                setPaddingRelative(DisplayUtil.dp2pxInt(4f), DisplayUtil.dp2pxInt(1f), DisplayUtil.dp2pxInt(4f), DisplayUtil.dp2pxInt(1f))
                            }
                        }
                    }
                } else {
                    fragment.tabLayout.getTabAt(i)?.removeCustomBadgeView()
                }
                fragment.tabLayout.getTabAt(i)?.customView?.let {
                    it.setOnClickListener {
                        fragment.tabLayout.getTabAt(i)?.select()
                    }
                    DogCat.getInstance().exposure(it).block("TopMultipleTabExpose").viewId(positionTab?.id.toString())
                            .param("tabid", positionTab?.id.toString())
                            .param("position", (i + 1).toString())
                            .commit()
                }
            }
            homeTopThemeModule?.setTheme(getTheme(homepageVO.tabs?.get(fragment.homePager.currentItem)), homepageVO.tabs?.get(fragment.homePager.currentItem), isNewcomer && needShowNewcomer)
            onTabSelectedListener?.let {
                fragment.tabLayout.removeOnTabSelectedListener(it)
            }
            val listener = object : MaterialTabLayout.OnTabSelectedListener {
                override fun onTabReselected(tab: MaterialTabLayout.Tab?) {
                }

                override fun onTabSelected(tab: MaterialTabLayout.Tab?) {
                    LogUtil.w("sj", "onTabSelected:${tab?.position}")
                    if (tab?.position.toNumber() >= homepageVO.tabs?.size.toNumber() || !ViewCompat.isLaidOut(fragment.tabLayout)) {
                        return
                    }
                    if (!fragment.isShowing) {
                        fragment.refresh_layout.refreshFinished()
                    }
                    homeTopThemeModule?.setTheme(getTheme(tab?.position?.let { homepageVO.tabs?.get(it) }), tab?.position?.let { homepageVO.tabs?.get(it) }, isNewcomer && needShowNewcomer)
                    tab?.customView?.findViewById<ImageView>(R.id.img)?.apply {
                        if (visibility == View.VISIBLE) {
                            GlideApp.with(this).load(fragment.context?.let {
                                CDNHelper.getInstance().autoFix(it, homepageVO.tabs?.get(tab.position)?.activeImage)
                            }).into(this)
                        }
                    }
                    tab?.customView?.findViewById<TextView>(R.id.txt)?.apply {
                        if (visibility == View.VISIBLE) {
                            textSize = 18f
                            paint.isFakeBoldText = true
                        }
                    }
                    tab?.removeCustomBadgeView()
                    tab?.position?.let {
                        if (!TextUtils.isEmpty(homepageVO.tabs?.get(it)?.superscript)) {
                            MovieCacheSet.getInstance().putString(CommonConstants.BADGE_TAB_HOME_SHOW + "_" + homepageVO.tabs?.get(tab.position)?.id, homepageVO.tabs?.get(tab.position)?.superscript.orEmpty())
                        }
                    }
                }

                override fun onTabUnselected(tab: MaterialTabLayout.Tab?) {
                    tab?.customView?.findViewById<ImageView>(R.id.img)?.apply {
                        if (visibility == View.VISIBLE) {
                            GlideApp.with(this).load(fragment.context?.let {
                                CDNHelper.getInstance().autoFix(it, homepageVO.tabs?.get(tab.position)?.image)
                            }).into(this)
                        }
                    }
                    tab?.customView?.findViewById<TextView>(R.id.txt)?.apply {
                        if (visibility == View.VISIBLE) {
                            textSize = 16f
                            paint.isFakeBoldText = false
                        }
                    }
                }
            }
            onTabSelectedListener = listener
            fragment.tabLayout.addOnTabSelectedListener(listener)
            fragment.homePager.addOnPageChangeListener(object : OnPageChangeListener {
                var hasSet = false
                override fun onPageScrollStateChanged(p0: Int) {
                    if (p0 == ViewPager.SCROLL_STATE_DRAGGING) {
                        if (!hasSet) {
                            hasSet = true
                            fragment?.refresh_layout?.refreshFinished()
                        }
                    }
                }

                override fun onPageScrolled(p0: Int, p1: Float, p2: Int) {
                }

                override fun onPageSelected(p0: Int) {
                    homepageVO.tabs?.size?.let {
                        if (p0 < it) {
                            if (p0 != lastIndex) {
                                DogCat.getInstance().click().ctrlName("TopMultipleTabClick")
                                        .param("tabid", homepageVO.tabs?.get(p0)?.id.toString())
                                        .param("position", (p0.plus(1)).toString())
                                        .commit()
                            }
                            lastIndex = p0
                            if (homepageVO.tabs?.get(p0)?.type == PositionTab.TYPE_DAMAI) {
                                pagerAdapter?.apply {
                                    if (currentFragment is BaseFragment) {
                                        fragment?.utPageName = (currentFragment as BaseFragment).utPageName
                                        DogCat.getInstance().updatePageName((currentFragment as BaseFragment).utPageName)
                                    }
                                }
                            } else {
                                fragment?.utPageName = UTConstants.PageMVHomePage
                                DogCat.getInstance().updatePageName(UTConstants.PageMVHomePage)
                            }
                            fragment?.homePager?.postDelayed({ fragment?.hideHomeBannerVideo() }, 500)
                        }
                    }
                }
            })
        } else {
            fragment.tabLayout.visibility = View.GONE
            homeTopThemeModule?.setTheme(if (isNewcomer && needShowNewcomer) Appbar.THEME_DARK else getTheme(homepageVO.tabs?.get(0)), homepageVO.tabs?.get(0), isNewcomer && needShowNewcomer)
        }
    }

    override fun setPagerTopMargin(hasMargin: Boolean) {
        if (fragment.homePager?.layoutParams is ViewGroup.MarginLayoutParams) {
            (fragment.homePager?.layoutParams as ViewGroup.MarginLayoutParams).topMargin = if (hasMargin) marginTop else 0
        }
    }

    override fun getViewPager(): ViewPager? {
        return if (fragment.lifecycle.currentState.isAtLeast(Lifecycle.State.STARTED)) {
            fragment.homePager
        } else {
            fragment.findViewById(R.id.homePager)
        }
    }

    override fun currentFragment(): Fragment? {
        return pagerAdapter?.currentFragment
    }

    override fun refreshAll() {
        pagerAdapter?.notifyDataSetChanged()
    }

    override fun itemCount(): Int {
        return pagerAdapter?.count ?: 0
    }

    /**
     * 更新子页面数据，只更新数据不刷新视图
     */
    override fun setRefreshData(data: Any) {
        pagerAdapter?.let {
            for (index in 0 until it.count) {
                val fragment = it.getItem(index)
                if (fragment is RefreshView) {
                    fragment.setRefreshData(data)
                }
            }
        }
    }

    private fun getTheme(tab: PositionTab?): Int {
        if (!TextUtils.isEmpty(tab?.backgroundLottieUrl) ||
                !TextUtils.isEmpty(tab?.backgroundImageUrl) ||
                !TextUtils.isEmpty(tab?.backgroundGradualColor) ||
                !TextUtils.isEmpty(tab?.backgroundColor)) {
            if (TextUtils.isEmpty(tab?.backgroundLottieUrl) &&
                    TextUtils.isEmpty(tab?.backgroundImageUrl) &&
                    TextUtils.isEmpty(tab?.backgroundGradualColor) &&
                    tab?.backgroundColor?.toLowerCase()?.contains("ffffff").isTrue()) {
                return Appbar.THEME_LIGHT
            }
            return Appbar.THEME_DARK
        }
        return Appbar.THEME_LIGHT
    }

    private fun isNewcomer(vo: HomepageVO) = vo.actForNewVO?.userIdentity?.haveTppTickets == false

    private fun needShowNewcomer(vo: HomepageVO) = vo.showModule?.actForNewShows?.isNotEmpty() == true
}