package com.taobao.movie.android.app.oscar.ui.smartvideo.item

import android.annotation.SuppressLint
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.Rect
import android.graphics.drawable.GradientDrawable
import android.os.Build
import android.os.Handler
import android.os.Message
import android.support.constraint.ConstraintLayout
import android.support.v4.graphics.ColorUtils
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.view.ViewTreeObserver
import android.widget.LinearLayout
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerViewHolder
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.oscar.ui.homepage.EVENT_AIYI_REFRESH_FINISH
import com.taobao.movie.android.app.oscar.ui.homepage.newcomer.DateCountDownTimer
import com.taobao.movie.android.app.presenter.community.FilmFestivalVO
import com.taobao.movie.android.app.presenter.community.YilianClubVO
import com.taobao.movie.android.arch.items.*
import com.taobao.movie.android.common.Region.RegionBizService
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.commonui.component.BaseFragment
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.utils.BackgroundManager
import com.taobao.movie.android.commonui.utils.ViewUtils
import com.taobao.movie.android.commonui.widget.IconFontTextView
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.commonutil.LocalEvent
import com.taobao.movie.android.commonutil.LocalEventBus
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.android.sdk.infrastructure.utils.TaskExcutorHelper
import com.taobao.movie.android.ut.click
import com.taobao.movie.android.ut.expose
import com.taobao.movie.appinfo.util.*
import com.taobao.movie.shawshank.time.ServerTime
import kotlinx.android.synthetic.main.community_aiyi_head_layout.view.*
import kotlinx.android.synthetic.main.item_community_aiyi_head_festival.view.*

/**
 * Created by 天逐 on 2019-12-09.
 * function:爱艺之城Item
 */
class CommunityAiyiItem : RecyclerExtDataItem<CommunityAiyiItem.ViewHolder, YilianClubVO>, View.OnClickListener, BaseFragment.PageLifecycle {

    var promotionShowFestival: FilmFestivalVO? = null
    var timer: DateCountDownTimer? = null
    var paletteColor = 0
    val AnimMsg = 0x100
    var isCancelled = false

    val decoration = object : RecyclerView.ItemDecoration() {
        override fun getItemOffsets(outRect: Rect, view: View, parent: RecyclerView, state: RecyclerView.State) {
            super.getItemOffsets(outRect, view, parent, state)
            val v = DisplayUtil.dp2pxInt(9f)
            val h = DisplayUtil.dp2pxInt(3f)
            outRect.top = v
            outRect.bottom = v
            outRect.left = h
            outRect.right = h
            if (parent.getChildLayoutPosition(view) == 0) {
                outRect.left = DisplayUtil.dp2px(15f).toInt()
            }/* else if (parent.getChildLayoutPosition(view) == parent.adapter!!.itemCount - 1) {
                outRect.right = DisplayUtil.dp2pxInt(15f)
            }*/
        }
    }

    var handler = @SuppressLint("HandlerLeak")
    object : Handler() {
        @Synchronized override fun handleMessage(msg: Message?) {
            if (isCancelled) {
                return
            }
            if (msg?.what == AnimMsg) {
                recycleViewHolder?.itemView?.list_top_card_festival?.scrollBy(1, 0)
                sendMessageDelayed(obtainMessage(AnimMsg), 32)
            }
        }
    }

    constructor(yilianClubVO: YilianClubVO) : super(yilianClubVO) {
    }

    constructor(yilianClubVO: YilianClubVO, l: RecyclerExtDataItem.OnItemEventListener<*>?) : super(yilianClubVO, l) {
    }

    init {
        recycleViewHolder?.itemView?.addOnAttachStateChangeListener(object : View.OnAttachStateChangeListener {
            override fun onViewDetachedFromWindow(v: View?) {
                cancelAnim()
                handler.removeCallbacksAndMessages(null)
            }

            override fun onViewAttachedToWindow(v: View?) {
                startAnim()
            }
        })
    }

    fun onClear() {
        cancelAnim()
        timer?.cancel()
    }

    fun onResume() {
        startAnim()
    }

    fun onPause() {
        cancelAnim()
    }

    private fun startAnim() {
        handler.removeCallbacksAndMessages(null)
        isCancelled = false
        handler.sendMessage(handler.obtainMessage(AnimMsg))
    }

    private fun cancelAnim() {
        isCancelled = true
        handler.removeCallbacksAndMessages(null)
    }

    override fun onPageAppear() {
        startAnim()
    }

    override fun onPageDisAppear() {
        cancelAnim()
    }

    override fun onBindViewHolder(viewHolder: ViewHolder) {
        if (getData() == null) {
            return
        }
        viewHolder.mHeadImg.url = OSSCDNHelper.autoFix(viewHolder.mHeadImg, getData().ossBgImage)
        viewHolder.mHeadImg.requestListener = object : MoImageView.SimpleRequestListener {
            override fun onLoadFailed(e: Exception?, model: Any?, isFirstResource: Boolean): Boolean {
                return false
            }

            override fun onResourceReady(picResource: Any?, model: Any?, rect: Rect?, isFirstResource: Boolean): Boolean {
                if (picResource is Bitmap) {
                    try {
                        val color = picResource.getPixel(0, picResource.height - 1)
                        viewHolder.overlayView.background = GradientDrawable().apply {
                            colors = intArrayOf(color, ColorUtils.setAlphaComponent(color, 0))
                            orientation = GradientDrawable.Orientation.TOP_BOTTOM
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
                return false
            }
        }
        viewHolder.mTvAddClub.setOnClickListener(this)
        if (getData().inClub != 0) { //已加入
            viewHolder.mTvAddClub.text = "已加入"
            viewHolder.mTvAddClubIcon.visibility = View.GONE
            viewHolder.mTvAddClub.setTextColor(ResHelper.getColor(R.color.transparent_white_070))
            ShapeBuilder.create().stroke(1, ResHelper.getColor(R.color.transparent_white_020)).solid(ResHelper.getColor(R.color.transparent_white_020)).radius(DisplayUtil.dp2px(15f)).build(viewHolder.mTvAddClubLayout)
        } else { //未加入
            viewHolder.mTvAddClub.text = "加入"
            viewHolder.mTvAddClubIcon.visibility = View.VISIBLE
            viewHolder.mTvAddClub.setTextColor(ResHelper.getColor(R.color.common_text_color46))
            ShapeBuilder.create().stroke(1, ResHelper.getColor(R.color.common_text_color46)).solid(ResHelper.getColor(R.color.transparent)).radius(DisplayUtil.dp2px(15f)).build(viewHolder.mTvAddClubLayout)
        }
        viewHolder.mTvPersonNum.text = Html.fromHtml("已有 " + "<b><font color=\"#ffffff\">" + NumberFormatUtil.getTenThousandOfANumber(getData().count) + "</font></b>" + " 名艺术电影爱好者加入")

        viewHolder.itemView.layout_card_festival.visibility = if (promotionShowFestival == null) View.GONE else View.VISIBLE
        promotionShowFestival?.let { vo ->
            viewHolder.itemView.tv_top_card_festival_title.expose()
                    .block("MovieFestivalBannerExpose")
                    .param("cityCode", RegionBizService.getUserRegion().cityCode)
                    .param("moviefestival_id", vo.id.toString())
                    .commit()
            viewHolder.itemView.overlay_top_card_click.click()
                    .ctrlName("MovieFestivalBannerClick")
                    .param("cityCode", RegionBizService.getUserRegion().cityCode)
                    .param("moviefestival_id", vo.id.toString())
                    .commit()

            viewHolder.itemView.tv_top_card_festival_title.text = vo.name
            viewHolder.itemView.tv_top_card_open_close_time.text = getOpenCloseTime(vo)
            viewHolder.itemView.overlay_top_card_click.setOnClickListener {
                MovieNavigator.handleUrl(viewHolder.itemView.context, vo.jumpUrl)
            }
            viewHolder.itemView.tv_top_card_raiders.setOnClickListener {
                MovieNavigator.handleUrl(viewHolder.itemView.context, vo.tutorialUrl)
            }
            viewHolder.itemView.img_top_card_festival.url = OSSCDNHelper.autoFix(viewHolder.itemView.img_top_card_festival, vo.ossAvatar)
            viewHolder.itemView.img_top_card_festival.requestListener = object : MoImageView.SimpleRequestListener {
                override fun onLoadFailed(e: java.lang.Exception?, model: Any?, isFirstResource: Boolean): Boolean {
                    return false
                }

                override fun onResourceReady(picResource: Any?, model: Any?, rect: Rect?, isFirstResource: Boolean): Boolean {
                    if (picResource is Bitmap) {
                        TaskExcutorHelper.getInstance().runOnBGThread {
                            val color = BackgroundManager.getInstance().getSmartMeanRgb(picResource)
                            TaskExcutorHelper.getInstance().runOnUIThread {
                                viewHolder.itemView.overlay_top_card.alpha = 0f
                                viewHolder.itemView.overlay_top_card.setBackgroundColor(color)
                                viewHolder.itemView.overlay_top_card.animate().alpha(0.3f).start()

                                viewHolder.itemView.overlay_top_card_gradient.alpha = 0f
                                viewHolder.itemView.overlay_top_card_gradient.background = GradientDrawable().apply {
                                    colors = intArrayOf(color, ColorUtils.setAlphaComponent(color, 0))
                                    orientation = GradientDrawable.Orientation.BOTTOM_TOP
                                }
                                viewHolder.itemView.overlay_top_card_gradient.animate().alpha(1f).start()
                                paletteColor = color
                                if (vo.purchaseOpenTime != null && vo.purchaseOpenTime <= ServerTime.getCurrentServerTime()) {
                                    setupBuyButton(viewHolder, vo, color)
                                }
                            }
                        }
                    }
                    return false
                }
            }
            if (vo.purchaseOpenTime == null) {
                viewHolder.itemView.layout_card_bottom_time.visibility = View.VISIBLE
                viewHolder.itemView.btn_card_buy_tickets.visibility = View.GONE
                viewHolder.itemView.tv_top_card_remaining_time_suffix.visibility = View.GONE
                viewHolder.itemView.tv_top_card_remaining_time.visibility = View.GONE
                viewHolder.itemView.tv_top_card_remaining_time_prefix.text = vo.buyButtonTip
                viewHolder.itemView.tv_top_card_raiders.text = vo.ticketGuideTip
            } else {
                val timestamp = ServerTime.getCurrentServerTime()
                val millisInFuture = vo.purchaseOpenTime - timestamp
                if (vo.purchaseOpenTime <= timestamp || millisInFuture < 1000) {
                    setupBuyButton(viewHolder, vo, paletteColor)
                } else {
                    viewHolder.itemView.layout_card_bottom_time.visibility = View.VISIBLE
                    viewHolder.itemView.btn_card_buy_tickets.visibility = View.GONE
                    viewHolder.itemView.tv_top_card_remaining_time_prefix.text = vo.buyButtonTip
                    viewHolder.itemView.tv_top_card_raiders.text = vo.ticketGuideTip
                    timer?.cancel()
                    timer = object : DateCountDownTimer(millisInFuture) {
                        override fun onFinish() {
                            setupBuyButton(viewHolder, vo, paletteColor, "立即抢票")
                            LocalEventBus.getDefault().post(LocalEvent(EVENT_AIYI_REFRESH_FINISH))
                        }

                        override fun onTick(dateString: String?) {
                            if (dateString == null) {
                                return
                            }
                            if (dateString.contains("天")) {
                                viewHolder.itemView.tv_top_card_remaining_time_suffix.visibility = View.VISIBLE
                                viewHolder.itemView.tv_top_card_remaining_time.text = dateString.replace("天", "")
                            } else {
                                viewHolder.itemView.tv_top_card_remaining_time_suffix.visibility = View.GONE
                                viewHolder.itemView.tv_top_card_remaining_time.text = dateString
                            }
                        }
                    }.apply {
                        setMillisFormat(DateCountDownTimer.DefaultMillisFormat2(vo.purchaseOpenTime))
                        start()
                    }
                    viewHolder.itemView.tv_top_card_remaining_time.visibility = View.VISIBLE
                }
            }

            // list
            ImageListAdapter().apply {
                viewHolder.itemView.list_top_card_festival?.let {
                    attach(it)
                    it.viewTreeObserver.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
                        override fun onGlobalLayout() {
                            if (it.canScrollHorizontally(1)) {
                                it.viewTreeObserver.removeOnGlobalLayoutListener(this)
                                startAnim()
                            }
                        }
                    })
                    it.layoutManager = LinearLayoutManager(it.context).apply {
                        orientation = LinearLayoutManager.HORIZONTAL
                    }
                    while (it.itemDecorationCount > 0) {
                        it.removeItemDecorationAt(0)
                    }
                    it.addItemDecoration(decoration)
                }
                val list = mutableListOf<RecyclerItem<ShowMo>>()
                vo.shows?.forEach {
                    if (it != null) {
                        list.add(convert(it) as RecyclerItem<ShowMo>)
                    }
                }
                submitList(list)
            }
        }
    }

    private fun setupBuyButton(viewHolder: ViewHolder, vo: FilmFestivalVO, color: Int, btnTip: String? = null) {
        val radius = DisplayUtil.dp2px(24f)
        viewHolder.itemView.layout_card_bottom_time.visibility = View.GONE
        viewHolder.itemView.btn_card_buy_tickets.visibility = View.VISIBLE
        viewHolder.itemView.btn_card_buy_tickets.text = btnTip ?: vo.buyButtonTip
        if (color == 0) {
            viewHolder.itemView.btn_card_buy_tickets.background = GradientDrawable().apply {
                colors = intArrayOf(0x80FFFFFF.toInt(), 0xFFFFFFFF.toInt())
                orientation = GradientDrawable.Orientation.LEFT_RIGHT
                cornerRadius = DisplayUtil.dp2px(24f)
            }
            viewHolder.itemView.btn_card_buy_tickets.setTextColor(0xFFFF2E62.toInt())
            return
        } else {
            viewHolder.itemView.btn_card_buy_tickets.setTextColor(Color.WHITE)
        }
        val hColors = BackgroundManager.getInstance().candidateRangeH
        when (color) {
            hColors[0], hColors[5] -> viewHolder.itemView.btn_card_buy_tickets.background = GradientDrawable().apply {
                colors = intArrayOf(0xFF4EB9FE.toInt(), 0xFF6583FF.toInt())
                orientation = GradientDrawable.Orientation.LEFT_RIGHT
                cornerRadius = radius
            }
            hColors[1], hColors[2], hColors[4] -> viewHolder.itemView.btn_card_buy_tickets.background = GradientDrawable().apply {
                colors = intArrayOf(0xFFFFAC28.toInt(), 0xFFF76B1C.toInt())
                orientation = GradientDrawable.Orientation.LEFT_RIGHT
                cornerRadius = radius
            }
            else -> viewHolder.itemView.btn_card_buy_tickets.background = GradientDrawable().apply {
                colors = intArrayOf(0xFFFF62B2.toInt(), 0xFFFF2E62.toInt())
                orientation = GradientDrawable.Orientation.LEFT_RIGHT
                cornerRadius = radius
            }
        }
    }

    private fun getOpenCloseTime(it: FilmFestivalVO): CharSequence? {
        val openTime = it.openTime?.toLong()?.let { it1 -> DateUtil.getMonthDayDate(it1) }
        val closeTime = DateUtil.getMonthDayDate(it.closeTime)
        val sb = StringBuilder()
        if (openTime?.isNotEmpty() == true) {
            sb.append(openTime)
        }
        if (closeTime.isNotEmpty()) {
            if (sb.isNotEmpty()) {
                sb.append("-")
            }
            sb.append(closeTime)
        }
        return sb.toString()
    }

    override fun onClick(v: View?) {
        if (v == null || getViewHolder() == null || getData() == null) {
            return
        }
        when (v) {
            getViewHolder()?.mTvAddClub -> {
                UTFacade.buttonClickWithoutPage("YilianJoinClick", "status", (getData().inClub).toString())
                onEvent(ItemEvent.EVENT_ID_COMMUNITY_AIYI_ADD)

            }
        }
    }


    class ViewHolder(itemView: View) : CustomRecyclerViewHolder<CommunityAiyiItem>(itemView) {
        var mHeadImg: MoImageView
        var mTvPersonNum: TextView
        var mTvAddClub: TextView
        var mTvAddClubIcon: IconFontTextView
        var mTvAddClubLayout: LinearLayout
        var mRootView: ConstraintLayout
        var overlayView: View

        init {
            mHeadImg = itemView.findViewById(R.id.community_aiyi_head_img)
            mTvPersonNum = itemView.findViewById(R.id.person_num_text)
            mTvAddClub = itemView.findViewById(R.id.tv_add_aiyi_community)
            mTvAddClubIcon = itemView.findViewById(R.id.icon_add_aiyi)
            mTvAddClubLayout = itemView.findViewById(R.id.ll_add_aiyi_layout)
            mRootView = itemView.findViewById(R.id.community_aiyi_head_layout)
            overlayView = itemView.findViewById(R.id.overlay_gradient)

            val h = DisplayUtil.dp2pxInt(126f)
            mHeadImg.layoutParams.height = if (Build.VERSION.SDK_INT >= 21) (h + DisplayUtil.getStatusBarHeight()).coerceAtLeast(DisplayUtil.dp2pxInt(150f)) else h
            itemView.btn_card_buy_tickets.background = GradientDrawable().apply {
                colors = intArrayOf(0x80FFFFFF.toInt(), 0xFFFFFFFF.toInt())
                orientation = GradientDrawable.Orientation.LEFT_RIGHT
                cornerRadius = DisplayUtil.dp2px(24f)
            }
            itemView.btn_card_buy_tickets.setTextColor(0xFFFF2E62.toInt())
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.community_aiyi_head_layout
    }

    class ImageListAdapter : CustomListAdapter<RecyclerItem<ShowMo>, CustomViewHolder<RecyclerItem<ShowMo>>>(RecyclerItem.DiffCallback()), ItemDataConverter<ShowMo> {
        override fun createCustomViewHolder(view: View?, parent: ViewGroup, viewType: Int): CustomViewHolder<RecyclerItem<ShowMo>>? {
            return if (view != null) ImageListViewHolder(view) else null
        }

        override fun convert(data: ShowMo): ItemData<ShowMo> {
            return RecyclerItem(data, ViewUtils.generateId(), R.layout.item_community_aiyi_head_festival)
        }

        override fun onViewDetachedFromWindow(holder: CustomViewHolder<RecyclerItem<ShowMo>>) {
            recyclerView?.getChildLayoutPosition(holder.itemView)?.let {
                currentList.add(currentList.removeAt(it))
                val list = mutableListOf<RecyclerItem<ShowMo>>()
                list.addAll(currentList)
                submitList(list)
            }
        }
    }

    class ImageListViewHolder(itemView: View) : CustomViewHolder<RecyclerItem<ShowMo>>(itemView) {

        override fun onBindItem(position: Int, itemData: RecyclerItem<ShowMo>) {
            itemView.img_community_head_festival_poster.url = itemData.listData.poster
        }
    }
}
