package com.taobao.movie.android.app.oscar.ui.smartvideo.item

import android.support.v7.widget.CardView
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerViewHolder
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.presenter.community.FilmFestivalVO
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.home.R
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.OSSCDNHelper
import com.taobao.movie.appinfo.util.ResHelper

/**
 * Created by 天逐 on 2019-12-09.
 * function:社区电影节Item
 */
class CommunityFestivalItem(private val filmFestivalVO: FilmFestivalVO, listener: OnItemEventListener<*>?) : RecyclerExtDataItem<CommunityFestivalItem.ViewHolder, FilmFestivalVO>(filmFestivalVO, listener), View.OnClickListener {

    override fun onBindViewHolder(viewHolder: ViewHolder) {
        if (getData() == null) {
            return
        }
//        viewHolder.mBgImg.url = getData().avatar
        viewHolder.mBgImg.url = OSSCDNHelper.autoFix(viewHolder.mBgImg, getData().ossAvatar)
        if (TextUtils.isEmpty(getData().tag)) {
            viewHolder.mTvTag.visibility = View.GONE
        } else {
            viewHolder.mTvTag.visibility = View.VISIBLE
            viewHolder.mTvTag.setText(getData().tag)
            ShapeBuilder.create().solid(ResHelper.getColor(R.color.common_color_1045)).roundRadius(DisplayUtil.dp2px(6f), 0f, 0f, DisplayUtil.dp2px(6f)).build(viewHolder.mTvTag)
        }
        viewHolder.mTvtitle.setText(getData().name)
        viewHolder.mTvtime.setText(getData().openDesc)
        viewHolder.itemView.setOnClickListener(this)
    }

    override fun onClick(v: View?) {
        UTFacade.buttonClickWithoutPage("MovieFestivalClick", "type", data?.type?.toString(), "moviefestival_id", data?.id?.toString())
        onEvent(ItemEvent.EVENT_ID_COMMUNITY_FILM_FESTIVAL_ITEM_CLICK)

    }


    class ViewHolder(itemView: View) : CustomRecyclerViewHolder<CommunityFestivalItem>(itemView) {
        var mCardtem: CardView
        var mBgImg: MoImageView
        var mTvtitle: TextView
        var mTvtime: TextView
        var mTvTag: TextView

        init {
            mCardtem = itemView.findViewById(R.id.community_festival_card_view)
            mBgImg = itemView.findViewById(R.id.community_festival_bg_img)
            mTvtitle = itemView.findViewById(R.id.tv_community_festival_title)
            mTvtime = itemView.findViewById(R.id.tv_community_festival_time)
            mTvTag = itemView.findViewById(R.id.tv_community_festival_tag)
            mCardtem.radius = DisplayUtil.dp2px(6f)
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.community_festival_item_layout
    }
}
