package com.taobao.movie.android.app.oscar.ui.smartvideo.item

import android.support.v7.widget.CardView
import android.view.View
import android.widget.TextView
import com.taobao.listitem.recycle.CustomRecyclerViewHolder
import com.taobao.listitem.recycle.RecyclerExtDataItem
import com.taobao.movie.android.app.oscar.ui.community.model.FilmFestivalModel
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.widget.shapebuilder.ShapeBuilder
import com.taobao.movie.android.home.R
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.OSSCDNHelper
import com.taobao.movie.appinfo.util.ResHelper

class CommunityFilmFestivalItem(filmFestivalModel: FilmFestivalModel) : RecyclerExtDataItem<CommunityFilmFestivalItem.ViewHolder, FilmFestivalModel>(filmFestivalModel), View.OnClickListener {

    override fun onBindViewHolder(viewHolder: ViewHolder?) {
        viewHolder?.mBgImg?.url = OSSCDNHelper.autoFix(viewHolder?.mBgImg, data?.ossAvatar)
        if (data?.tag.isNullOrEmpty()) {
            viewHolder?.mTvTag?.visibility = View.GONE
        } else {
            viewHolder?.mTvTag?.visibility = View.VISIBLE
            viewHolder?.mTvTag?.text = data?.tag
            ShapeBuilder.create().solid(ResHelper.getColor(R.color.common_color_1045)).roundRadius(DisplayUtil.dp2px(6f), 0f, 0f, DisplayUtil.dp2px(6f)).build(viewHolder?.mTvTag)
        }
        viewHolder?.mTvtitle?.text = data?.name
        viewHolder?.mTvtime?.text = data?.openDesc
        viewHolder?.itemView?.setOnClickListener(this)
    }

    override fun onClick(v: View?) {
        UTFacade.buttonClickWithoutPage("MovieFestivalClick", "type", data?.type?.toString(), "moviefestival_id", data?.id)
        MovieNavigator.handleUrl(v?.context, data.jumpUrl)
    }

    override fun getLayoutId(): Int {
        return R.layout.film_festival_item
    }

    class ViewHolder(itemView: View) : CustomRecyclerViewHolder<CommunityFilmFestivalItem>(itemView) {

        var mCardtem: CardView
        var mBgImg: MoImageView
        var mTvtitle: TextView
        var mTvtime: TextView
        var mTvTag: TextView

        init {
            mCardtem = itemView.findViewById(R.id.community_festival_card_view)
            mBgImg = itemView.findViewById(R.id.community_festival_bg_img)
            mTvtitle = itemView.findViewById(R.id.tv_community_festival_title)
            mTvtime = itemView.findViewById(R.id.tv_community_festival_time)
            mTvTag = itemView.findViewById(R.id.tv_community_festival_tag)
            mCardtem.radius = DisplayUtil.dp2px(6f)
        }

    }

}