package com.taobao.movie.android.app.oscar.ui.smartvideo.item

import android.graphics.Typeface
import android.text.TextUtils
import android.view.View
import com.taobao.movie.android.app.community.CommunityFilmCalendarFragment
import com.taobao.movie.android.app.presenter.community.ArtShowTabVO
import com.taobao.movie.android.app.presenter.community.CommunityHomepageVO
import com.taobao.movie.android.business.R
import com.taobao.movie.android.common.item.homepage.CommunityShowMoreItem
import com.taobao.movie.android.common.item.homepage.CommunitySoonShowItem
import com.taobao.movie.android.common.item.homepage.HomepageShowContainerBaseItem
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.uiInfo.SoonShowTabListModuleVO
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DataUtil
import com.taobao.movie.appinfo.util.ResHelper

/**
 * 社区正在热映和即将上映模块
 */
class CommunityShowAndSoonContainerItem(private val mCommunityHomepageVO: CommunityHomepageVO, listener: OnItemEventListener<*>?)
    : HomepageShowContainerBaseItem<CommunityHomepageVO>(mCommunityHomepageVO, listener), View.OnClickListener {
    companion object {
        /* 初始化的值 */
        const val POSITION_INIT = -1
        /* 热映影片 对应位置 */
        const val POSITION_MAIN_LEFT = 0
        /* 即将上映 对应位置 */
        const val POSITION_MAIN_RIGHT = 1
    }

    /* 当前展示的位置 */
    private var currentMainPosition = POSITION_INIT
    /* 灰色 */
    private val color1001 = ResHelper.getColor(R.color.common_color_1001)
    /* 黑色 */
    private val colorBlack = ResHelper.getColor(R.color.black)

    private var defaultTabType = 0

    override fun onBindViewHolder(viewHolder: ViewHolder?) {
        if (viewHolder == null || mCommunityHomepageVO.artShowTabs.isNullOrEmpty()) return

        viewHolder.initTitleContain()
        viewHolder.initSoonShowTab()
        defaultTabType = mCommunityHomepageVO.defaultTabType;
        viewHolder.allTabTitles[0].visibility = View.INVISIBLE
        viewHolder.allTabTitles[1].visibility = View.INVISIBLE
        viewHolder.allTabTitles[2].visibility = View.INVISIBLE
        viewHolder.vsFlexboxLayout.visibility = View.VISIBLE
        viewHolder.vsDivider2.visibility = View.INVISIBLE
        viewHolder.vsDivider3.visibility = View.INVISIBLE
        viewHolder.vsBillboardLayout.visibility = View.GONE
        viewHolder.vsTitleContainerLayout.visibility = View.VISIBLE
        viewHolder.vsTitle.text = "艺术电影日历"
        viewHolder.vsDivider3.visibility = View.GONE
        var postion = setTabTitleAndGetFirstSelectPostion(viewHolder)
        onTabClick(postion)
        viewHolder.allTabTitles[0].setOnClickListener(onLeftClickListener)
        viewHolder.allTabTitles[1].setOnClickListener(onRightClickListener)
        viewHolder.vsTitle.setOnClickListener(this)
        viewHolder.vsArrow.setOnClickListener(this)

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.tv_homepage_show_container_title, R.id.tv_homepage_show_container_right_arrow -> {
                onEvent(ItemEvent.EVENT_ID_COMMUNITY_GO_FILM_CALENDAR, CommunityFilmCalendarFragment.PAGE_FROM_CALENDAR_TITLE)

            }
        }
    }


    private fun setTabTitleAndGetFirstSelectPostion(viewHolder: ViewHolder?): Int {
        if (viewHolder == null || mCommunityHomepageVO.artShowTabs.isNullOrEmpty()) return -1
        var size = DataUtil.getListSize(mCommunityHomepageVO.artShowTabs)
        if (size <= 0) {
            viewHolder.itemView.visibility = View.GONE
            return -1
        }
        if (size >= 1) {
            viewHolder.allTabTitles.get(0).visibility = View.VISIBLE
            viewHolder.allTabTitles.get(0).setText(if (data?.artShowTabs?.get(0)?.type == ArtShowTabVO.TYPE_HOT_FILM) "今日热映" else "即将上映")
        }
        if (size >= 2) {
            viewHolder.allTabTitles.get(1).visibility = View.VISIBLE
            viewHolder.vsDivider2.visibility = View.VISIBLE
            viewHolder.allTabTitles.get(1).setText(if (data?.artShowTabs?.get(1)?.type == ArtShowTabVO.TYPE_HOT_FILM) "今日热映" else "即将上映")
            if (mCommunityHomepageVO.artShowTabs?.get(1)?.type == defaultTabType) {
                return POSITION_MAIN_RIGHT
            }

        }
        return POSITION_MAIN_LEFT
    }

    /**
     * 热映影片 和 即将上映 tab的点击
     */
    private fun onTabClick(position: Int) {
        if (position < 0 || currentMainPosition == position || mCommunityHomepageVO.artShowTabs.isNullOrEmpty()) return
        currentMainPosition = position
        val vh = viewHolder
        vh.videoExpressAdapter.clearItems()
        if (position == POSITION_MAIN_LEFT) {
            viewHolder.allTabTitles.get(0).setTextColor(colorBlack)
            viewHolder.allTabTitles.get(1).setTextColor(color1001)
            viewHolder.allTabTitles.get(0).setTypeface(Typeface.DEFAULT_BOLD)
            viewHolder.allTabTitles.get(1).setTypeface(Typeface.DEFAULT)
            val v0 = mCommunityHomepageVO.artShowTabs?.get(0)
            if (v0 != null && v0?.shows.isNotEmpty()) {
                v0?.shows.forEach { vh.videoExpressAdapter.addItem(CommunitySoonShowItem(it, listener)) }
                if (DataUtil.getListSize(v0?.shows) >= 8) {
                    vh.videoExpressAdapter.addItem(CommunityShowMoreItem(v0.count, listener))
                }
            } else {
                vh.videoExpressAdapter.addItem(EmptyShowSoonItem())
            }
        } else {
            viewHolder.allTabTitles.get(0).setTextColor(color1001)
            viewHolder.allTabTitles.get(1).setTextColor(colorBlack)
            viewHolder.allTabTitles.get(1).setTypeface(Typeface.DEFAULT_BOLD)
            viewHolder.allTabTitles.get(0).setTypeface(Typeface.DEFAULT)

            val v1 = mCommunityHomepageVO.artShowTabs?.get(1)
            if (v1 != null && v1.shows.isNotEmpty()) {
                v1.shows.forEach { vh.videoExpressAdapter.addItem(CommunitySoonShowItem(it, listener)) }
                if (DataUtil.getListSize(v1.shows) >= 8) {
                    vh.videoExpressAdapter.addItem(CommunityShowMoreItem(v1.count, listener))
                }

            } else {
                vh.videoExpressAdapter.addItem(EmptyShowSoonItem())
            }
        }
        vh.videoExpressAdapter.notifyDataSetChanged()
        vh.rvArticleFilmExpress.scrollToPosition(0)
    }


    /* 热映影片 点击事件 */
    private val onLeftClickListener = View.OnClickListener {
        data?.artShowTabs?.takeIf { it.size > 0 }?.let {
            UTFacade.buttonClickWithoutPage("ArtMvCalendarTabClick", "tab", if (it[0]?.type == ArtShowTabVO.TYPE_HOT_FILM) "todayMV" else "FutureMV")
        }
        onTabClick(POSITION_MAIN_LEFT)
    }

    /* 即将上映 点击事件 */
    private val onRightClickListener = View.OnClickListener {
        data?.artShowTabs?.takeIf { it.size > 1 }?.let {
            UTFacade.buttonClickWithoutPage("ArtMvCalendarTabClick", "tab", if (it[1]?.type == ArtShowTabVO.TYPE_HOT_FILM) "todayMV" else "FutureMV")

        }
        onTabClick(POSITION_MAIN_RIGHT)
    }

    /**
     * 更改想看状态
     */
    fun changeFavor(id: String?, favorCount: Int?, userShowStatus: Int?) {
        if (id == null || TextUtils.isEmpty(id) || getViewHolder() == null || userShowStatus == null
                || DataUtil.isNullOrEmpty(data?.artShowTabs) || adapter == null || viewHolder.videoExpressAdapter == null) {
            return
        }
        updateAdapter(id, favorCount, userShowStatus)
        for (i in data.artShowTabs.indices) {
            updateData(data.artShowTabs.get(i).shows, id, favorCount, userShowStatus)
        }
    }

    /**
     * 更新 adapter
     */
    private fun updateAdapter(id: String, favorCount: Int?, userShowStatus: Int) {
        val vh = getViewHolder() ?: return
        val count = vh.videoExpressAdapter?.itemCount ?: return
        (0..count).mapNotNull { vh.videoExpressAdapter?.getItem(it) }
                .filter { isSameId(it, id) }
                .filter { !((getUserShowStatus(it) == userShowStatus) && favorCount == null) }
                .forEach { changeUI(it, favorCount, userShowStatus) }
    }

    /**
     * 更改 UI
     */
    private fun changeUI(item: Any, favorCount: Int?, userShowStatus: Int) {
        val showMo = getShowMo(item) ?: return
        val oldShowStatus = showMo.getUserShowStatus()
        showMo.userShowStatus = userShowStatus
        if (favorCount != null && favorCount > 0) showMo.wantCount = favorCount
        else if (oldShowStatus != ShowMo.USER_SHOW_STATUS_WATCHED) { //从看过，到想看或者已想看，想看数不能变化
            if (userShowStatus == ShowMo.USER_SHOW_STATUS_WANT) showMo.wantCount++
            else if (userShowStatus == ShowMo.USER_SHOW_STATUS_DEFAULT) showMo.wantCount--
        }
        showMo.scoreAndFavor?.favorCount = showMo.wantCount
        refreshFavor(item)
    }

    /**
     * 获取 UserShowStatus
     */
    private fun getUserShowStatus(item: Any): Int? = when (item) {
        is CommunitySoonShowItem -> item.data.getUserShowStatus()
        else -> 0
    }

    /**
     * 判断 id 是否相同
     */
    private fun isSameId(item: Any, id: String): Boolean = when (item) {
        is CommunitySoonShowItem -> id == item.data.id
        else -> false
    }

    /**
     * 从 HomepageShowItem 或者 HomepageSoonShowItem 中获取 ShowMo
     */
    private fun getShowMo(item: Any): ShowMo? = when (item) {
        is CommunitySoonShowItem -> item.data
        else -> null
    }

    /**
     * 刷新 HomepageShowItem 或者 HomepageSoonShowItem
     */
    private fun refreshFavor(item: Any) = when (item) {
        is CommunitySoonShowItem -> item.refreshItem()
        else -> {
        }
    }

    /**
     * 更新数据
     */
    private fun updateData(showMoList: List<ShowMo>?, id: String, favorCount: Int?, userShowStatus: Int) {
        showMoList?.filter { it?.id == id }
                ?.forEach {
                    it.userShowStatus = userShowStatus
                    it.wantCount = favorCount ?: it.wantCount
                }
    }

    /**
     * 更新数据
     */
    private fun updateSoonListData(list: List<SoonShowTabListModuleVO>?, id: String, favorCount: Int?, userShowStatus: Int) {
        list?.flatMap { it?.showItems }
                ?.filter { it.id == id }
                ?.forEach {
                    it.userShowStatus = userShowStatus
                    it.wantCount = favorCount ?: it.wantCount
                }
    }
}
