package com.taobao.movie.android.app.oscar.ui.smartvideo.player.newplay.layer

import android.animation.Animator
import android.animation.ObjectAnimator
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import android.view.animation.LinearInterpolator
import com.taobao.movie.appinfo.util.DisplayUtil
import kotlin.math.min

class TimerPlayView : View, Animator.AnimatorListener {
    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr)

    private val rectF = RectF()
    private var angle = 0f
    private var anim: ObjectAnimator? = null
    private val INTERPOLATOR = LinearInterpolator()
    private var callback: TimerCallback? = null

    private val paint = Paint().apply {
        isAntiAlias = true
        strokeWidth = DisplayUtil.dp2px(1.5f)
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        rectF.set(left.toFloat() + paint.strokeWidth, top.toFloat() + paint.strokeWidth, right.toFloat() - paint.strokeWidth, bottom.toFloat() - paint.strokeWidth)
        paint.color = 0x99000000.toInt()
        paint.style = Paint.Style.FILL
        canvas?.drawCircle(rectF.centerX(), rectF.centerY(), getRadius(), paint)
        paint.color = Color.WHITE
        paint.style = Paint.Style.STROKE
        canvas?.drawArc(rectF, -90f, angle, false, paint)
    }

    fun setAngle(angle: Float) {
        this.angle = angle
        invalidate()
    }

    fun setStrokeWidth(strokeWidth: Float) {
        paint.strokeWidth = strokeWidth
    }

    fun startTimer(duration: Long, callback: TimerCallback?) {
        this.callback = callback
        anim = ObjectAnimator.ofFloat(this, "angle", angle, 360f).apply {
            this.duration = duration
            interpolator = INTERPOLATOR
            removeListener(this@TimerPlayView)
            addListener(this@TimerPlayView)
        }
        anim?.start()
    }

    fun startTimer() {
        anim?.let {
            val duration = 16f.coerceAtLeast(it.duration * (360 - angle) / 360)
            startTimer(duration.toLong(), callback)
        }
    }

    fun stopTimer() {
        anim?.isStarted?.let {
            if (it) anim?.cancel()
        }
    }

    fun cancelTimer() {
        anim?.isStarted?.let {
            if (it) anim?.cancel()
        }
        angle = 0f
        invalidate()
    }

    fun isStarted(): Boolean {
        return anim?.isStarted ?: false
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        stopTimer()
        angle = 0f
    }

    override fun onVisibilityChanged(changedView: View, visibility: Int) {
        super.onVisibilityChanged(changedView, visibility)
        if (visibility != VISIBLE) {
            cancelTimer()
        }
    }

    private fun getRadius(): Float {
        return min(rectF.width() / 2, rectF.height() / 2)
    }

    public interface TimerCallback {
        fun onTimerFinish()
    }

    override fun onAnimationRepeat(animation: Animator?) {
    }

    override fun onAnimationEnd(animation: Animator?) {
        callback?.onTimerFinish()
        anim?.removeListener(this)
    }

    override fun onAnimationCancel(animation: Animator?) {
        anim?.removeListener(this)
    }

    override fun onAnimationStart(animation: Animator?) {
    }
}