package com.taobao.movie.android.app.presenter.feed

import android.support.v4.app.Fragment
import android.text.TextUtils
import com.taobao.movie.android.app.oscar.biz.mtop.FeedDoubleRowListInfoResponseVo
import com.taobao.movie.android.app.presenter.common.LceeDefaultPresenter
import com.taobao.movie.android.app.presenter.feed.FeedBusinessPresenter.PAGE_FEEDTYPE_RECOMMOD
import com.taobao.movie.android.app.presenter.feed.FeedBusinessPresenter.getFeedAdLastIdCacheKey
import com.taobao.movie.android.app.vinterface.feed.IFeedDoubleRowView
import com.taobao.movie.android.common.Region.RegionExtServiceImpl
import com.taobao.movie.android.common.basebizserver.NetWorkHelper
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.integration.CommonConstants
import com.taobao.movie.android.integration.CommonConstants.RequestType.PullUpRefresh
import com.taobao.movie.android.net.mtop.rx.ApiException
import com.taobao.movie.android.net.mtop.rx.ShawShankApiObserver
import com.taobao.movie.appinfo.util.MovieCacheSet

class FeedRecommendPresenter(var currentPageFeedType : Int) : LceeDefaultPresenter<IFeedDoubleRowView>(){

    lateinit var regionExtBizService: RegionExtServiceImpl
    lateinit var getFeedsRequest: GetFeedsRequest
    var hasMore : Boolean = true
    var isLoading : Boolean = false

    override fun attachView(view: IFeedDoubleRowView?) {
        super.attachView(view)
        regionExtBizService = RegionExtServiceImpl()
        getFeedsRequest = GetFeedsRequest()
    }

    fun requestData(fetchType : String) {
        if (isLoading) {
            return
        }
        isLoading = true
        getFeedsRequest.fetchType = fetchType
        getFeedsRequest.needTopData = if (TextUtils.equals(fetchType, PullUpRefresh)) 0 else 1
        getFeedsRequest.cityCode = regionExtBizService.userRegion.cityCode
        getFeedsRequest.networkType = if (NetWorkHelper.getNetState()== NetWorkHelper.NET_4G) 2 else 1
        getFeedsRequest.latestAdIdList = MovieCacheSet.getInstance().getString(getFeedAdLastIdCacheKey())
        getFeedsRequest.subscribe(getAutomaticResource(),object : ShawShankApiObserver.ApiConsumer<FeedDoubleRowListInfoResponseVo>(){

            override fun onBefore() {
                view.showFeedLoadingView(PAGE_FEEDTYPE_RECOMMOD,false)
            }

            override fun onSuccess(resp: FeedDoubleRowListInfoResponseVo?) {
                isLoading = false
                if (isViewAttached) {
                    hasMore = !resp?.feedData.isNullOrEmpty() || !resp?.topData.isNullOrEmpty()
                    if (!resp?.feedData.isNullOrEmpty() || !resp?.topData.isNullOrEmpty()) {
                        view.showFeedContentView(currentPageFeedType, false, resp, fetchType)
                    } else {
                        view.showFeedEmpty(currentPageFeedType, fetchType)
                    }
                }
            }

            override fun onFail(e: ApiException) {
                isLoading = false
                if (isViewAttached) {
                    view.showFeedError(currentPageFeedType,fetchType,e.resultCode,e.returnCode,e.message)
                }
            }
        })
    }

    fun fetchType() = getFeedsRequest.fetchType

    override fun isViewAttached(): Boolean {
        val isViewCanTouch = super.isViewAttached()
        if (isViewCanTouch && view is Fragment) {//此处Fragment为v4包下的
            val fragment = view as Fragment
            return UiUtils.isFragmentStateValid(fragment)
        }
        return isViewCanTouch
    }

    override fun hasMore(): Boolean {
        return hasMore
    }

    fun doLoadMore() : Boolean {
        if (isLoading || !hasMore) {
            return false
        }
        requestData(CommonConstants.RequestType.PullUpRefresh)
        return true
    }
}