package com.taobao.movie.android.app.spring

import android.arch.lifecycle.Lifecycle
import android.arch.lifecycle.LifecycleObserver
import android.arch.lifecycle.LifecycleOwner
import android.arch.lifecycle.OnLifecycleEvent
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade

/**
 * 花支跳端埋点工具类
 * @author yx
 * @date 19_12_25
 */
class AlipayJumpHelper private constructor() : LifecycleObserver {

    private var status = INIT

    private var pageCode: String? = null

    private var owner: LifecycleOwner? = null

    private var pageOut = false



    private fun removeObserver() {
        owner?.lifecycle?.removeObserver(this)
        owner = null
    }

    private fun registerObserver() {
        owner?.lifecycle?.removeObserver(this)
        owner?.lifecycle?.addObserver(this)
    }

    /**
     * 跳转支付宝成功
     */
    fun jumpAlipay(owner: LifecycleOwner, success: Boolean, pageCode: String?) {
        this.pageCode = pageCode
        this.owner = owner
        if (success) {
            status = JUMP_SUCCESS
            uploadEvent()
            registerObserver()
        } else {
            status = JUMP_FAILED
            uploadEvent()
        }
    }

    private fun uploadEvent() {
        if (status == JUMP_SUCCESS || status == JUMP_FAILED) {
            val install = if (status == JUMP_SUCCESS) "1" else "0"
            UTFacade.extEvent("Page_All_TppSCardJumpOut", "install", install, "pagecode", pageCode)
            return
        }
        if (status == JUMP_RETURN_SUCCESS || status == JUMP_RETURN_FAILED) {
            val suc = if (status == JUMP_RETURN_SUCCESS) "1" else "0"
            UTFacade.extEvent("Page_All_TppSCardJumpBack", "success", suc, "pagecode", pageCode)
            return
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    fun unregister() {
        if (status == JUMP_SUCCESS && owner?.lifecycle?.currentState == Lifecycle.State.RESUMED && pageOut) {
            returnFailed()
            removeObserver()
            pageOut = false
        }
    }


    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    fun pageOut() {
        pageOut = true
    }

    /**
     * 跳转支付宝返回成功
     */
    fun returnSuccess() {
        if (status == JUMP_SUCCESS) {
            status = JUMP_RETURN_SUCCESS
            uploadEvent()
            removeObserver()
        }
        reset()
    }

    /**
     * 跳转支付宝返回失败
     */
    private fun returnFailed() {
        if (status == JUMP_SUCCESS) {
            status = JUMP_RETURN_FAILED
            uploadEvent()
            removeObserver()
        }
        reset()
    }

    fun reset() {
        status = INIT
        pageOut = false
    }

    companion object {
        val instance: AlipayJumpHelper by lazy { AlipayJumpHelper() }
        private const val INIT = 0
        private const val JUMP_SUCCESS = 1
        private const val JUMP_FAILED = 2
        private const val JUMP_RETURN_SUCCESS = 3
        private const val JUMP_RETURN_FAILED = 4
    }

}