package com.taobao.movie.android.app.ui.common

import android.content.Context
import android.graphics.Bitmap
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.taobao.movie.android.common.util.DisplayMetrics
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.widget.imageloader.CommonImageProloadUtil
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.utils.Opt
import com.taobao.movie.android.sdk.infrastructure.glide.GlideApp
import com.taobao.movie.appinfo.util.DisplayUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class SpringOpenCardContentItem(var data: OpenCardContentData?, val context: Context, val refresh: RefreshClick) {

    var itemView: View? = null

    init {
        itemView = LayoutInflater.from(context).inflate(getLayoutId(), null, false)
    }

    fun onItemBind() {
        var subtitle: TextView? = itemView?.findViewById(R.id.tv_spring_dialog_subtitle)
        var errorView: ImageView? = itemView?.findViewById(R.id.iv_spring_dialog_content_error)
        var contentView: MoImageView? = itemView?.findViewById(R.id.iv_spring_dialog_content)
        var loadingView: View? = itemView?.findViewById(R.id.pb_loading_item)
        var desView: TextView? = itemView?.findViewById(R.id.tv_loading_des)
        var refreshButton: TextView? = itemView?.findViewById(R.id.btn_spring_open_card_refresh)
        var loadingContainView: View? = itemView?.findViewById(R.id.cl_open_card_loading_contain)
        contentView?.setUseOriginalUrl(true)
        when (data?.status) {
            0 -> {
                subtitle?.visibility = View.GONE
                errorView?.visibility = View.GONE
                contentView?.visibility = View.GONE
                loadingView?.visibility = View.VISIBLE
                desView?.visibility = View.GONE
                refreshButton?.visibility = View.GONE
                loadingContainView?.visibility = View.VISIBLE
            }
            1 -> {
                if (!TextUtils.isEmpty(data?.message)) {
                    subtitle?.visibility = View.VISIBLE
                    subtitle?.text = data?.message
                } else {
                    subtitle?.visibility = View.GONE
                }
                errorView?.visibility = View.GONE
                contentView?.visibility = View.VISIBLE
                loadingContainView?.visibility = View.GONE
                contentView?.url = data?.url
                loadingView?.visibility = View.GONE
                desView?.visibility = View.GONE
                refreshButton?.visibility = View.GONE
            }
            2 -> {
                subtitle?.visibility = View.GONE
                errorView?.visibility = View.VISIBLE
                contentView?.visibility = View.GONE
                loadingView?.visibility = View.GONE
                desView?.visibility = View.VISIBLE
                loadingContainView?.visibility = View.VISIBLE
                desView?.text = data?.errorMessage
                refreshButton?.visibility = View.VISIBLE
                refreshButton?.text = "刷新"
                errorView?.setImageResource(R.drawable.sys_network_error_icon)
                refreshButton?.setOnClickListener {
                    data?.status = 0
                    onItemBind()
                    refresh.dialogRefresh()
                }
            }
            else ->{
                subtitle?.visibility = View.GONE
                errorView?.visibility = View.VISIBLE
                contentView?.visibility = View.GONE
                loadingView?.visibility = View.GONE
                desView?.visibility = View.VISIBLE
                loadingContainView?.visibility = View.VISIBLE
                desView?.text = data?.errorMessage
                refreshButton?.visibility = View.VISIBLE
                CommonImageProloadUtil.loadImageSrc(errorView, CommonImageProloadUtil.GlideImageURL.OPEN_CARD_ERROR)
                refreshButton?.text = "知道了"
                refreshButton?.setOnClickListener {
                    refresh.dialogDismiss()
                }
            }

        }
    }

    fun update(data: OpenCardContentData?) {
        this.data = data
        onItemBind()
    }

    fun updateImageSize(width: Int, height: Int) {
        var contentView: MoImageView? = itemView?.findViewById(R.id.iv_spring_dialog_content)
        var layoutParams = contentView?.layoutParams as LinearLayout.LayoutParams
        layoutParams?.height = (height * 1.0f / width * (DisplayUtil.getDisplayWidthInPx() - DisplayUtil.dp2pxInt(30f))).toInt()
        contentView.layoutParams = layoutParams
    }


    fun getLayoutId(): Int {
        return R.layout.spring_open_card_dialog_content
    }


    data class OpenCardContentData(var status: Int?, var message: String?, var url: String?, var errorMessage: String?)

    interface RefreshClick {

        fun dialogRefresh()

        fun dialogDismiss()
    }

}