@file:JvmName("CouponUtil")

package com.taobao.movie.android.app.ui.product.util

import android.content.Context
import android.content.Intent
import android.os.Bundle
import com.taobao.movie.android.app.order.ui.activity.SalesOrderActivity
import com.taobao.movie.android.app.oscar.ui.cinema.activity.ScheduleListRootActivity
import com.taobao.movie.android.app.presenter.order.SalesOrderPresenter
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.scheme.MovieNavigator
import com.taobao.movie.android.integration.product.model.BizCouponsMo
import com.taobao.movie.android.integration.product.model.FCodeLabel
import com.taobao.movie.appinfo.util.DataUtil

fun gotoExchangeSaleCoupon(context: Context, data: BizCouponsMo) {
    if (data.type != BizCouponsMo.PRESALE || data.target != FCodeLabel.COUPON_SALEGOODS) {
        return
    }
    if (!data.url.isNullOrEmpty()) {
        MovieNavigator.handleUrl(context, data.url, false)
        return
    }
    Intent().apply {
        when {
            data.cinemaIds == null || data.cinemaIds.size != 1 -> {
                //跳去首页
                MovieNavigator.gotoHomeAction(context, Bundle(), IntentConstants.Action.ACTION_HOME_PAGE)
            }
            DataUtil.getListSize(data.saleIds) == 1 -> {
                //跳去小食确认订单
                putExtra(SalesOrderPresenter.SALES_CINEMA_SALE_INFO, data.saleIds[0].toString() + ":1")
                putExtra(SalesOrderPresenter.SALES_CINEMA_ID, data.cinemaIds[0])
                putExtra(SalesOrderPresenter.SALES_PRESALE_CODE, data.code)
                setClass(context, SalesOrderActivity::class.java)
                context.startActivity(this)
            }
            else -> {
                //跳去场次页
                putExtra(IntentConstants.KEY_OSCAR_CINEMA_ID, data.cinemaIds[0])
                putExtra(IntentConstants.KEY_OSCAR_CINEMA_NAME, data.cinemaName)
                setClass(context, ScheduleListRootActivity::class.java)
                context.startActivity(this)
            }
        }

    }

}

fun isValidCoupon(data: BizCouponsMo):Boolean = with(data) {
    when (type) {
        BizCouponsMo.PRESALE ->
            BizCouponsMo.PresaleStatus.valueOf(status).let {
                return@with it == BizCouponsMo.PresaleStatus.CAN_SEAT || it == BizCouponsMo.PresaleStatus.LOCKING || it == BizCouponsMo.PresaleStatus.NO_SCHEDULE
                        || it == BizCouponsMo.PresaleStatus.CANNOT_USE || it == BizCouponsMo.PresaleStatus.NOT_OPEN
            }
        BizCouponsMo.COUPON ->
            BizCouponsMo.DiscountStatus.valueOf(status).let {
                return@with it == BizCouponsMo.DiscountStatus.NORMAL || it == BizCouponsMo.DiscountStatus.LOCKED
            }
        BizCouponsMo.COUPON_BATCH ->
            BizCouponsMo.DiscountStatus.valueOf(status).let {
                return@with it == BizCouponsMo.DiscountStatus.NORMAL || it == BizCouponsMo.DiscountStatus.NOT_OPEN || it == BizCouponsMo.DiscountStatus.APPROVED
            }
    }
    return@with false
}