package com.taobao.movie.android.app.ui.schedule.widget

import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.graphics.Color
import android.os.Bundle
import android.os.Handler
import android.text.Spannable
import android.text.SpannableStringBuilder
import android.text.style.StrikethroughSpan
import android.view.*
import android.view.animation.AccelerateInterpolator
import android.widget.LinearLayout
import com.taobao.movie.android.app.ui.schedule.event.SchedulePageRefreshEvent
import com.taobao.movie.android.app.ui.schedule.event.SchedulePageRefreshFinishEvent
import com.taobao.movie.android.arch.obtainViewModel
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.Region.RegionExtServiceImpl
import com.taobao.movie.android.common.userprofile.UserProfileWrapper
import com.taobao.movie.android.commonui.BaseDialogFragment
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.commonui.utils.ViewUtils
import com.taobao.movie.android.commonutil.LocalEventBus
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.viewmodel.ExchangableCouponMo
import com.taobao.movie.android.integration.product.model.FCodeLabel
import com.taobao.movie.android.integration.profile.model.LotteryDrawResultModel
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DataUtil
import com.taobao.movie.appinfo.util.ToastUtil
import de.greenrobot.event.EventBus
import kotlinx.android.synthetic.main.dialog_lottery.*
import kotlinx.android.synthetic.main.dialog_lottery.fl_win
import kotlinx.android.synthetic.main.dialog_lottery.iv_lottery_close_btn
import kotlinx.android.synthetic.main.dialog_lottery.iv_ribbons
import kotlinx.android.synthetic.main.dialog_lottery.ll_amount
import kotlinx.android.synthetic.main.dialog_lottery.ll_draw_btn
import kotlinx.android.synthetic.main.dialog_lottery.ll_initial
import kotlinx.android.synthetic.main.dialog_lottery.ll_lose
import kotlinx.android.synthetic.main.dialog_lottery.progress_loading
import kotlinx.android.synthetic.main.dialog_lottery.tv_amount
import kotlinx.android.synthetic.main.dialog_lottery.tv_draw_btn
import kotlinx.android.synthetic.main.dialog_lottery.tv_draw_hint
import kotlinx.android.synthetic.main.dialog_lottery.tv_happy_coin_num
import kotlinx.android.synthetic.main.dialog_lottery.tv_hint
import kotlinx.android.synthetic.main.dialog_lottery.tv_multiple_hint

const val KEY_HAPPY_COIN_NUM = "happy_coin_num"

open class LotteryDialogFragment : BaseDialogFragment() {

    protected lateinit var couponMo: ExchangableCouponMo
    protected lateinit var showId: String
    private lateinit var spmValue: String
    protected var happyCoinNum: Int = 0
    protected val handler = Handler()
    private lateinit var lotteryDrawViewModel:LotteryDrawViewModel

    protected var luckyType = "1"

    companion object {
        @JvmStatic
        fun newInstance(couponMo: ExchangableCouponMo, happyCoinNum: Int, spm: String, showId: String) = LotteryDialogFragment().apply {
            arguments = Bundle().apply {
                putString(IntentConstants.KEY_EXCHANGE_WINDOW_SPM_VALUE, spm)
                putSerializable(IntentConstants.KEY_EXCHANGE_WINDOW_DATA, couponMo)
                putInt(KEY_HAPPY_COIN_NUM, happyCoinNum)
                putString(IntentConstants.KEY_SHOWID, showId)
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        couponMo = arguments!![IntentConstants.KEY_EXCHANGE_WINDOW_DATA] as ExchangableCouponMo
        spmValue = arguments!!.getString(IntentConstants.KEY_EXCHANGE_WINDOW_SPM_VALUE, "")
        showId = arguments!!.getString(IntentConstants.KEY_SHOWID, "")
        happyCoinNum = arguments!!.getInt(KEY_HAPPY_COIN_NUM, 0)
        lotteryDrawViewModel = obtainViewModel(LotteryDrawViewModel::class.java)
    }

    override fun getContentViewId(): Int = R.layout.dialog_lottery

    override fun onConfigWindow(lp: WindowManager.LayoutParams) {
        lp.gravity = Gravity.CENTER
        lp.width = ViewUtils.dpToPx(270).toInt()
        lp.height = ViewGroup.LayoutParams.WRAP_CONTENT
        isCancelable = false
    }

    override fun onDestroyView() {
        super.onDestroyView()
        handler.removeCallbacksAndMessages(null)
    }

    override fun setupView(view: View) {
        tv_amount.text = DataUtil.getFormatPriceClean(couponMo.costPrice.toLong())
        tv_hint.text = getDiscountStr(couponMo.costPrice)
        iv_lottery_close_btn.setOnClickListener {
            buttonClickUT("2")
            dismiss()
        }
        ll_draw_btn.setOnClickListener {
            onClickDraw(true)
        }
        tv_exchange_btn.setOnClickListener {
            onClickDraw(false)
        }
        if (!couponMo.tagDescInPop.isNullOrEmpty()) {
            tv_discount_hint_label.visibility = View.VISIBLE
            tv_discount_hint_label.text = couponMo.tagDescInPop
        }
        when (couponMo.fCodeLabel.luckyMultipleStatus) {
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY -> {
                tv_happy_coin_num.visibility = View.GONE
                tv_draw_hint.text = getString(R.string.lottery_draw_hint_without_happy_coin)
                tv_exchange_btn.visibility = View.GONE
                (tv_multiple_hint.layoutParams as LinearLayout.LayoutParams).topMargin += ViewUtils.dpToPx(10).toInt()
                (fl_draw_btn.layoutParams as LinearLayout.LayoutParams).topMargin += ViewUtils.dpToPx(25).toInt()
            }
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_ENOUGH -> {
                showCoinLotteryUI(true)
            }
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_NOT_ENOUGH -> {
                showCoinLotteryUI(false)
            }
        }
    }

    private fun showCoinLotteryUI(coinEnough: Boolean) {
        tv_draw_hint.text = getString(R.string.lottery_draw_hint_with_happy_coin, happyCoinNum)
        if (couponMo.oriHappyCoinInPop > 0) {
            tv_happy_coin_num.text = SpannableStringBuilder("${couponMo.activityHappyCoinInPop} ").apply {
                val start = length
                append("${couponMo.oriHappyCoinInPop}")
                setSpan(StrikethroughSpan(), start, length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
        } else {
            tv_happy_coin_num.text = "${couponMo.activityHappyCoinInPop}"
        }
        if (!coinEnough) {
            tv_draw_btn.setTextColor(Color.parseColor("#66895A00"))
            tv_happy_coin_num.setTextColor(Color.parseColor("#66895A00"))
        }
    }

    fun onClickDraw(drawLuckyMultiple: Boolean) {
        if (drawLuckyMultiple &&
                couponMo.fCodeLabel.luckyMultipleStatus == FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_NOT_ENOUGH) {
            ToastUtil.showToast(R.string.lottery_draw_coin_not_enough)
        } else {
            startLoadingAnimation()
            lotteryDrawViewModel.doLottery(spmValue = spmValue,
                    lotteryMixId = couponMo.lotteryMixId,
                    luckyMultiple = drawLuckyMultiple,
                    onSuccess = {result ->
                        EventBus.getDefault().post(SchedulePageRefreshEvent())
                        LocalEventBus.getDefault().register(this,
                                LocalEventBus.EventReceiver<SchedulePageRefreshFinishEvent> {
                                    if (UiUtils.isFragmentStateValid(this)) {
                                        stopLoadingAnimation()
                                        if (drawLuckyMultiple) {
                                            onDrawSuccess(result)
                                        } else {
                                            dismiss()
                                        }
                                    }
                        })
                    },
                    onFailed = {
                        if (luckyType == "3") {
                            EventBus.getDefault().post(SchedulePageRefreshEvent())
                        }
                        onDrawFailed("小二很忙，请稍后再试")
                    },
                    onAlreadyDrawn = {
                        EventBus.getDefault().post(SchedulePageRefreshEvent())
                        onAlreadyDrawn("你已领取该券，请刷新")
                    })
        }
        buttonClickUT(when (couponMo.fCodeLabel.luckyMultipleStatus) {
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY -> if( "1" == luckyType) "1" else  "5"
            else -> if (drawLuckyMultiple && "1" == luckyType) "3" else if ("1" == luckyType) "4" else "6"
        })
    }

    open fun onAlreadyDrawn(msg: String){
        stopLoadingAnimation()
        ToastUtil.showToast(msg)
        UTFacade.buttonClickWithoutPage("CouponDrawStatusClick",
                "type", "${couponMo.fundSource}",
                "vip_type", "2",
                "lucky_type", luckyType,
                "draw_result", "2",
                "show_id", showId,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode)
    }

    open fun startLoadingAnimation() {
        fl_draw_btn.visibility = View.GONE
        tv_exchange_btn.visibility = View.GONE
        progress_loading.visibility = View.VISIBLE
        iv_lottery_close_btn.isEnabled = false
    }

    open fun stopLoadingAnimation() {
        fl_draw_btn.visibility = View.VISIBLE
        if (couponMo.fCodeLabel.luckyMultipleStatus != FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY) {
            tv_exchange_btn.visibility = View.VISIBLE
        }
        progress_loading.visibility = View.GONE
        iv_lottery_close_btn.isEnabled = true
    }

    fun buttonClickUT(clickFlag: String) {
        UTFacade.buttonClickWithoutPage("LuckyCouponClick",
                "type", "${couponMo.fundSource}",
                "click", clickFlag,
                "show_id", showId,
                "coupon_code", couponMo.code,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode,
                "lucky_happy_coin", if (couponMo.fCodeLabel.luckyMultipleStatus != FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY) "1" else "2",
                "happy_coin_status", if (couponMo.fCodeLabel.luckyMultipleStatus == FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_ENOUGH) "1" else "2")
    }

    open fun showWinUI(newPrice: Int, multipleTimes: Float) {
        ll_initial.visibility = View.GONE
        iv_ribbons.visibility = View.VISIBLE
        fl_win.visibility = View.VISIBLE

        iv_ribbons.imageType = MoImageView.ImageViewType.GIF
        iv_ribbons.setLocalDrawable(R.drawable.bg_lottery_ribbons_gif)
        handler.postDelayed({
            iv_ribbons.visibility = View.GONE
        }, 1200)
        tv_multiple_times.text = "${multipleTimes}倍"

        tv_amount.text = DataUtil.getFormatPriceClean(newPrice.toLong())
        tv_hint.text = getDiscountStr(newPrice)

        //animation for winner
        ObjectAnimator.ofFloat(ll_amount, "alpha", 0F, 1F).setDuration(6*1000/24).start()
        ObjectAnimator.ofFloat(tv_hint, "alpha", 0F, 1F).setDuration(6*1000/24).start()
        ObjectAnimator.ofFloat(fl_win, "alpha", 0F, 1F).setDuration(9*1000/24).start()
        val animX = ObjectAnimator.ofFloat(tv_multiple_times, "scaleX", 0.06F, 1.1F, 1.0F).setDuration(12*1000/24)
        val animY = ObjectAnimator.ofFloat(tv_multiple_times, "scaleY", 0.06F, 1.1F, 1.0F).setDuration(12*1000/24)
        AnimatorSet().apply {
            playTogether(animX, animY)
            interpolator = AccelerateInterpolator()
            start()
        }
    }

    fun getDiscountStr(price: Int):String {
        if (couponMo.fCodeLabel.discountSeatDesc == null) {
            return ""
        }
        val splitDiscountStr = couponMo.fCodeLabel.discountSeatDesc.split("\n")
        if (splitDiscountStr.isNotEmpty()) {
            return getString(R.string.lottery_discount_description,
                    DataUtil.getFormatPriceClean(price.toLong())) + "\n" + splitDiscountStr[1]
        }
        return couponMo.fCodeLabel.discountSeatDesc
    }

    open fun showLoseUI() {
        ll_initial.visibility = View.GONE
        ll_lose.visibility = View.VISIBLE
        ObjectAnimator.ofFloat(ll_lose, "alpha", 0F, 1F).setDuration(6*1000/24).start()
    }

    private fun onDrawSuccess(resp: LotteryDrawResultModel) {
        if (resp.rewards.isEmpty()) {
            showLoseUI()
            return
        }
        var win = false
        if (luckyType == "1" && resp.rewards[0].extResults.isNotEmpty() && resp.rewards[0].extResults[0].multiple > 0) {
            win = true
            showWinUI(resp.rewards[0].costPrice, resp.rewards[0].extResults[0].multiple)
        } else if (luckyType == "3") {
            win = true
            showWinUI(resp.rewards[0].costPrice, 0f)
        } else {
            showLoseUI()
        }
        UTFacade.buttonClickWithoutPage("CouponDrawStatusClick",
                "type", "${couponMo.fundSource}",
                "vip_type", "2",
                "lucky_type", luckyType,
                "draw_result", if (win) "3" else "4",
                "show_id", showId,
                "coupon_code", resp.rewards[0].code,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode,
                "track_info", resp.trackInfo)
    }

    open fun onDrawFailed(msg: String) {
        stopLoadingAnimation()
        ToastUtil.showToast(msg)
        UTFacade.buttonClickWithoutPage("CouponDrawStatusClick",
                "type", "${couponMo.fundSource}",
                "vip_type", "2",
                "lucky_type", luckyType,
                "draw_result", "2",
                "show_id", showId,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode)
    }
}