package com.taobao.movie.android.app.ui.schedule.widget

import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.os.Bundle
import android.text.Spannable
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.StrikethroughSpan
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.view.animation.AccelerateInterpolator
import com.taobao.movie.android.app.ui.schedule.event.SchedulePageRefreshFinishEvent
import com.taobao.movie.android.common.IntentConstants
import com.taobao.movie.android.common.Region.RegionExtServiceImpl
import com.taobao.movie.android.common.userprofile.UserProfileWrapper
import com.taobao.movie.android.commonui.moimage.MoImageView
import com.taobao.movie.android.commonui.utils.UiUtils
import com.taobao.movie.android.commonui.utils.ViewUtils
import com.taobao.movie.android.commonutil.LocalEventBus
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.oscar.viewmodel.ExchangableCouponMo
import com.taobao.movie.android.integration.product.model.FCodeLabel
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DataUtil
import com.taobao.movie.appinfo.util.DisplayUtil
import com.taobao.movie.appinfo.util.ToastUtil
import kotlinx.android.synthetic.main.dialog_schedule_luck.*
import kotlinx.android.synthetic.main.dialog_schedule_luck.fl_win
import kotlinx.android.synthetic.main.dialog_schedule_luck.iv_lottery_close_btn
import kotlinx.android.synthetic.main.dialog_schedule_luck.iv_ribbons
import kotlinx.android.synthetic.main.dialog_schedule_luck.ll_draw_btn
import kotlinx.android.synthetic.main.dialog_schedule_luck.ll_initial
import kotlinx.android.synthetic.main.dialog_schedule_luck.progress_loading
import kotlinx.android.synthetic.main.dialog_schedule_luck.tv_draw_btn
import kotlinx.android.synthetic.main.dialog_schedule_luck.tv_draw_hint
import kotlinx.android.synthetic.main.dialog_schedule_luck.tv_happy_coin_num

/**
 * Created by yuxin on 2020-02-03
 **/
class LuckDialogFragment : LotteryDialogFragment() {


    companion object {
        @JvmStatic
        fun newInstance(couponMo: ExchangableCouponMo, happyCoinNum: Int, spm: String, showId: String) = LuckDialogFragment().apply {
            arguments = Bundle().apply {
                putString(IntentConstants.KEY_EXCHANGE_WINDOW_SPM_VALUE, spm)
                putSerializable(IntentConstants.KEY_EXCHANGE_WINDOW_DATA, couponMo)
                putInt(KEY_HAPPY_COIN_NUM, happyCoinNum)
                putString(IntentConstants.KEY_SHOWID, showId)
            }
        }
    }

    override fun getContentViewId(): Int = R.layout.dialog_schedule_luck


    override fun setupView(view: View) {
        luckyType = "3"
        iv_lottery_close_btn.setOnClickListener {
            buttonClickUT("2")
            dismiss()
        }
        ll_draw_btn.setOnClickListener {
            onClickDraw(true)
        }
        tv_draw_btn.text = getString(R.string.luck_draw_hint)
        iv_luck_film_poster.url = couponMo.showPosterUrl
        if (TextUtils.isEmpty(iv_luck_film_poster.url)) {
            v_luck_poster_top_pic.visibility = View.GONE
        }
        when (couponMo.fCodeLabel.luckyMultipleStatus) {
            //普通拼手气
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY -> {
                tv_happy_coin_num.visibility = View.GONE
                tv_draw_hint.text = getString(R.string.luck_draw_tag_hint)
            }
            //积分拼手气
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_ENOUGH -> {
                showCoinLotteryUI(true)
                tv_draw_hint.text = getString(R.string.luck_draw_tag_score_hint, happyCoinNum)
            }
            //积分拼手气但积分不足
            FCodeLabel.LUCKY_MULTIPLE_STATUS_LOTTERY_WITH_COIN_NOT_ENOUGH -> {
                showCoinLotteryUI(false)
                tv_draw_hint.text = getString(R.string.luck_draw_tag_score_hint, happyCoinNum)
            }
        }
    }

    override fun onConfigWindow(lp: WindowManager.LayoutParams) {
        lp.gravity = Gravity.CENTER
        lp.width = ViewUtils.dpToPx(290).toInt()
        lp.height = ViewGroup.LayoutParams.WRAP_CONTENT
        isCancelable = false
    }

    private fun showCoinLotteryUI(coinEnough: Boolean) {
        if (couponMo.oriHappyCoinInPop > 0 && couponMo.activityHappyCoinInPop < couponMo.oriHappyCoinInPop) {
            tv_happy_coin_num.text = SpannableStringBuilder("${couponMo.activityHappyCoinInPop} ").apply {
                val start = length
                append("${couponMo.oriHappyCoinInPop}")
                setSpan(StrikethroughSpan(), start, length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
        } else {
            tv_happy_coin_num.text = "${couponMo.activityHappyCoinInPop}"
        }
    }


    override fun startLoadingAnimation() {
        ll_draw_btn.visibility = View.GONE
        progress_loading.visibility = View.VISIBLE
        iv_lottery_close_btn.isEnabled = false
    }


    override fun stopLoadingAnimation() {
        ll_draw_btn.visibility = View.VISIBLE
        progress_loading.visibility = View.GONE
        iv_lottery_close_btn.isEnabled = true
    }


    override fun showWinUI(newPrice: Int, multipleTimes: Float) {
        removePoster()
        v_luck_poster_top_pic.visibility = View.GONE
        ll_initial.visibility  = View.GONE
        fl_win.visibility = View.VISIBLE
        rl_luck_top.visibility = View.VISIBLE
        iv_ribbons.imageType = MoImageView.ImageViewType.GIF
        iv_ribbons.setLocalDrawable(R.drawable.bg_lottery_ribbons_gif)
        handler.postDelayed({
            iv_ribbons.visibility = View.GONE
        }, 1200)

        tv_amount.text = DataUtil.getFormatPriceClean(newPrice.toLong())
        tv_hint.text = getDiscountStr(newPrice)

        val animX = ObjectAnimator.ofFloat(fl_win, "scaleX", 0.06F, 1.1F, 1.0F).setDuration(12*1000/24)
        val animY = ObjectAnimator.ofFloat(fl_win, "scaleY", 0.06F, 1.1F, 1.0F).setDuration(12*1000/24)
        AnimatorSet().apply {
            playTogether(animX, animY)
            interpolator = AccelerateInterpolator()
            start()
        }
    }

    override fun showLoseUI() {
        removePoster()
        v_luck_poster_top_pic.visibility = View.GONE
                ll_initial.visibility  = View.GONE
        rl_luck_top.visibility = View.INVISIBLE
        ll_lose_logo.visibility = View.VISIBLE
        ll_lose.visibility = View.VISIBLE
        ObjectAnimator.ofFloat(ll_lose, "alpha", 0F, 1F).setDuration(6*1000/24).start()
    }


    private fun removePoster() {
        val animator = ObjectAnimator.ofFloat(iv_luck_film_poster, "translationY", 0F, -DisplayUtil.dp2px(210f)).setDuration(333)
        animator.interpolator = AccelerateInterpolator()
        animator.start()

    }


    override fun onDrawFailed(msg: String) {
        LocalEventBus.getDefault().register(this,
                LocalEventBus.EventReceiver<SchedulePageRefreshFinishEvent> {
                    if (UiUtils.isFragmentStateValid(this)) {
                        stopLoadingAnimation()
                        showLoseUI()
                    }
                })
        UTFacade.buttonClickWithoutPage("CouponDrawStatusClick",
                "type", "${couponMo.fundSource}",
                "vip_type", "2",
                "lucky_type", luckyType,
                "draw_result", "2",
                "show_id", showId,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode)
    }



    override fun onAlreadyDrawn(msg: String) {
        stopLoadingAnimation()
        ToastUtil.showToast(msg)
        UTFacade.buttonClickWithoutPage("CouponDrawStatusClick",
                "type", "${couponMo.fundSource}",
                "vip_type", "2",
                "lucky_type", luckyType,
                "draw_result", "2",
                "show_id", showId,
                "vip_level", UserProfileWrapper.getInstance().currentUserLevel,
                "lottery_mix_id", couponMo.lotteryMixId,
                "city", RegionExtServiceImpl().getUserRegion().cityCode)
    }
}