package com.taobao.movie.android.app.ui.schedule.widget

import android.app.Activity
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import android.widget.PopupWindow
import com.taobao.listitem.recycle.CustomRecyclerAdapter
import com.taobao.movie.android.app.ui.schedule.items.PricePopupFestivalItem
import com.taobao.movie.android.app.ui.schedule.items.PricePopupItem
import com.taobao.movie.android.app.ui.schedule.model.PricePopupFestivalModel
import com.taobao.movie.android.home.R
import com.taobao.movie.android.integration.order.model.ItemDetailMo
import com.taobao.movie.appinfo.util.DisplayUtil

/**
 * 场次页价格明细弹窗
 */
class PricePopupWindow(context: Activity, private val locationView: View) : PopupWindow(context) {
    private val priceRecycler: RecyclerView
    private val rootView: View = LayoutInflater.from(context).inflate(R.layout.price_popup_window, null, false)
    private val context: Activity
    private val arrowUp: ImageView
    private val arrowDown: ImageView
    private val adapter: CustomRecyclerAdapter
    private val itemHeight: Int
    private val screenHeight: Int
    private var xoffset: Int
    private var yoffset: Int
    private val padding: Int

    init {
        priceRecycler = rootView.findViewById(R.id.rv_price)
        arrowUp = rootView.findViewById(R.id.iv_arrow_up)
        arrowDown = rootView.findViewById(R.id.iv_arrow_down)
        adapter = CustomRecyclerAdapter(context)
        priceRecycler.adapter = adapter
        priceRecycler.layoutManager = LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        itemHeight = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 24f, DisplayUtil.getDisplayMetrics()).toInt()
        screenHeight = DisplayUtil.getDisplayHeightInPx() - DisplayUtil.getNavigationBarHeight()
        xoffset = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 79f, DisplayUtil.getDisplayMetrics()).toInt()
        yoffset = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12f, DisplayUtil.getDisplayMetrics()).toInt()
        padding = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 16f, DisplayUtil.getDisplayMetrics()).toInt()
        this.context = context
    }

    fun updateOffset() {
        xoffset = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 62f, DisplayUtil.getDisplayMetrics()).toInt()
        yoffset = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 22f, DisplayUtil.getDisplayMetrics()).toInt()
    }

    @JvmOverloads
    fun show(itemDetails: List<ItemDetailMo?>, cardCouponDiscountTag: String? = null, festivalPriceTag: String? = null, festivalTip: String? = null) {
        width = WindowManager.LayoutParams.WRAP_CONTENT
        height = WindowManager.LayoutParams.WRAP_CONTENT
        setContentView(rootView)
        isFocusable = true
        isOutsideTouchable = true
        isClippingEnabled = true
        update()
        setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        for (itemDetailMo in itemDetails) {
            adapter.addItem(PricePopupItem(itemDetailMo))
        }
        var totalHeight = itemHeight * itemDetails.size
        cardCouponDiscountTag?.let {
            adapter.addItem(PricePopupItem(ItemDetailMo().apply {
                priceDesc = cardCouponDiscountTag
            }))
            totalHeight += itemHeight
        }
        festivalPriceTag?.let {
            adapter.addItem(PricePopupFestivalItem(PricePopupFestivalModel(festivalPriceTag, festivalTip)))
            totalHeight += DisplayUtil.dp2px(40.0F).toInt()
        }
        val location = IntArray(2)
        locationView.getLocationOnScreen(location)
        if (location[1] + totalHeight + yoffset + padding > screenHeight) {
            arrowUp.visibility = View.GONE
            arrowDown.visibility = View.VISIBLE
            super.showAtLocation(locationView, Gravity.NO_GRAVITY, location[0] - xoffset,
                    location[1] - totalHeight - padding)
        } else {
            arrowUp.visibility = View.VISIBLE
            arrowDown.visibility = View.GONE
            super.showAtLocation(locationView, Gravity.NO_GRAVITY, location[0] - xoffset, location[1] + yoffset)
        }
    }

}