package com.taobao.movie.android.arch.items

import android.support.v7.recyclerview.extensions.AsyncDifferConfig
import android.support.v7.recyclerview.extensions.ListAdapter
import android.support.v7.util.DiffUtil
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup

abstract class CustomListAdapter<T, VH : CustomViewHolder<T>> : ListAdapter<T, VH> {

    var recyclerView: RecyclerView? = null
    var currentList = mutableListOf<T>()
    var onItemClickListener: OnItemClickListener<T>? = null

    constructor(diffCallback: DiffUtil.ItemCallback<T>) : super(diffCallback)
    constructor(config: AsyncDifferConfig<T>) : super(config)

    override fun onAttachedToRecyclerView(recyclerView: RecyclerView) {
        super.onAttachedToRecyclerView(recyclerView)
        this.recyclerView = recyclerView
    }

    override fun getItemViewType(position: Int): Int {
        val item = getItem(position)
        return if (item is ItemData<*>) item.getItemViewType() else View.NO_ID
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): VH {
        val view = LayoutInflater.from(parent.context).inflate(viewType, parent, false)
        val vh = createCustomViewHolder(view, parent, viewType)
                ?: CustomViewHolder<T>(View(parent.context)) as VH
        return vh.apply {
            adapter = this@CustomListAdapter
            itemView.setOnClickListener{
                onItemClickListener?.onItemClick(itemView, this.adapterPosition, this@CustomListAdapter as CustomListAdapter<T, CustomViewHolder<T>>)
            }
            onCreate()
        }
    }

    abstract fun createCustomViewHolder(view: View?, parent: ViewGroup, viewType: Int): VH?

    override fun onBindViewHolder(holder: VH, position: Int) {
        val itemData = getItem(position)
        if (itemData is ItemData<*>) {
            holder.onBindItem(position, getItem(position))
        } else {
            bindCustomViewHolder(holder, position)
        }
    }

    open fun bindCustomViewHolder(holder: CustomViewHolder<T>, position: Int) {}

    public override fun getItem(position: Int): T {
        return super.getItem(position)
    }

    fun getCurrentItems(): List<T> {
        return currentList
    }

    fun addItem(t: T) {
        submitList(mutableListOf(t))
    }

    fun addList(list: List<T>) {
        currentList.addAll(list)
        super.submitList(currentList.toMutableList())
    }

    fun addCurrentItem(t: T) {
        currentList.add(t)
    }

    fun addCurrentList(list: List<T>) {
        currentList.addAll(list)
    }

    override fun submitList(list: MutableList<T>?) {
        currentList.clear()
        list?.let { currentList.addAll(it) }
        super.submitList(list?.toMutableList())
    }

    fun attach(recyclerView: RecyclerView): CustomListAdapter<T, VH> {
        this.recyclerView = recyclerView
        if (recyclerView.layoutManager == null) {
            recyclerView.layoutManager = LinearLayoutManager(recyclerView.context)
        }
        recyclerView.adapter = this
        return this
    }
}