package com.taobao.movie.android.common.item.homepage

import android.graphics.Color
import android.text.TextUtils
import android.util.TypedValue
import android.view.View
import com.taobao.movie.android.business.R
import com.taobao.movie.android.common.item.theme.ItemEvent
import com.taobao.movie.android.integration.oscar.model.ShowMo
import com.taobao.movie.android.integration.oscar.uiInfo.ShowAndSoonModuleV0
import com.taobao.movie.android.sdk.infrastructure.usertrack.UTFacade
import com.taobao.movie.appinfo.util.DataUtil
import com.taobao.movie.appinfo.util.ResHelper

/**
 * 正在热映和即将上映模块
 *
 * @author yidong.yyd
 * @since 2019-12-04
 */
class HomePageShowAndSoonContainerItem(showAndSoonModuleV0: ShowAndSoonModuleV0?, listener: OnItemEventListener<*>?)
    : HomepageShowContainerBaseItem<ShowAndSoonModuleV0>(showAndSoonModuleV0, listener) {
    companion object {
        /* 初始化的值 */
        const val POSITION_INIT = -1
        /* 热映影片 对应位置 */
        const val POSITION_MAIN_LEFT = 0
        /* 即将上映 对应位置 */
        const val POSITION_MAIN_RIGHT = 1
        /* 当前位置 */
        var sCurrentPosition = POSITION_INIT
    }

    /* 当前展示的位置 */
    private var currentMainPosition = POSITION_INIT
    /* 灰色 */
    private val color1001 = ResHelper.getColor(R.color.common_color_1001)

    override fun onBindViewHolder(viewHolder: ViewHolder?) {
        if (viewHolder == null) return
        currentMainPosition = POSITION_INIT
        viewHolder.initShowOrSoonContain()
        viewHolder.vsTitleCombineContainerLayout.visibility = View.VISIBLE

        viewHolder.vsLeftTitleTextView.visibility = View.VISIBLE
        viewHolder.vsTitleDividerView.visibility = View.VISIBLE
        viewHolder.vsRightTitleTextView.visibility = View.VISIBLE

        if (DataUtil.isNullOrEmpty(data.showModuleVO.showList)) {
            viewHolder.vsLeftTitleTextView.visibility = View.GONE
            viewHolder.vsTitleDividerView.visibility = View.GONE
        }
        if (DataUtil.isNullOrEmpty(data.soonShowModuleVO.showList)) {
            viewHolder.vsRightTitleTextView.visibility = View.GONE
            viewHolder.vsTitleDividerView.visibility = View.GONE
        }

        viewHolder.vsLeftTitleTextView.setOnClickListener(onLeftClickListener)
        viewHolder.vsRightTitleTextView.setOnClickListener(onRightClickListener)
        viewHolder.vsEntranceCombineLayout.setOnClickListener(onEntranceClickListener)

        viewHolder.vsLeftTitleTextView.text = if (TextUtils.isEmpty(data.showModuleVO.title)) "正在热映" else data.showModuleVO.title
        viewHolder.vsRightTitleTextView.text = if (TextUtils.isEmpty(data.soonShowModuleVO.title)) "即将上映" else data.soonShowModuleVO.title

        // show 数据不为空
        if (!DataUtil.isNullOrEmpty(data.showModuleVO.showList)
                // 初次进入
                && (sCurrentPosition == POSITION_INIT
                        // 如果是下拉刷新，且刷新前也是显示的 show
                        || sCurrentPosition == POSITION_MAIN_LEFT
                        // 如果是下拉刷新，刷新前显示的 soon show，刷新后这部分数据没了
                        || (sCurrentPosition == POSITION_MAIN_RIGHT && DataUtil.isNullOrEmpty(data.soonShowModuleVO.showList)))) {
            onTabClick(POSITION_MAIN_LEFT)
        } else if (!DataUtil.isNullOrEmpty(data.soonShowModuleVO.showList) // soon show 数据不为空
                && (sCurrentPosition == POSITION_INIT || sCurrentPosition == POSITION_MAIN_RIGHT // 初次进入 || 如果是下拉刷新，且刷新前也是显示的 soon show
                        || DataUtil.isNullOrEmpty(data.showModuleVO.showList))) { // show 的数据为空
            onTabClick(POSITION_MAIN_RIGHT)
        } else viewHolder.itemView.visibility = View.GONE

        UTFacade.setSpmTag(viewHolder.vsLeftTitleTextView, "MovieStatusTabExpose.${viewHolder.vsLeftTitleTextView.id}")
        UTFacade.setExposureTag(viewHolder.vsLeftTitleTextView, "tabname", "0")
        UTFacade.setSpmTag(viewHolder.vsRightTitleTextView, "MovieStatusTabExpose.${viewHolder.vsRightTitleTextView.id}")
        UTFacade.setExposureTag(viewHolder.vsRightTitleTextView, "tabname", "1")
    }

    /**
     * 热映影片 和 即将上映 tab的点击
     */
    private fun onTabClick(position: Int) {
        if (currentMainPosition == position || data == null) return
        currentMainPosition = position
        sCurrentPosition = position
        val vh = viewHolder
        vh.vsLeftTitleTextView.setTextColor(if (position == POSITION_MAIN_LEFT) Color.BLACK else color1001)
        vh.vsLeftTitleTextView.paint.isFakeBoldText = position == POSITION_MAIN_LEFT
        vh.vsLeftTitleTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, if (position == POSITION_MAIN_LEFT) 18F else 15F)

        vh.vsRightTitleTextView.setTextColor(if (position == POSITION_MAIN_RIGHT) Color.BLACK else color1001)
        vh.vsRightTitleTextView.paint.isFakeBoldText = position == POSITION_MAIN_RIGHT
        vh.vsRightTitleTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, if (position == POSITION_MAIN_RIGHT) 18F else 15F)

        vh.videoExpressAdapter.clearItems()
        if (position == POSITION_MAIN_LEFT) {
            val v0 = data.showModuleVO
            var needGoneTag = v0.showList?.find { !DataUtil.isNullOrEmpty(it.showTags) } == null
            v0.showList?.forEach { vh.videoExpressAdapter.addItem(HomepageShowItem(it, v0.abTest, needGoneTag, listener)) }
            if (v0.showCount > 0) vh.videoExpressAdapter.addItem(HomepageShowMoreItem(v0.showCount, listener))
        } else {
            val v0 = data.soonShowModuleVO
            v0.showList?.forEach { vh.videoExpressAdapter.addItem(HomepageSoonShowItem(it, listener, HomepageSoonShowItem.ORIGIN_HOME_PAGE)) }
            if (v0.showList?.size ?: 0 > 0) vh.videoExpressAdapter.addItem(HomepageSoonShowMoreItem(v0.showList?.size ?: 0, listener))
        }
        vh.videoExpressAdapter.notifyDataSetChanged()
        vh.rvArticleFilmExpress.scrollToPosition(0)
    }

    /* 热映影片 点击事件 */
    private val onLeftClickListener = View.OnClickListener {
        UTFacade.buttonClickWithoutPage("MovieStatusTabClick", "tabname", "0")
        onTabClick(POSITION_MAIN_LEFT)
    }

    /* 即将上映 点击事件 */
    private val onRightClickListener = View.OnClickListener {
        UTFacade.buttonClickWithoutPage("MovieStatusTabClick", "tabname", "1")
        onTabClick(POSITION_MAIN_RIGHT)
    }

    /* 全部影片 点击事件 */
    private val onEntranceClickListener = View.OnClickListener {
        UTFacade.buttonClickWithoutPage(if (currentMainPosition == POSITION_MAIN_LEFT) "NowShowingAllButtonClick" else "SoonShowAllButtonClick")
        UTFacade.buttonClickWithoutPage("MovieStatusTabAllClick", "tabname", if (currentMainPosition == POSITION_MAIN_LEFT) "0" else "1")
        onEvent(if (currentMainPosition == POSITION_MAIN_LEFT) ItemEvent.EVENT_ID_HOME_PAGE_SHOW_MORE_CLICK else ItemEvent.EVENT_ID_HOME_PAGE_SOON_SHOW_MORE_CLICK)
    }

    /**
     * 更改想看状态
     */
    fun changeFavor(id: String?, favorCount: Int?, userShowStatus: Int?) {
        if (id == null || TextUtils.isEmpty(id) || getViewHolder() == null || userShowStatus == null
                || data == null || adapter == null || viewHolder.videoExpressAdapter == null) {
            return
        }
        updateAdapter(id, favorCount, userShowStatus)
        updateData(data.showModuleVO.showList, id, favorCount, userShowStatus)
        updateData(data.soonShowModuleVO.showList, id, favorCount, userShowStatus)
    }

    /**
     * 更新 adapter
     */
    private fun updateAdapter(id: String, favorCount: Int?, userShowStatus: Int) {
        val vh = getViewHolder() ?: return
        val count = vh.videoExpressAdapter?.itemCount ?: return
        (0..count).mapNotNull { vh.videoExpressAdapter?.getItem(it) }
                .filter { isSameId(it, id) }
                .filter { !((getUserShowStatus(it) == userShowStatus) && favorCount == null) }
                .forEach { changeUI(it, favorCount, userShowStatus) }
    }

    /**
     * 更改 UI
     */
    private fun changeUI(item: Any, favorCount: Int?, userShowStatus: Int) {
        val showMo = getShowMo(item) ?: return
        val oldShowStatus = showMo.getUserShowStatus()
        showMo.userShowStatus = userShowStatus
        if (favorCount != null && favorCount > 0) showMo.wantCount = favorCount
        else if (oldShowStatus != ShowMo.USER_SHOW_STATUS_WATCHED) { //从看过，到想看或者已想看，想看数不能变化
            if (userShowStatus == ShowMo.USER_SHOW_STATUS_WANT) showMo.wantCount++
            else if (userShowStatus == ShowMo.USER_SHOW_STATUS_DEFAULT) showMo.wantCount--
        }
        showMo.scoreAndFavor?.favorCount = showMo.wantCount
        refreshFavor(item)
    }

    /**
     * 获取 UserShowStatus
     */
    private fun getUserShowStatus(item: Any): Int? = when (item) {
        is HomepageShowItem -> item.data.getUserShowStatus()
        is HomepageSoonShowItem -> item.data.getUserShowStatus()
        else -> null
    }

    /**
     * 判断 id 是否相同
     */
    private fun isSameId(item: Any, id: String): Boolean = when (item) {
        is HomepageShowItem -> id == item.data.id
        is HomepageSoonShowItem -> id == item.showId
        else -> false
    }

    /**
     * 从 HomepageShowItem 或者 HomepageSoonShowItem 中获取 ShowMo
     */
    private fun getShowMo(item: Any): ShowMo? = when (item) {
        is HomepageShowItem -> item.data
        is HomepageSoonShowItem -> item.data
        else -> null
    }

    /**
     * 刷新 HomepageShowItem 或者 HomepageSoonShowItem
     */
    private fun refreshFavor(item: Any) = when (item) {
        is HomepageShowItem -> item.refreshItem()
        is HomepageSoonShowItem -> item.refreshFavor()
        else -> {
        }
    }

    /**
     * 更新数据
     */
    private fun updateData(showMoList: List<ShowMo>?, id: String, favorCount: Int?, userShowStatus: Int) {
        showMoList?.filter { it.id == id }
                ?.forEach {
                    it.userShowStatus = userShowStatus
                    it.wantCount = favorCount ?: it.wantCount
                }
    }
}
