package com.taobao.movie.android.commonui.behavior

import android.app.Activity
import android.content.Context
import android.graphics.Rect
import android.support.design.widget.CoordinatorLayout
import android.support.v4.view.GravityCompat
import android.support.v4.view.ViewCompat
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import com.taobao.movie.android.commonui.widget.Appbar
import com.taobao.movie.android.component.R
import com.taobao.movie.android.sdk.infrastructure.utils.ContextUtil

class UnderAppbarBehavior<V : View> : CoordinatorLayout.Behavior<V> {
    private val mTempRect = Rect()
    private val mTempRect2 = Rect()
    private var appbarId = View.NO_ID
    private var parent: CoordinatorLayout? = null
    private var appbar: View? = null

    constructor() {}

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {

        val a = context.obtainStyledAttributes(attrs, R.styleable.UnderAppbarBehavior)
        appbarId = a.getResourceId(R.styleable.UnderAppbarBehavior_appbarId, View.NO_ID)
        a.recycle()
    }

    override fun layoutDependsOn(parent: CoordinatorLayout, child: V, dependency: View): Boolean {
        if (this.parent == null) {
            this.parent = parent
        }
        return dependency is Appbar
    }

    override fun onMeasureChild(parent: CoordinatorLayout, child: V, parentWidthMeasureSpec: Int, widthUsed: Int,
                                parentHeightMeasureSpec: Int, heightUsed: Int): Boolean {
        val childLpHeight = child.layoutParams.height
        if (childLpHeight == ViewGroup.LayoutParams.MATCH_PARENT) {
            // If the menu's height is set to match_parent/wrap_content then measure it
            // with the maximum visible height

            val dependencies = parent.getDependencies(child)
            val header = findAppbar(dependencies)
            if (header != null) {
                if (ViewCompat.getFitsSystemWindows(header) && !ViewCompat.getFitsSystemWindows(child)) {
                    // If the header is fitting system windows then we need to also,
                    // otherwise we'll get CoL's compatible measuring
                    ViewCompat.setFitsSystemWindows(child, true)

                    if (ViewCompat.getFitsSystemWindows(child)) {
                        // If the set succeeded, trigger a new layout and return true
                        child.requestLayout()
                        return true
                    }
                }

                val topRange = header.measuredHeight

                var availableHeight = View.MeasureSpec.getSize(parentHeightMeasureSpec)
                if (availableHeight == 0) {
                    // If the measure spec doesn't specify a size, use the current height
                    availableHeight = parent.height
                }

                val height = availableHeight - topRange
                val heightMeasureSpec = View.MeasureSpec.makeMeasureSpec(height, View.MeasureSpec.EXACTLY)

                // Now measure the scrolling view with the correct height
                parent.onMeasureChild(child, parentWidthMeasureSpec, widthUsed, heightMeasureSpec, heightUsed)

                return true
            }
        }
        return false
    }

    override fun onLayoutChild(parent: CoordinatorLayout, child: V, layoutDirection: Int): Boolean {
        val dependencies = parent.getDependencies(child)
        val header = findAppbar(dependencies)
        if (header != null) {
            val lp = child.layoutParams as CoordinatorLayout.LayoutParams

            val available = mTempRect
            val left = parent.paddingLeft + lp.leftMargin
            val right = parent.width - parent.paddingRight - lp.rightMargin
            val top = header.bottom + lp.topMargin
            val bottom: Int
            bottom = if (lp.height == ViewGroup.LayoutParams.MATCH_PARENT) {
                parent.height - parent.paddingBottom - lp.bottomMargin
            } else {
                top + child.height
            }
            available.set(left, top, right, bottom)

            val out = mTempRect2
            GravityCompat.apply(resolveGravity(lp.gravity), child.measuredWidth, child.measuredHeight, available, out,
                    layoutDirection)
            child.layout(out.left, out.top, out.right, out.bottom)
        } else {
            // If we don't have a dependency, let super handle it
            parent.onLayoutChild(child, layoutDirection)
        }
        return true
    }

    private fun resolveGravity(gravity: Int): Int {
        return if (gravity == Gravity.NO_GRAVITY) GravityCompat.START or Gravity.TOP else gravity
    }

    private fun findFirstDependency(views: List<View>?): Appbar? {
        if (views != null) {
            for (view in views) {
                if (view is Appbar) {
                    return view
                }
            }
        }
        return null
    }

    private fun findAppbar(views: List<View>): View? {
        if (appbar != null) {
            return appbar
        }
        appbar = findFirstDependency(views)
        if (appbar != null) {
            return appbar
        }
        if (appbarId != View.NO_ID && parent != null) {
            val activity = ContextUtil.getActivity<Activity>(parent!!)
            if (activity != null) {
                appbar = activity.findViewById(appbarId)
            }
        }
        return appbar
    }
}
