package com.taobao.movie.android.commonui.behavior

import android.app.Activity
import android.content.Context
import android.support.design.widget.CoordinatorLayout
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import com.taobao.movie.android.commonui.widget.Appbar
import com.taobao.movie.android.commonui.widget.MaterialTabLayout
import com.taobao.movie.android.component.R
import com.taobao.movie.android.sdk.infrastructure.utils.ContextUtil

/**
 * 高度为指定两个view的高度之和
 */
class ViewHeightBehavior<V : View> : CoordinatorLayout.Behavior<V> {

    private var firstViewId = View.NO_ID
    private var secondViewId = View.NO_ID
    private var firstView: View? = null
    private var secondView: View? = null
    private var parent: CoordinatorLayout? = null

    constructor() {}

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {

        val a = context.obtainStyledAttributes(attrs, R.styleable.ViewHeightBehavior)
        firstViewId = a.getResourceId(R.styleable.ViewHeightBehavior_firstViewId, View.NO_ID)
        secondViewId = a.getResourceId(R.styleable.ViewHeightBehavior_secondViewId, View.NO_ID)
        a.recycle()
    }

    override fun layoutDependsOn(parent: CoordinatorLayout, child: V, dependency: View): Boolean {
        if (this.parent == null) {
            this.parent = parent
        }
        if (firstView == null) {
            firstView = findViewId(firstViewId)
        }
        if (secondView == null) {
            secondView = findViewId(secondViewId)
        }
        return dependency is MaterialTabLayout || dependency is Appbar
    }

    override fun onMeasureChild(parent: CoordinatorLayout, child: V, parentWidthMeasureSpec: Int, widthUsed: Int,
                                parentHeightMeasureSpec: Int, heightUsed: Int): Boolean {
        val childLpHeight = child.layoutParams.height
        if (childLpHeight == ViewGroup.LayoutParams.MATCH_PARENT || childLpHeight == ViewGroup.LayoutParams.WRAP_CONTENT) {
            if (firstView == null) {
                firstView = findViewId(firstViewId)
            }
            if (secondView == null) {
                secondView = findViewId(secondViewId)
            }
            val firstHeight = firstView?.measuredHeight ?: 0
            val secondHeight = secondView?.measuredHeight ?: 0

            val height = firstHeight + secondHeight

            val heightMeasureSpec = View.MeasureSpec.makeMeasureSpec(height, View.MeasureSpec.EXACTLY)

            parent.onMeasureChild(child, parentWidthMeasureSpec, widthUsed, heightMeasureSpec, heightUsed)

            return true
        }
        return false
    }

    private fun findViewId(viewId: Int): View? {
        var view = parent?.findViewById<View>(viewId)
        if (view == null && parent != null) {
            view = ContextUtil.getActivity<Activity>(parent!!)?.findViewById(viewId)
        }
        return view
    }
}