package com.taobao.movie.android.commonui.utils

import android.app.Activity
import android.app.Application
import android.os.Bundle
import android.text.TextUtils
import com.taobao.movie.android.commonui.component.BaseActivity
import com.taobao.movie.android.sdk.infrastructure.utils.GlobalAppUtil
import com.taobao.orange.OrangeConfig

/**
 * fps 关联生命周期
 *
 * @author yidong.yyd
 * @since 2019-11-26
 */
class FpsRelation private constructor() : Application.ActivityLifecycleCallbacks {
    /* Activity 与 FpsApmHelper 映射表 */
    private val map: HashMap<Activity, FpsApmHelper>
    /* 排除列表 */
    private val excludeList: ArrayList<String>

    private val permissionMap: HashMap<String, Boolean>

    companion object {
        /* 单例 */
        val ready = Holder.INSTANCE
    }

    private object Holder {
        /* 单例 */
        val INSTANCE = FpsRelation()
    }

    init {
        GlobalAppUtil.getApplication().registerActivityLifecycleCallbacks(this)
        map = HashMap()
        permissionMap = HashMap()
        excludeList = ArrayList()
        excludeList.add("com.taobao.movie.android.app.home.activity.MainActivity")
    }

    override fun onActivityPaused(activity: Activity?) {
        map[activity]?.onPause()
    }

    override fun onActivityResumed(activity: Activity?) {
        if (activity is BaseActivity && !map.containsKey(activity) && !excludeList.contains(activity.javaClass.name)
                && permissionMap[activity.javaClass.name] == true) map[activity] = FpsApmHelper(activity)
        map[activity]?.onResume()
    }

    override fun onActivityDestroyed(activity: Activity?) {
        map[activity]?.onDestroy()
        map.remove(activity)
    }

    override fun onActivityCreated(activity: Activity?, savedInstanceState: Bundle?) {
        if (activity !is BaseActivity) return
        val name = activity.javaClass.name
        val key = "fps_opt_" + activity.utPageName
        if (!permissionMap.containsKey(name)) {
            var ret: String? = null
            val value = OrangeConfig.getInstance().getConfig("android_movie_config", key, "none")
            if (!TextUtils.equals("none", value)) ret = value
            val isOpen = TextUtils.equals("true", ret)
            permissionMap[name] = isOpen
        }
        if (permissionMap[name] == true) activity.onUTButtonClick(key)
    }

    override fun onActivityStarted(activity: Activity?) {
    }

    override fun onActivitySaveInstanceState(activity: Activity?, outState: Bundle?) {
    }

    override fun onActivityStopped(activity: Activity?) {
    }
}