@file:JvmName("ExtensionsKt")

package com.taobao.movie.android.commonutil.kotlin

import android.graphics.Color
import android.view.View
import com.taobao.movie.android.component.BuildConfig

/**
 * 将String解析成颜色
 */
fun String.parseRGB() = Color.parseColor(if (this.startsWith("#")) this else "#".plus(this))

/**
 * Boolean有空判断下的判断逻辑
 */
fun Boolean?.isTrue() = when (this) {
    true -> true
    else -> false
}

/**
 * [View]是否可见扩展方法
 */
fun View.isVisible() = visibility == View.VISIBLE

fun View.optVisible() = takeIf { isVisible() }

/**
 * Int有空判断下的逻辑
 */
@JvmOverloads
fun Int?.toNumber(defaultValue: Int = 0) = this ?: defaultValue

/**
 * Long有空判断下的逻辑
 */
@JvmOverloads
fun Long?.toNumber(defaultValue: Long = 0L) = this ?: defaultValue

/**
 * Float有空判断下的逻辑
 */
@JvmOverloads
fun Float?.toNumber(defaultValue: Float = 0F) = this ?: defaultValue

/**
 * String to int 安全转换
 * @param defaultValue 默认值，没有给的话就是0
 */
@JvmOverloads
fun String?.parseInt(defaultValue: Int = 0) = try {
    this?.toInt() ?: defaultValue
} catch (e: Exception) {
    e.print()
    defaultValue
}

/**
 * Exception log 帮助，
 * 主要是想将 exception 的日志在 debug 环境下打印出来方便错误查询
 * throwable.print()
 */
fun Throwable.print() {
    if (BuildConfig.DEBUG) {
        this.printStackTrace()
    }
}